/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: PCircAdaptDefines.cpp 1018 2014-01-21 11:08:27Z eelco $
 *
 */

#include <QFont>
#include <QObject>
#include "PCircAdaptDefines.h"

//#include "PGLColor.h"

using namespace Peacs::CircAdapt;
using namespace Peacs::Support::Widget;


const char * Peacs::CircAdapt::CASE_FILE_EXT                    = ".CircAdaptcase";

const double Peacs::CircAdapt::PASCAL2MMHG                      = 0.00750061683;
const double Peacs::CircAdapt::MMHG2PASCAL                      = 133.322368;

const char *     Peacs::CircAdapt::SYSTEMIC_R                   = "systemicR";
const char *     Peacs::CircAdapt::PULMONARY_R                  = "pulmonaryR";

// Make sure that the names of the signal types are in the same sequence as the types defined
// in the 'signal_type' enumeration
const QString   Peacs::CircAdapt::signal_type_str[]             = { QObject::tr( "Pressure" ),
                                                                    QObject::tr( "Flow" ),
                                                                    QObject::tr( "Volume" ),
                                                                    QObject::tr( "Area" ),
                                                                    QObject::tr( "Flow velocity" ) };

// Make sure that the names of the signal sub-types are in the same sequence as the types
// defined in the 'sunSignal_type' enumeration
const QString   Peacs::CircAdapt::subSignal_type_str[]          = { QObject::tr( "CURRENT" ),
                                                                    QObject::tr( "REFERENCE" ),
                                                                    QObject::tr( "SNAPSHOT" ) };


const char*      Peacs::CircAdapt::VOLUME_UNITS                   = "ml";
const char*      Peacs::CircAdapt::PRESSURE_UNITS                 = "mmHg";
const char*      Peacs::CircAdapt::DVOLUME_DT_UNITS               = "???";
const char*      Peacs::CircAdapt::AREA_UNITS                     = "cm2";
const char*      Peacs::CircAdapt::IMPEDANCE_UNITS                = "mmHg";
const char*      Peacs::CircAdapt::FLOW_UNITS                     = "ml/min";
const char*      Peacs::CircAdapt::FLOWVELOCITY_UNITS             = "m/s";
const char*      Peacs::CircAdapt::DFLOW_UNITS                    = "mmHg";
const char*      Peacs::CircAdapt::LENGTH_UNITS                   = "mmHg";
const char*      Peacs::CircAdapt::DLENGTH_UNITS                  = "mmHg";
const char*      Peacs::CircAdapt::CONTRACTION_UNITS              = "mmHg";
const char*      Peacs::CircAdapt::DCONTRACTION_UNITS             = "mmHg";
const char*      Peacs::CircAdapt::STRAIN_UNITS                   = "mmHg";
const char*      Peacs::CircAdapt::STRESS_UNITS                   = "mmHg";
const char*      Peacs::CircAdapt::STIFFNESS_UNITS                = "mmHg";
const char*      Peacs::CircAdapt::CURVATIRE_UNITS                = "mmHg";
const char*      Peacs::CircAdapt::TENSION_UNITS                  = "mmHg";
const char*      Peacs::CircAdapt::DTENSION_UNITS                 = "mmHg";

/* colors definitions */
const QColor Peacs::CircAdapt::HEART_COLOR              = QColor(233,150,122);
const QColor Peacs::CircAdapt::ATRIA_COLOR              = QColor(255, 165, 0);
const QColor Peacs::CircAdapt::RIGHTATRIA_COLOR         = QColor(255,69,0);
const QColor Peacs::CircAdapt::LEFTATRIA_COLOR          = QColor(30,144,255);
const QColor Peacs::CircAdapt::RIGHTVENTRICLE           = QColor(0,0,255);
const QColor Peacs::CircAdapt::LEFTVENTRICLE_COLOR      = QColor(255,0,0);
const QColor Peacs::CircAdapt::VENTRICLE_COLOR          = QColor(165,42, 42);
const QColor Peacs::CircAdapt::AORTA_COLOR              = QColor(176, 48, 96);
const QColor Peacs::CircAdapt::PULMONARYVEINS_COLOR     = QColor(255,140,0);
const QColor Peacs::CircAdapt::PULMONARYARTERY_COLOR    = QColor(25,25,237);
const QColor Peacs::CircAdapt::SYSTEMICVEINS_COLOR      = QColor(106,90,205);
const QColor Peacs::CircAdapt::LVEN_COLOR               = QColor( 127, 255, 212 );
const QColor Peacs::CircAdapt::LAV_COLOR                = QColor(   0, 100,   0 );
const QColor Peacs::CircAdapt::LART_COLOR               = QColor( 128, 128,   0 );
const QColor Peacs::CircAdapt::RVEN_COLOR               = QColor(  32, 178, 170 );
const QColor Peacs::CircAdapt::RAV_COLOR                = QColor(   0,   0, 170 );      // QColor(14,139,34);
const QColor Peacs::CircAdapt::RART_COLOR               = QColor(  27, 140, 143 );    // QColor(148,0,211);
const QColor Peacs::CircAdapt::ASD_COLOR                = QColor( 255, 255,   0 );
const QColor Peacs::CircAdapt::VSD_COLOR                = QColor( 255, 215,   0 );
const QColor Peacs::CircAdapt::DUCT_COLOR               = QColor( 218, 165,  32 );
const QColor Peacs::CircAdapt::LOGO_TEXT_COLOR          = QColor(  90,  90,  90 );
const QColor Peacs::CircAdapt::LOGO_HEART_COLOR         = QColor( 227,   6,  19 );

/* CircAdaptMainWindow settings */
const char*         Peacs::CircAdapt::CIRCADAPT_MAINPOS             = "CircAdaptMainGroup/pos";
const char*         Peacs::CircAdapt::CIRCADAPT_MAINSIZE            = "CircAdaptMainGroup/size";
const char*         Peacs::CircAdapt::CIRCADAPT_MAINSTATE           = "CircAdaptMainGroup/state";
const char*         Peacs::CircAdapt::CIRCADAPT_MAINSPLITERSTATE    = "CircAdaptMainGroup/mainsplitterstate";
const char*         Peacs::CircAdapt::CIRCADAPT_TOPSPLITERSTATE     = "CircAdaptMainGroup/topsplitterstate";
const char*         Peacs::CircAdapt::CIRCADAPT_BOTTOMSPLITERSTATE  = "CircAdaptMainGroup/bottomsplitterstate";
const char*         Peacs::CircAdapt::CIRCADAPT_CURDIR              = "CircAdaptMainGroup/curdirectory";
const char*         Peacs::CircAdapt::CIRCADAPT_LASTFILE            = "CircAdaptMainGroup/lastfile";
const char*         Peacs::CircAdapt::CIRCADAPT_OPENLASTFILE        = "CircAdaptMainGroup/OpenLastFile";
const char*         Peacs::CircAdapt::CIRCADAPT_RECENTFILES         = "CircAdaptMainGroup/ResentFilesListed";
const char*         Peacs::CircAdapt::CIRCADAPT_UNDO_STEPS          = "CircAdaptMainGroup/NumberOfUndoSteps";

/* CircAdapt PValueView settings */
const char*         Peacs::CircAdapt::CIRCADAPT_VALUEVIEW_PART      = "CircAdaptValueView/partition";

// Settings identifiers and default values
// Grid settings
const char*         Peacs::CircAdapt::GRIDTYPE                      = "Signals/GridType";
const Grid_type     Peacs::CircAdapt::GRIDTYPE_DEFAULT              = SHOW_GRID;
// Signal view settings
const char*         Peacs::CircAdapt::SIGNAL_FONT_LABEL             = "Signals/font-label";
const char*         Peacs::CircAdapt::SIGNAL_FONT_SCALE             = "Signals/font-scale";
const char*         Peacs::CircAdapt::SIGNAL_FONT_TEXT              = "Signals/font-text";
const char*         Peacs::CircAdapt::SIGNAL_FONT_TEXT_DEFAULT      = "Calibri,18,-1,5,63,0,0,0,0,0";   // QFont( "Calibri", -1, QFont::DemiBold )
const char*         Peacs::CircAdapt::SIGNAL_FONT_LABEL_DEFAULT     = "Calibri,18,-1,5,63,0,0,0,0,0";   // QFont( "Calibri", -1, QFont::DemiBold )
const char*         Peacs::CircAdapt::SIGNAL_FONT_SCALE_DEFAULT     = "Calibri,14,-1,5,63,0,0,0,0,0";   // QFont( "Calibri", -1, QFont::DemiBold )
const char*         Peacs::CircAdapt::SIGNAL_COLOR_BG               = "Signals/color-bg";
const char*         Peacs::CircAdapt::SIGNAL_COLOR_BG_GRAPH             = "Signals/color-bg-init";

// Parameter tabs settings
const char*         Peacs::CircAdapt::PARAMETER_TABS_SELECTED        = "Parameters/DefaultTabSelected";
const char*         Peacs::CircAdapt::PARAMETER_TABS_GEN             = "Parameters/generalTabVisible";
const char*         Peacs::CircAdapt::PARAMETER_TABS_VENTR           = "Parameters/ventriclesTabVisible";
const char*         Peacs::CircAdapt::PARAMETER_TABS_ATRIA           = "Parameters/atriaTabVisible";
const char*         Peacs::CircAdapt::PARAMETER_TABS_VALVES          = "Parameters/valvesTabVisible";
const char*         Peacs::CircAdapt::PARAMETER_TABS_VASC            = "Parameters/vasculatureTabVisible";
const char*         Peacs::CircAdapt::PARAMETER_TABS_SHUNT           = "Parameters/shuntTabVisible";
const char*         Peacs::CircAdapt::PARAMETER_TABS_ADAPT           = "Parameters/adaptationTabVisible";

const QColor Peacs::CircAdapt::DEFAULTBACKGROUNDCOLOR(   186,    180,    194 );
const QColor Peacs::CircAdapt::DEFAULTBACKGROUNDCOLORGRAPH(  195,    189,    202 );
//const QColor Peacs::CircAdapt::DEFAULTBACKGROUNDCOLOR(   255,    225,    225 );         // Voor Tammo
//const QColor Peacs::CircAdapt::DEFAULTBACKGROUNDCOLORGRAPH(  255,    235,    235 );     // Voor Tammo
const QColor Peacs::CircAdapt::TIMELINECOLOR(           0xFF,   0xDD,   0x00 );
const QColor Peacs::CircAdapt::TIMELINECOLORSAVED(      0x5F,   0x5F,   0x5F ); //255,    150,     25 );
const QColor Peacs::CircAdapt::TIMELINECOLORINIT(       0x00,   0x00,   0x00 ); //25,    255,     25 );
