/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: PCircAdaptDefines.h 1018 2014-01-21 11:08:27Z eelco $
 *
 */

#ifndef __PCIRCADAPTDEFINES_H__
#define __PCIRCADAPTDEFINES_H__

#include <QColor>
#include <QFont>
#include <QPen>

#include "PWidgetDefines.h"


namespace Peacs
{
    namespace CircAdapt
    {
        extern const char *     CASE_FILE_EXT;


        // types

        /** @brief the type of grid as used in the ECGview and apview*/
//        enum Grid_type
//        {
//            NO_GRID         = 0,
//            SHOW_GRID       = 1,
//            SHOW_FULL_GRID  = 2
//        };

        enum chamber_type
        {
            UNDEFINED_TYPE  = 0,
            RIGHT_ATRIA     = 1,
            LEFT_ATRIA      = 2,
            RIGHT_VENTRICLE = 3,
            SEPTUM          = 4,
            LEFT_VENTRICLE  = 5
        };

        enum ventricle_type
        {
            BISEG           = 2,
            TRISEG          = 3
        };

        enum signal_type
        {
            PRESSURE        = 0,    //1,
            FLOW            = 1,    //2,
            VOLUME          = 2,    //3,
            AREA            = 3,    //4,
            FLOWVELOCITY    = 4     //5
        };

        /** @brief This define contains the strings that correspond to the signal types
          * The sequence of the strings defines in this array must follow the same sequence
          * of the defined types in the enumeration signal_type
          */
        extern const QString signal_type_str[];

        enum subSignal_type
        {
            CURRENT_PLOT    = 0,
            INITIAL_PLOT    = 1,
            SAVED_PLOT      = 2
        };

        /** @brief This define contains the strings that correspond to the signal sub-types
          * The sequence of the strings defines in this array must follow the same sequence
          * of the defined types in the enumeration sunSignal_type
          */
        extern const QString subSignal_type_str[];

        enum pvloop_type
        {
            PV_RIGHT_ATRIA      = 1,
            PV_LEFT_ATRIA       = 2,
            PV_RIGHT_VENTRICLES = 3,
            PV_LEFT_VENTRICLES  = 4
        };

        extern const char *     SYSTEMIC_R;
        extern const char *     PULMONARY_R;

        // constants
        const int       NOTUSED                     = -1;
        const int       DEFAULT_SIGNAL_LENGTH       = 10000;
        const double    AVDELAY_RELATIVEFROMRATE    = 0.1765;

        enum AdaptStatusType { TransitionAdaptation, RestAdaptation,ExerciseAdapation,NOADAPTATION};


        //----------------------------------------------------------------------------
        extern const double     PASCAL2MMHG;
        extern const double     MMHG2PASCAL;

        extern const char*      VOLUME_UNITS;
        extern const char*      DVOLUME_DT_UNITS;
        extern const char*      PRESSURE_UNITS;
        extern const char*      AREA_UNITS;
        extern const char*      IMPEDANCE_UNITS;
        extern const char*      FLOW_UNITS;
        extern const char*      DFLOW_UNITS;
        extern const char*      FLOWVELOCITY_UNITS;
        extern const char*      LENGTH_UNITS;
        extern const char*      DLENGTH_UNITS;
        extern const char*      CONTRACTION_UNITS;
        extern const char*      DCONTRACTION_UNITS;
        extern const char*      STRAIN_UNITS;
        extern const char*      STRESS_UNITS;
        extern const char*      STIFFNESS_UNITS;
        extern const char*      CURVATIRE_UNITS;
        extern const char*      TENSION_UNITS;
        extern const char*      DTENSION_UNITS;

        /* CircAdaptMainWindow settings keywords */
        extern const char*      CIRCADAPT_MAINPOS;             // for storing the mainwindow postion
        extern const char*      CIRCADAPT_MAINSIZE;            // for storing the mainwindow  size
        extern const char*      CIRCADAPT_MAINSTATE;           // for storing the mainwindow state
        extern const char*      CIRCADAPT_MAINSPLITERSTATE;    // for storing the main splitter state
        extern const char*      CIRCADAPT_TOPSPLITERSTATE;     // for storing the main top splitter state
        extern const char*      CIRCADAPT_BOTTOMSPLITERSTATE;  // for storing the main bottom splitter state
        extern const char*      CIRCADAPT_CURDIR;              // current directory
        extern const char*      CIRCADAPT_LASTFILE;            // last loaded File
        extern const char*      CIRCADAPT_OPENLASTFILE;        // Flag that indicates if the last opened file must be loaded during startup
        extern const char*      CIRCADAPT_RECENTFILES;         // The number of files shown in the recent file list
        extern const char*      CIRCADAPT_UNDO_STEPS;          // The number undo steps stored in the stack

        /* CircAdapt PValueView settings */
        extern const char*      CIRCADAPT_VALUEVIEW_PART;       // Holds the partition of the PValueView list

        /* Settings are stored in the settings file
         * For each option the keyword is given and the default value */
        extern const char*                          GRIDTYPE;
        extern const Support::Widget::Grid_type     GRIDTYPE_DEFAULT;
        extern const char*                          SIGNAL_FONT_LABEL;
        extern const char*                          SIGNAL_FONT_SCALE;
        extern const char*                          SIGNAL_FONT_TEXT;
        extern const char*                          SIGNAL_FONT_TEXT_DEFAULT;
        extern const char*                          SIGNAL_FONT_LABEL_DEFAULT;
        extern const char*                          SIGNAL_FONT_SCALE_DEFAULT;
        extern const char*                          SIGNAL_COLOR_BG;
        extern const char*                          SIGNAL_COLOR_BG_GRAPH;
        extern const char*                          PARAMETER_TABS_SELECTED;
        extern const char*                          PARAMETER_TABS_GEN;
        extern const char*                          PARAMETER_TABS_VENTR;
        extern const char*                          PARAMETER_TABS_ATRIA;
        extern const char*                          PARAMETER_TABS_VALVES;
        extern const char*                          PARAMETER_TABS_VASC;
        extern const char*                          PARAMETER_TABS_SHUNT;
        extern const char*                          PARAMETER_TABS_ADAPT;

        /* colors definitions */
        extern const QColor HEART_COLOR;
        extern const QColor ATRIA_COLOR;
        extern const QColor RIGHTATRIA_COLOR;
        extern const QColor LEFTATRIA_COLOR;
        extern const QColor RIGHTVENTRICLE;
        extern const QColor LEFTVENTRICLE_COLOR;
        extern const QColor VENTRICLE_COLOR;
        extern const QColor AORTA_COLOR;
        extern const QColor PULMONARYVEINS_COLOR;
        extern const QColor PULMONARYARTERY_COLOR;
        extern const QColor SYSTEMICVEINS_COLOR;
        extern const QColor LVEN_COLOR;
        extern const QColor LAV_COLOR;
        extern const QColor LART_COLOR;
        extern const QColor RVEN_COLOR;
        extern const QColor RAV_COLOR;
        extern const QColor RART_COLOR;
        extern const QColor ASD_COLOR;
        extern const QColor VSD_COLOR;
        extern const QColor DUCT_COLOR;
        extern const QColor LOGO_TEXT_COLOR;
        extern const QColor LOGO_HEART_COLOR;
        extern const QColor DEFAULTBACKGROUNDCOLOR;
        extern const QColor DEFAULTBACKGROUNDCOLORGRAPH;
        extern const QColor TIMELINECOLOR;                  // current time line color
        extern const QColor TIMELINECOLORSAVED;             // Snapshot (saved) time line color
        extern const QColor TIMELINECOLORINIT;              // Initial time line color
    }
}

#endif /* __PCIRCADAPTDEFINES_H__ */
