/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012-2014 StITPro (www.stitpro.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: PCircAdaptMainwindow.cpp 1174 2014-03-21 14:15:01Z eelco $
 *
 */

#include <QDir>
#include <QStringList>
#include <QMessageBox>
#include <QFileDialog>
#include <QPrintDialog>
#include <QPrinter>         //QtPrintSupport/
#include <QCloseEvent>
#include <QSplitter>
#include <QApplication>
#include <QMenu>
#include <QFileInfo>
#include <QList>
#include <QBitmap>
#ifdef QT_NO_DEBUG  // Only in release versions
    #include <QElapsedTimer>
#endif

//#include <QDebug>

#include "PCircAdaptMainwindow.h"
#include "pCircAdaptAboutdialog.h"

#include "pcircadapt.h"
#include "PWidgetDefines.h"


using namespace Peacs::Support;
using namespace Peacs::CircAdapt;


const int MAX_UNDO_STEPS_DEFAULT        = 60;
const int RECENT_FILES_DEFAULT          = 5;        // The default number of recent files to show in the menu
const int STATUSBAR_MESSAGE_TIME        = 20000;    // Milli seconds


PCircAdaptMainwindow::PCircAdaptMainwindow( /*const QString &filename,*/ QWidget * parent ) :
    QMainWindow(parent),
    mSplashScreenPtr( NULL ),
    mData(),
    mUi(),
    mCirculation( /*filename*/ ),
    mDataDir(),
    mCurrentDir(),
    mDirty(false),
    mWinSigMapper( NULL ),
    mTimerId( -1 ),
    mWidgetInfo(),
//        mUndoStack( NULL ),
#ifndef QT_NO_DEBUG
//        mUndoView( NULL ),
#endif
    //mStatusLabel( NULL ),
    mMaxRecentFiles( RECENT_FILES_DEFAULT ),
    mRecentFileActs(),
    mRecentFileSeparator(),
    mMaxUndoSteps( MAX_UNDO_STEPS_DEFAULT ),
    mInitialScale( false ),
    mPVView( NULL ),
    mSignalView( NULL ),
    mParamView(  NULL ),
//    mSystemView( NULL ),
    mValuesView( NULL )
{
}


PCircAdaptMainwindow::~PCircAdaptMainwindow()
{
    mCirculation.abort();   // Signal the thread to terminate

    mCirculation.wait( 2000 );   // Wait for the termination of the thread for 2 sec.
}


void PCircAdaptMainwindow::setupUi( void )
{
    if ( !mPVView )
    {
        QPixmap         pixmap( ":/CircAdapt/icons/CircAdaptSplash.png" );
#ifdef QT_NO_DEBUG  // Only in release versions
        QElapsedTimer   splashTimer;
#endif

    //    if ( pixmap.width() > 768 )
    //    {
    //        // This will probably fitt on all screen types
    //        pixmap = pixmap.scaledToWidth( 768, Qt::SmoothTransformation );
    //    }

        mSplashScreenPtr = new QSplashScreen( pixmap, Qt::WindowStaysOnTopHint );
        mSplashScreenPtr->setMask( pixmap.mask() ); // Make sure that transparent part are transparent

        mSplashScreenPtr->show();

        QApplication::processEvents();

        mSplashScreenPtr->setFont( QFont( Widget::FONTFAMILY, 22, QFont::DemiBold ) ) ;
        mSplashScreenPtr->showMessage( tr("Initializing CircAdapt..."), Qt::AlignTop | Qt::AlignHCenter, LOGO_TEXT_COLOR );

        mUi.setupUi( this );

        // For now, hide all help related menu items
        mUi.actionContext_Help_F1->setVisible( false );
        mUi.actionIndex->setVisible( false );

        QSplitter* topSplitter      = new QSplitter( Qt::Horizontal, this );
        QSplitter* bottomSplitter   = new QSplitter( Qt::Horizontal, this );
        QSplitter* mainSplitter     = new QSplitter( Qt::Vertical, this );

        topSplitter->setOpaqueResize( false );
        topSplitter->setChildrenCollapsible( false );
        bottomSplitter->setOpaqueResize( false );
        bottomSplitter->setChildrenCollapsible( false );
        mainSplitter->setOpaqueResize( false );
        mainSplitter->setChildrenCollapsible( false );

        mainSplitter->addWidget( topSplitter );
        mainSplitter->addWidget( bottomSplitter );

        setCentralWidget( mainSplitter );

        // The signal mapper can connect and bundle signals and add defined data to the signal, like
        // connect a clicked action to a widget
        mWinSigMapper = new QSignalMapper( this );
        connect( mWinSigMapper, SIGNAL(mapped(QWidget*)), this, SLOT(toggleWindowVisibleState(QWidget*)) );

        mSignalView =  new PSignalView( mData, topSplitter );
        topSplitter->addWidget( mSignalView );

        mPVView =  new PPVView( mData, topSplitter );
        topSplitter->addWidget( mPVView );

        mValuesView = new PValueView( mData, bottomSplitter );
        bottomSplitter->addWidget( mValuesView );

        mParamView  =  new PParamView( mData, mCirculation, bottomSplitter );
        bottomSplitter->addWidget( mParamView );

    //    mUndoStack = new PEcgSimUndoStack( this );    // Create the Qt undo/redo stack to handle these commands

        setObjectName( "CircAdapt" );
        setWindowTitle( "CircAdapt" );

        statusBar();

    //    mStatusLabel = new QLabel( this );
    //    statusBar()->addPermanentWidget(mStatusLabel);

        QApplication::processEvents();

        // build menus and toolbars
        buildMenu();

        QApplication::processEvents();

        // read the settings after all members have been created
        readSettings();

    //    mUndoStack->setUndoLimit( mMaxUndoSteps );  // Set the max steps after setting are read

        // Read the recent opened files
        updateRecentFileActions();

        createWidgetInfo();

        connect( &mCirculation, SIGNAL( started() ),                                this,           SLOT( thread_started() ) );
        connect( &mCirculation, SIGNAL( stateChanged(int) ),                        this,           SLOT( stateChanged(int) ) );
        connect( &mCirculation, SIGNAL( modelLoaded(int) ),                         this,           SLOT( modelLoaded(int) ) );
        connect( &mCirculation, SIGNAL( modelSaved() ),                             this,           SLOT( modelSaved() ) );
        connect( &mCirculation, SIGNAL( actionFailed(int) ),                        this,           SLOT( actionFailed(int) ) );
        connect( &mCirculation, SIGNAL( initialRunDone() ),                         this,           SLOT( initialHeartbeatsDone() ) );
        connect( &mCirculation, SIGNAL( modelSaved() ),                             & mData,        SLOT( setRefValues() ) );
        connect( &mCirculation, SIGNAL( initializedModel(const TCirculation &) ),   & mData,        SLOT( initializeUser(const TCirculation &) ) );
        connect( &mCirculation, SIGNAL( updateBeatInfo(const TCirculation &) ),     & mData,        SLOT( addData(const TCirculation &) ) );
        connect( &mCirculation, SIGNAL( specsCopied() ),                            mParamView,     SLOT( specsHaveBeenCopied() ) );

        connect( &mData,        SIGNAL( initialized() ),                            mParamView,     SLOT( Initialize() ) );
        connect( &mData,        SIGNAL( initialized() ),                            this,           SLOT( allInitialized() ) );

        connect( &mData,        SIGNAL( updatedData() ),                            mSignalView,    SLOT( updateInfo() ) );
        connect( &mData,        SIGNAL( updatedData() ),                            mPVView,        SLOT( updateInfo() ) );
        connect( &mData,        SIGNAL( updatedData() ),                            mValuesView,    SLOT( updateInfo() ) );
        connect( &mData,        SIGNAL( updatedData() ),                            mParamView,     SLOT( updateInfo() ) );
        connect( &mData,        SIGNAL( updatedData() ),                            this,           SLOT( updateInfo() ) );

        // The next two connections MUST be queued because the update of the view must be triggered asynchrone. Updating the view
        // will trigger handling of the mouse move events again, otherwise handling stays disabled
        connect( &mData,        SIGNAL( updateTimeMarker() ),                       mSignalView,    SLOT( updateSceneForeGround() ), Qt::QueuedConnection );
        connect( &mData,        SIGNAL( updateTimeMarker() ),                       mPVView,        SLOT( updateSceneForeGround() ), Qt::QueuedConnection );

        connect( mParamView,    SIGNAL( updateHeartRateInterval(int) ),             & mCirculation, SLOT( updateHeartInterval(int) ) );
        connect( mParamView,    SIGNAL( updateSpecs(const TCirculation &) ),        & mCirculation, SLOT( updateSpecs(const TCirculation & ) ) );


#ifndef QT_NO_DEBUG
    //    mUndoView = new QUndoView(mUndoStack);
    //    mUndoView->setWindowTitle(tr("Undo List"));
    //    mUndoView->show();
    //    mUndoView->setAttribute(Qt::WA_QuitOnClose, false);
#endif

    //    QApplication::processEvents();

#ifdef QT_NO_DEBUG  // Only in release versions
        splashTimer.start();
        while( !splashTimer.hasExpired( 3000 ) ) // Here we make sure that the splashscreen will be shown for a longer period (added 3 sec.)
        {
            QCoreApplication::processEvents();
        }
#endif

//        if ( !filename.isEmpty() )
//        {
//            mSplashScreenPtr->showMessage( tr("Loading case file ..."), Qt::AlignTop | Qt::AlignHCenter, LOGO_TEXT_COLOR );
//        }

//        // TODO This should be done in the constructor. Default the machine must be started
//        // Checking must be done to

//        loadDefaultCase( filename );

//        //mSplashScreenPtr->showMessage( tr("Initializing the model..."), Qt::AlignTop | Qt::AlignHCenter, LOGO_TEXT_COLOR );

//        QApplication::processEvents();

//        mCirculation.start( /*QThread::LowestPriority*/ );
    }
}


void PCircAdaptMainwindow::writeSettings( void )
{
    /* Save postion/size of main window */
    QSettings & settings        = pAppPtr->getSettings();
    QSplitter*  mainSplitterPtr = qobject_cast<QSplitter*>( centralWidget() );

    settings.setValue( CIRCADAPT_MAINPOS, pos() );
    settings.setValue( CIRCADAPT_MAINSIZE, size() );
    settings.setValue( CIRCADAPT_MAINSTATE, saveState() );

    if ( mainSplitterPtr )
    {
        QSplitter* TmpSplitterPtr = NULL;

        settings.setValue( CIRCADAPT_MAINSPLITERSTATE, mainSplitterPtr->saveState() );

        if ( mainSplitterPtr->count() >= 1 )
        {
            TmpSplitterPtr = qobject_cast<QSplitter*>( mainSplitterPtr->widget( 0 ) );

            if ( TmpSplitterPtr )
            {
                settings.setValue( CIRCADAPT_TOPSPLITERSTATE, TmpSplitterPtr->saveState() );
            }
        }

        if ( mainSplitterPtr->count() >= 2 )
        {
            TmpSplitterPtr = qobject_cast<QSplitter*>( mainSplitterPtr->widget( 1 ) );

            if ( TmpSplitterPtr )
            {
                settings.setValue( CIRCADAPT_BOTTOMSPLITERSTATE, TmpSplitterPtr->saveState() );
            }
        }
    }

    settings.setValue(CIRCADAPT_CURDIR,mCurrentDir);

    if ( !settings.value( CIRCADAPT_OPENLASTFILE, false).toBool() )    // Dirty tric to write if the key not excists
    {
        settings.setValue( CIRCADAPT_OPENLASTFILE, false );
    }
    settings.setValue( CIRCADAPT_OPENLASTFILE, true );

    settings.setValue( CIRCADAPT_LASTFILE, mCirculation.getFilename() );

    settings.setValue( CIRCADAPT_RECENTFILES, mMaxRecentFiles );

    settings.setValue( CIRCADAPT_UNDO_STEPS, mMaxUndoSteps );

    if ( mValuesView )
    {
        mValuesView->writeSettings( settings );
    }
}

void PCircAdaptMainwindow::readSettings( void )
{
    bool        ok;
    int         intVal;
    QSettings & settings        = pAppPtr->getSettings();
    QSplitter*  mainSplitterPtr = qobject_cast<QSplitter*>( centralWidget() );
    QList<int>  tmpList;

    move( settings.value( CIRCADAPT_MAINPOS, QPoint(200, 200)).toPoint() );

    QSize size( settings.value( CIRCADAPT_MAINSIZE, QSize(10, 10)).toSize() );

    if ( size != QSize( 10, 10 ) )
    {
        resize( size );
    }
    else
    {
        if ( isVisible() )
        {
            showMaximized();
        }
        else
        {
            setWindowState( Qt::WindowMaximized );
        }
    }

    restoreState( settings.value( CIRCADAPT_MAINSTATE, QByteArray() ).toByteArray() );

    if ( mainSplitterPtr )
    {
        QSplitter* TmpSplitterPtr = NULL;

        if ( !mainSplitterPtr->restoreState( settings.value( CIRCADAPT_MAINSPLITERSTATE,
                                                             QByteArray()).toByteArray() ) )
        {
            // If no previous state was found, use a predefined size
            tmpList.append( height() * 0.600 );
            tmpList.append( height() * 0.400 );

            mainSplitterPtr->setSizes( tmpList );

            tmpList.clear();
        }

        if ( mainSplitterPtr->count() >= 1 )
        {
            TmpSplitterPtr = qobject_cast<QSplitter*>( mainSplitterPtr->widget( 0 ) );

            if ( TmpSplitterPtr )
            {
                if ( !TmpSplitterPtr->restoreState( settings.value( CIRCADAPT_TOPSPLITERSTATE,
                                                                    QByteArray()).toByteArray() ) )
                {
                    // If no previous state was found, use a predefined size
                    tmpList.append( width() * 0.400 );
                    tmpList.append( width() * 0.600 );

                    mainSplitterPtr->setSizes( tmpList );

                    tmpList.clear();
                }
            }
        }

        if ( mainSplitterPtr->count() >= 2 )
        {
            TmpSplitterPtr = qobject_cast<QSplitter*>( mainSplitterPtr->widget( 1 ) );

            if ( TmpSplitterPtr )
            {
                if ( ! TmpSplitterPtr->restoreState( settings.value( CIRCADAPT_BOTTOMSPLITERSTATE,
                                                                     QByteArray()).toByteArray() ) )
                {
                    // If no previous state was found, use a predefined size
                    tmpList.append( width() * 0.600 );
                    tmpList.append( width() * 0.400 );

                    mainSplitterPtr->setSizes( tmpList );
                }
            }
        }
    }

    mCurrentDir     = QDir::homePath();

#ifdef Q_WS_MACX
    mDataDir        = QApplication::applicationDirPath() + "/../../../cases";
#else
    mDataDir        = QApplication::applicationDirPath() + "/cases";
#endif

    if ( mCirculation.getFilename().isEmpty() &&
         settings.value( CIRCADAPT_OPENLASTFILE, false).toBool() )
    {
        mCirculation.load( settings.value( CIRCADAPT_LASTFILE, QString() ).toString() );
    }

    intVal = settings.value(CIRCADAPT_RECENTFILES, RECENT_FILES_DEFAULT).toInt( &ok );
    if ( ok && intVal != mMaxRecentFiles )
    {
        mMaxRecentFiles = intVal;
    }

    intVal = settings.value(CIRCADAPT_UNDO_STEPS, MAX_UNDO_STEPS_DEFAULT).toInt( &ok );
    if ( ok && intVal != mMaxUndoSteps )
    {
        mMaxUndoSteps = intVal;
    }

    if ( mValuesView )
    {
        mValuesView->readSettings( settings );
    }
}


void PCircAdaptMainwindow::loadDefaultCase(const QString &filename)
{
    if ( !filename.isEmpty() )
    {
        open( filename );
    }
    else if ( mCirculation.getFilename().isEmpty() )
    {
        QDir        dirn( mDataDir );
        QStringList filters;

        filters << ( QString( "*" ) + CASE_FILE_EXT );

        QFileInfoList cases = dirn.entryInfoList( filters );

        if ( cases.size() > 0 )
        {
            open( cases[0].filePath() );
        }
    }
}


void PCircAdaptMainwindow::open( const QString & filename )
{
    QFileInfo file( filename );

    if ( file.exists() )
    {
        QApplication::setOverrideCursor( Qt::WaitCursor );

        bool active = mCirculation.isRunning();

        if ( active )
        {
            mCirculation.pauseHeartbeat();
        }

        if ( mSplashScreenPtr != NULL )
        {
            mSplashScreenPtr->showMessage( tr("Loading case file ...\n\n") + filename, Qt::AlignTop | Qt::AlignHCenter, LOGO_TEXT_COLOR );
        }

        mCirculation.load( filename );

        if ( active )
        {
            mCirculation.startHeartbeat();
        }

    //    setCursor( QCursor( Qt::WaitCursor ) );

        mUi.actionRun->setEnabled( false );
        mUi.actionStore_current->setEnabled( false );
        mUi.actionReset->setEnabled( false );
        mUi.actionAutoscale->setEnabled( false );
    }
    else
    {
        // Inform the user
        actionFailed( MSimulateCirculation::openFailed );
    }
}


void PCircAdaptMainwindow::initSpecs(const TCirculation &specs)
{
    mData.copySpecifications(specs);
}


void PCircAdaptMainwindow::InitUndoStack()
{
//    // (Re)Initialyse the undo stack
//    mUndoStack->addInitial<PCircAdaptData, PDataUndo>( mCirculation );
//    mUndoStack->addInitial<PActivationConstruction, PActConstrUndo>( mCirculation.atrialAct());
//    mUndoStack->addInitial<PActivationConstruction, PActConstrUndo>( mCirculation.ventrAct() );
//    // Add all the signal systems
//    for ( int index = 0; index < mCirculation.leadSystems().size(); index++ )
//    {
//        mUndoStack->addInitial<PLeadSystem, PLeadSystemUndo>( const_cast<QVector<PLeadSystem>&>(mCirculation.leadSystems())[index] );
//    }
}


void PCircAdaptMainwindow::buildMenu( void )
{
    // Due to the Qt implementation of the undo/redo functionality the undo and redo
    // action buttons / menu items must be created in code. De actions are of type
    // QUndoAction and not of type QAction.
//    QAction *UndoAction = mUndoStack->createUndoAction(this);
//    UndoAction->setIcon( QIcon(":/CircAdapt/undo.png") );
//    UndoAction->setShortcuts(QKeySequence::Undo);
//    UndoAction->setToolTip( tr("Undo the previous change") );

//    QAction *RedoAction = mUndoStack->createRedoAction(this);
//    RedoAction->setIcon( QIcon(":/CircAdapt/redo.png") );
//    RedoAction->setShortcuts(QKeySequence::Redo);
//    RedoAction->setToolTip( tr("Redo the previous undone change") );
    //ui.APToolBar->insertAction( ui.actionUndoAll, RedoAction );
//    mUi.menuEdit->insertAction( mUi.actionUndoAll, RedoAction );


    QAction * actionPtr = NULL;
    QMenu *   wndMenu   = new QMenu( "&Windows", this );

    actionPtr = wndMenu->addAction( tr("signals") );
    actionPtr->setCheckable( true );
    actionPtr->setChecked( true );
    connect( actionPtr, SIGNAL(triggered()), mWinSigMapper, SLOT(map()) );
    mWinSigMapper->setMapping( actionPtr, mSignalView );

    actionPtr = wndMenu->addAction( tr("PV-loop") );
    actionPtr->setCheckable( true );
    actionPtr->setChecked( true );
    connect( actionPtr, SIGNAL(triggered()), mWinSigMapper, SLOT(map()) );
    mWinSigMapper->setMapping( actionPtr, mPVView );

    actionPtr = wndMenu->addAction( tr("Values") );
    actionPtr->setCheckable( true );
    actionPtr->setChecked( false );
    mValuesView->hide();
    connect( actionPtr, SIGNAL(triggered()), mWinSigMapper, SLOT(map()) );
    mWinSigMapper->setMapping( actionPtr, mValuesView );

    actionPtr = wndMenu->addAction( tr("Controls") );
    actionPtr->setCheckable( true );
    actionPtr->setChecked( true );
    connect( actionPtr, SIGNAL(triggered()), mWinSigMapper, SLOT(map()) );
    mWinSigMapper->setMapping( actionPtr, mParamView );

    mUi.menuView->addMenu( wndMenu );

    // For now (this version) remove the edit menu from the main menu. It isn't used
    mUi.menuEdit->setDisabled( true );
    mUi.menuEdit->setVisible( false );
}


void PCircAdaptMainwindow::toggleWindowVisibleState( QWidget * windowPtr )
{
    if ( windowPtr )
    {
        if ( windowPtr->isVisible() )
        {
            windowPtr->hide();
        }
        else
        {
            windowPtr->show();
        }
    }
}



void PCircAdaptMainwindow::createWidgetInfo( void )
{
//    mWidgetInfo.insert(mHeartView, tr("heart"));
//    mWidgetInfo.insert(mHeartMain, tr("heart"));
//    mWidgetInfo.insert(mThoraxView, tr("thorax"));
//    mWidgetInfo.insert(mThoraxMain, tr("thorax"));
//    mWidgetInfo.insert(mECGView, tr("ecg"));
//    mWidgetInfo.insert(mECGMain, tr("ecg"));
//    mWidgetInfo.insert(mAPView, tr("tmp"));
//    mWidgetInfo.insert(mAPMain, tr("tmp"));
//    mWidgetInfo.insert(mCircAdaptToolbox, tr("toolbox"));
//    mWidgetInfo.insert(mUi.movieDockWidget, tr("thorax::movie"));
//    mWidgetInfo.insert(mFocusWidget, tr("focus"));
}


//void PCircAdaptMainwindow::refreshWindow( void )
//{


//}


/*****************************************************************************************************/

void PCircAdaptMainwindow::closeEvent(QCloseEvent *event)
{
    // TODO
    // For the future
    //    if ( mDirty &&
    //        QMessageBox::question(this,tr("Save File"),tr("Data has been changed"),QMessageBox::Yes,QMessageBox::No ) == QMessageBox::Yes )
    //    {
    //        on_actionSave_triggered();
    //    }

    writeSettings();
    event->accept();
}


void PCircAdaptMainwindow::updateAllSettings( int changed )
{
    // For the meaning of the parameter POptionsDialog::mChangedItems

    Q_UNUSED( changed )

//    POptionsDialog::ChangedOptionsType changedType( changed );

//    if ( changedType.testFlag( POptionsDialog::PARAM_TABS_CHANGED ) )
//    {
//        // This is the only part of read settings that has to be read afer the
//        // options have been changed
//        QSettings & settings = pAppPtr->getSettings();
//        bool        ok;

//        if ( ok )
//        {
//        }
//    }
}


void PCircAdaptMainwindow::timerEvent(QTimerEvent *event)
{
    if ( event->timerId() == mTimerId )
    {
        //int time;
    }
}




void PCircAdaptMainwindow::on_action_Open_triggered()
{
    QString fn = QFileDialog::getOpenFileName( this,
                                               tr("open case file"),
                                               mCurrentDir,
                                               tr( "CircAdapt case files") + " (*" + CASE_FILE_EXT + ")" );

    if ( !fn.isEmpty() )
    {
        mCurrentDir = fn;
        fn.resize( fn.lastIndexOf( CASE_FILE_EXT ) );
        mCurrentDir.resize( mCurrentDir.lastIndexOf( "/" ) );
        fn += CASE_FILE_EXT;
        open( fn );
    }
}


//void PCircAdaptMainwindow::on_actionDefault_open_triggered()
//{
//    QString fn = QFileDialog::getOpenFileName( this,
//                                               "open case file",
//                                               mDataDir,
//                                               tr( "CircAdapt case files") + " (*" + CASE_FILE_EXT + ")" );
//    if ( !fn.isEmpty() )
//    {
//        mDataDir = fn;
//        fn.resize( fn.lastIndexOf( CASE_FILE_EXT ) );
//        fn += CASE_FILE_EXT;
//        open( fn );
//    }
//}


void PCircAdaptMainwindow::on_actionSave_as_triggered()
{
    QString fn = QFileDialog::getSaveFileName( this,
                                               "save case file",
                                               mCurrentDir,
                                               QString( "*" ) + CASE_FILE_EXT );

    if ( !fn.isEmpty() && (fn != mCirculation.getFilename()) )
    {
        if ( !fn.endsWith( CASE_FILE_EXT ) )
        {
            fn += CASE_FILE_EXT;
        }

        mCirculation.save( fn );

        // TODO This should be done in a slot for a signal to make
        //if (  )
        {
            setCurrentFile( mCirculation.getFilename() );

            setWindowTitle( "CircAdapt  " + mCirculation.getFilename() );
        }
    }
}

void PCircAdaptMainwindow::on_actionSave_triggered()
{
    QFileInfo info( mCirculation.getFilename() );

    if ( !info.isWritable() )
    {
        on_actionSave_as_triggered();
    }
    else
    {
        mCirculation.save( mCirculation.getFilename() );
    }
}




void PCircAdaptMainwindow::on_actionPrint_triggered()
{
//    QPrinter printer(QPrinter::HighResolution);
//    printer.setOrientation(QPrinter::Portrait);

//    QPrintDialog *dialog = new QPrintDialog(&printer, this);
//    dialog->setWindowTitle(tr("Print CircAdapt report"));

//    int retVal = dialog->exec();

//    if ( retVal != QDialog::Accepted)
//    {
//        delete dialog;

//        return;
//    }

//    if (printer.isValid())
//    {
        //        printer.setOutputFileName("print.ps");
//        QPainter painter;
//        painter.begin(&printer);
//        mECGView->setColors(true);
//        mECGView->render(&painter);
//        mECGView->setColors(false);

//        printer.newPage();
//        QRectF rectF = printer.pageRect();
//        rectF.setBottom(rectF.bottom() * 0.33333);
//        QRect viewport = mAPView->viewport()->rect();
//        mAPView->setColors(true);
//        mAPView->render(&painter,rectF);
//        mAPView->setColors(false);


//        QRect rect = printer.pageRect();
//        rect.setTop(rect.bottom() * 0.3333333);
//        rect.setBottom(rect.bottom() * 0.666667);
//        rect.setLeft(rect.right()*.2);
//        rect.setRight(rect.right()*.8);
//        QPixmap pixmap(mHeartView->renderPixmap(rect.width(),rect.height()));
//        painter.drawPixmap(rect,pixmap);

//        rect = printer.pageRect();
//        rect.setTop(rect.bottom() * 0.666667);
//        rect.setLeft(rect.right()*.2);
//        rect.setRight(rect.right()*.8);
//        pixmap =mThoraxView->renderPixmap(rect.width(),rect.height());
//        painter.drawPixmap(rect,pixmap);

//        painter.end();
//    }

//    delete dialog;

//    mHeartView->repaint();
}

void PCircAdaptMainwindow::on_actionCopy_triggered()
{
    if ( mSignalView->underMouse() )
    {
        mSignalView->copyToClipBoard();
    }
    else if ( mPVView->underMouse() )
    {
        mPVView->copyToClipBoard();
    }
    else if ( mParamView->underMouse() )
    {
        mParamView->copyToClipBoard();
    }
    else if ( mValuesView->underMouse() )
    {
        mValuesView->copyToClipBoard();
    }
}


void PCircAdaptMainwindow::on_actionOptions_triggered()
{
    POptionsDialog opt( this );

    connect( &opt,
             SIGNAL( applySettings(int) ),
             this,
             SLOT( updateAllSettings(int) ) );
    connect( &opt,
             SIGNAL( applySettings(int) ),
             mSignalView,
             SLOT( updateAllSettings(int) ) );
    connect( &opt,
             SIGNAL( applySettings(int) ),
             mPVView,
             SLOT( updateAllSettings(int) ) );
    connect( &opt,
             SIGNAL( applySettings(int) ),
             mParamView,
             SLOT( updateAllSettings(int) ) );

    opt.exec();

    disconnect( &opt,
                SIGNAL( applySettings(int) ),
                this,
                SLOT( updateAllSettings(int) ) );
    disconnect( &opt,
                SIGNAL( applySettings(int) ),
                mSignalView,
                SLOT( updateAllSettings(int) ) );
    disconnect( &opt,
                SIGNAL( applySettings(int) ),
                mPVView,
                SLOT( updateAllSettings(int) ) );
    disconnect( &opt,
                SIGNAL( applySettings(int) ),
                mParamView,
                SLOT( updateAllSettings(int) ) );
}


void PCircAdaptMainwindow::on_actionExit_triggered()
{
    close();
}


void PCircAdaptMainwindow::on_actionAbout_triggered()
{
    PCircAdaptAboutDialog about( this );

    about.exec();
}


void PCircAdaptMainwindow::on_actionContext_Help_F1_triggered()
{
    QList<QWidget*> myChildren = findChildren<QWidget*>();

    bool found = false;
    int i;

    for ( i = 0; i < myChildren.size(); i++ )
    {
        if ( myChildren[i]->underMouse() &&
             mWidgetInfo.contains( myChildren[i] ) )
        {
            found = true;
            break;
        }
    }

    if ( found )
    {
        QStringList lst = mWidgetInfo.value(myChildren[i]).split(QLatin1Char(' '));
//        mHelp->showHelpForId(lst.last());
    }
    else
    {
//        mHelp->showDocumentation("index.html");
    }
}


void PCircAdaptMainwindow::on_actionIndex_triggered()
{
//    mHelp->showDocumentation("index.html");
}


void PCircAdaptMainwindow::setCurrentFile(const QString &fileName)
{
    QSettings & settings    = pAppPtr->getSettings();
    QStringList files       = settings.value( "recentFileList" ).toStringList();

    files.removeAll( fileName );
    files.prepend( fileName );

    while ( files.size() > mMaxRecentFiles )
    {
        files.removeLast();
    }

    settings.setValue( "recentFileList", files );

    foreach ( QWidget *widget, QApplication::topLevelWidgets() )
    {
        PCircAdaptMainwindow *mainWin = qobject_cast<PCircAdaptMainwindow *>( widget );

        if ( mainWin )
        {
            mainWin->updateRecentFileActions();
        }
    }
}


void PCircAdaptMainwindow::updateRecentFileActions()
{
    int         index;
    QSettings & settings    = pAppPtr->getSettings();
    QStringList files       = settings.value("recentFileList").toStringList();

    if ( mRecentFileActs.isEmpty() )    // If the resent file list isn't created yet ...
    {
        mUi.menuFile->insertSeparator( mUi.actionExit );

        for ( index = 0; index < mMaxRecentFiles; ++index )
        {
            QAction* newItemPtr = new QAction( this );

            if ( newItemPtr != NULL )
            {
                newItemPtr->setVisible( false );

                connect( newItemPtr, SIGNAL(triggered()), this, SLOT(openRecentFile()) );

                mUi.menuFile->insertAction( mUi.actionExit, newItemPtr );

                mRecentFileActs.append( newItemPtr );
            }
        }

        mRecentFileSeparator = mUi.menuFile->insertSeparator(mUi.actionExit);
    }

    if ( files.isEmpty() )
    {
        QDir        dir = QDir( mDataDir );
        QStringList filters;

        filters << QString( "*" ) + CASE_FILE_EXT;

        files = dir.entryList( filters, QDir::Files );

    }

    int numRecentFiles  = qMin( files.size(), mRecentFileActs.size() );
    int number = 0;

    for ( index = 0; index < numRecentFiles; ++index )
    {
        if ( QFileInfo( files[index]).exists() )
        {
            QString text = tr("&%1 %2").arg(number + 1).arg( strippedName( files[index] ) );

            mRecentFileActs[index]->setText( text );
            mRecentFileActs[index]->setData( files[index] );
            mRecentFileActs[index]->setVisible( true );

            number++;
        }
    }

    numRecentFiles = index;

    for ( ; index < mRecentFileActs.size(); ++index )
    {
        mRecentFileActs[index]->setVisible( false );
    }

    mRecentFileSeparator->setVisible( numRecentFiles > 0 );
}


QString PCircAdaptMainwindow::strippedName( const QString & fullFileName )
{
    return QFileInfo( fullFileName ).fileName();
}


void PCircAdaptMainwindow::openRecentFile()
{
    QAction *action = qobject_cast<QAction *>( sender() );

    if ( action )
    {
        open( action->data().toString() );
    }
}


void PCircAdaptMainwindow::thread_started( void )
{
    // For now, start the simulation
    on_actionRun_triggered();

    if ( mSplashScreenPtr != NULL )
    {
        mSplashScreenPtr->showMessage( tr("Initializing the model..."), Qt::AlignTop | Qt::AlignHCenter, LOGO_TEXT_COLOR );
    }
}


void PCircAdaptMainwindow::on_actionRun_triggered()
{
    if ( mCirculation.isRunning() )
    {
        if ( mCirculation.isActive() )
        {
            mCirculation.pauseHeartbeat();
            mUi.actionRun->setText( tr("Play") );
        }
        else
        {
            mCirculation.startHeartbeat();
            mUi.actionRun->setText( tr("Pause") );
        }
    }
}


void PCircAdaptMainwindow::on_actionStore_current_triggered()
{
    mData.saveSignals();
}


void PCircAdaptMainwindow::on_actionAutoscale_triggered()
{
    mPVView->autoScale();
    mSignalView->autoScale();
}


void PCircAdaptMainwindow::on_actionReset_triggered()
{
    QApplication::setOverrideCursor( Qt::WaitCursor );

    mCirculation.reset();

//    setCursor(QCursor(Qt::WaitCursor));

    mUi.actionRun->setEnabled( false );
    mUi.actionStore_current->setEnabled( false );
    mUi.actionReset->setEnabled( false );
    mUi.actionAutoscale->setEnabled( false );
}


void PCircAdaptMainwindow::stateChanged( /*MSimulateCirculation::statetype*/ int state )
{
    QString text;

    switch ( (MSimulateCirculation::statetype)state )
    {
    case MSimulateCirculation::pauseState:
        text = tr( "The simulation is paused" );
        if ( mUi.actionRun->isChecked() )
        {
            // Just to be sure that the button is in the correct state
            mUi.actionRun->setChecked( false );
            mUi.actionRun->setText( tr("Play") );
        }
        break;
    case MSimulateCirculation::activeState:
        text = tr( "The simulation is active" );
        if ( !mUi.actionRun->isChecked() )
        {
            // Just to be sure that the button is in the correct state
            mUi.actionRun->setChecked( true );
            mUi.actionRun->setText( tr("Pause") );
        }
        break;
    case MSimulateCirculation::initState:
    default:
        text = tr( "The simulation is being initialized" );
        break;
    }

    statusBar()->showMessage( text, STATUSBAR_MESSAGE_TIME );
}


void PCircAdaptMainwindow::modelLoaded( /*MSimulateCirculation::modeltype*/ int type )
{
    QString text;

    mInitialScale = true;   // Signalview must probably rescale these views

    switch ( (MSimulateCirculation::modeltype)type )
    {
    case MSimulateCirculation::loadedmodel:
        text = tr("The simulation model is (re)loaded");

        if ( !mCirculation.getFilename().isEmpty() )
        {
            setCurrentFile( mCirculation.getFilename() );
        }

        setWindowTitle( "CircAdapt  " + mCirculation.getFilename() );
        break;
    case MSimulateCirculation::defaultmodel:
    default:
        text = tr("The simulation model is (re-)initialized with default settings");
        setWindowTitle( "CircAdapt" );
        break;
    }

    // Clear the whole undo stack; reinitialisation must be done afterwards
//        mUndoStack->clear();
//        mUndoStack->clearInitial();

    // (Re)Initialyse the undo stack
//  InitUndoStack();

    statusBar()->showMessage( text, STATUSBAR_MESSAGE_TIME );
}


void PCircAdaptMainwindow::modelSaved( void )
{
    statusBar()->showMessage( tr("The current model is saved"), STATUSBAR_MESSAGE_TIME );
}


void PCircAdaptMainwindow::actionFailed( /*MSimulateCirculation::failtype*/ int type )
{
    QString title;
    QString text;

    switch ( (MSimulateCirculation::failtype)type )
    {
    case MSimulateCirculation::openFailed:
        title   = tr( "Open failed" );
        text    = tr( "Opening the case failed! Continuing to simulate previous or default situation." );
        break;
    case MSimulateCirculation::saveFailed:
        title   = tr( "Save failed" );
        text    = tr( "Saving this case failed! Changes are not (correctly) stored!" );
        break;
    default:
        title   = tr( "Action failed" );
        text    = tr( "A previous initiated action failed!" );
        break;
    }

    mUi.actionRun->setEnabled( true );
    mUi.actionStore_current->setEnabled( true );
    mUi.actionReset->setEnabled( true );
    mUi.actionAutoscale->setEnabled( true );

//    setCursor(QCursor(Qt::ArrowCursor));
    QApplication::restoreOverrideCursor();

    // Inform the user of the failure
    QMessageBox::warning( this, title, text );
}



void PCircAdaptMainwindow::allInitialized( void )
{
    // After startup, this point must be reacht at least one time. Default enable these buttons
    mUi.actionRun->setEnabled( true );
    mUi.actionStore_current->setEnabled( true );
    mUi.actionReset->setEnabled( true );
    mUi.actionAutoscale->setEnabled( true );

    mSignalView->initializeView();
    mPVView->initializeView();

//    setCursor(QCursor(Qt::ArrowCursor));
    QApplication::restoreOverrideCursor();

//    on_actionAutoscale_triggered(); // Just to be sure that all will be shown correctly

    statusBar()->showMessage( tr("Model initialyzed"), STATUSBAR_MESSAGE_TIME );

//    if ( mSplashScreenPtr != NULL )
//    {
//        mSplashScreenPtr->finish( this );
//        delete mSplashScreenPtr;
//        mSplashScreenPtr = NULL;
//    }
}


void PCircAdaptMainwindow::initialHeartbeatsDone( void )
{

    statusBar()->showMessage( tr("Ready"), STATUSBAR_MESSAGE_TIME );

    if ( mSplashScreenPtr != NULL )
    {
        mSplashScreenPtr->finish( this );
        delete mSplashScreenPtr;
        mSplashScreenPtr = NULL;
    }
}


void PCircAdaptMainwindow::updateInfo( void )
{
    if ( mInitialScale )
    {
        mInitialScale = false;

        // New (initial/fresh) data is recieved. Probably a rescale is needed. Just do it.
        on_actionAutoscale_triggered();
    }
}

