/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: PCircAdaptOptionsDialog.h 776 2013-07-11 20:33:36Z eelco $
 *
 */

#ifndef _POPTIONSDIALOG_H
#define _POPTIONSDIALOG_H


//#include <QComboBox>
#include <QStringList>

#include "ui_PCircAdaptOptionsDialog.h"
#include "PCircAdaptDefines.h"


namespace Peacs
{
    //using namespace Support;
    namespace CircAdapt
    {
        /** @class POptionsDialog POptionsDialog.h "CircAdapt/POptionsDialog.h"
          * @brief The options dialog class.
          * This is the only place wher the seetings of CircAdapt are writen to file.
          */
        class POptionsDialog : public QDialog
        {
            Q_OBJECT
            Q_DISABLE_COPY( POptionsDialog )

        public:

            static const char * PARAM_TAB_ID_GEN;
            static const char * PARAM_TAB_ID_VENTR;
            static const char * PARAM_TAB_ID_ATRIA;
            static const char * PARAM_TAB_ID_VALVES;
            static const char * PARAM_TAB_ID_VASC;
            static const char * PARAM_TAB_ID_SHUNT;
            static const char * PARAM_TAB_ID_ADAPT;

            enum ChangedOption
            {
                NOTHING_CHANGED                 = 0x00000000,
                GRID_CHANGED                    = 0x00000001,
                SIGNAL_FONT_CHANGED             = 0x00000002,
                PARAM_TABS_CHANGED              = 0x00000004,
                SIGNAL_COLOR_CHANGED            = 0x00000008
            };

            Q_DECLARE_FLAGS( ChangedOptionsType, ChangedOption )

//            Q_FLAGS( ChangedOptionsType )
//            Q_DECLARE_METATYPE( ChangedOptionsType )

            /** @brief constructor
              * @param parent The parent for this dialog (default 0)
              */
            explicit POptionsDialog( QWidget * parent = 0 );


        signals:

            /** @brief Signal emitted when the settings should be applied
              *
              * @param changed indicated what options have changed. This is a 32 bit-flag value
              */
            void applySettings( int changed );


        protected:

            /** @brief not implemented yet. Handles for instance language changes
              */
            virtual void changeEvent(QEvent *e);


        private slots:

            /** @brief This function is called when the user presses one of the buttons in the button box
              */
            void on_buttonBox_clicked( QAbstractButton * button );


            /** @brief This function is called when the user selectes an other grid setting in the combobox
              *
              * @param index the new index being selected
              */
            void on_cbGrid_currentIndexChanged( int /*index*/ );


            /** @brief This function is called when the user selectes an other font type in the combobox
              *
              * @param index the new index being selected
              */
            void on_signalsFontsCombo_currentIndexChanged( int /*index*/ );


            /** @brief This function is called when the user presses the select font button
              * This function shows a font selection dialog to the user to specify a new font to use
              * for the selected signal text attribute.
              *
              * @param checked inducated if the button is checked if this option is enabled
              */
            void on_SignalsFontSelect_clicked( bool /*checked*/ );


            /** @brief This function is called when the user (un)checked a parameter tabs visible checkbox
              *
              * @param id The id of the checkbox the user (un)checked
              */
            void on_paramTabSelectChkBg_buttonClicked ( int id );


            /** @brief This function is called when the user selectes a default selected tab radio button
              *
              * @param id The id of the radio button the user selected
              */
            void on_paramTabSelectRadioBg_buttonClicked ( int /*id*/ );


            /** @brief This function is called when the user presses a signal color change button
              *
              * @param id The id of the push button the user pressed
              */
            void on_signalColorChangeBg_buttonClicked ( int id );


        private:

            /**
              */
            void init( void );


            /** @brief write all settings to file
              */
            void writeCircAdaptSettings(void);


            /** @brief read all settings from file
              */
            void readCircAdaptSettings(void);


            /** @brief restore all options to  the default values
              */
            void restoreDefaults(void);


            /**
              */
            void setDirty( bool dirty );


            /**
              */
            void setSignalsFontDisplayText( void );


            /** @brief the Qt generated ui items  */
            Ui::POptionsDialog ui;


            /** @brief the AP grid type*/
            Support::Widget::Grid_type mGridType;


            /** @brief A list containing the font descriptions used to draw text in the signals view
              */
            QStringList mSignalFonts;


            /** @brief A list containing the colors used when drawing the content of the signals view
              */
            QList<QColor> mSignalColors;


            /** @brief true if the user changed any setting
              */
            bool mDirty;


            /** @brief Indicated roughly what has changed. 0 means nothing
              * This item holds a 32 bit flag value
              */
            ChangedOptionsType mChangedItems;
        };
    }
}
#endif // _POPTIONSDIALOG_H
