/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: main.cpp 1174 2014-03-21 14:15:01Z eelco $
 *
 */


#include <QtCore>
//#include <QStringList>

//#include <QDomDocument>
//#include <QTextStream>
//#include <QFileInfo>

//#ifdef Q_WS_MAC
//    #include "main.h"
//    #include <QFileOpenEvent>
//#else
//    #include "PCircAdaptMainwindow.h"
//#endif

#include "pcircadapt.h"
#include "tcirculation.h"


//using namespace Peacs::CircAdapt;


int main( int argc, char *argv[] )
{
    Q_INIT_RESOURCE(pcircadapt);

    int             retVal = 0;

    try {
        Peacs::CircAdapt::PCircAdapt      circadapt( argc, argv );

    // This enables creation of objects at run-time
    qRegisterMetaType<Peacs::CircAdapt::TCirculation>( "TCirculation" );
//    qRegisterMetaType<THeart>( "THeartState" );
//    qRegisterMetaType<TResistance>( "TResistanceState" );
//    qRegisterMetaType<TTube>( "TTubeState" );

        retVal = circadapt.exec();

    }
    catch( const char * str )
    {
        qErrnoWarning(retVal,QString("critical Error %s").arg(str).toAscii());
        retVal = -1;
    }
    catch( QtConcurrent::Exception &var)
    {
        qErrnoWarning(retVal, QString("critical error due to QT exception, ECGsim ends: ").toAscii() + var.what());
        retVal = -2;
    }
    catch(const std::bad_alloc &ba)
    {
        qErrnoWarning(retVal, QString("critical error due to memory allocation error, ECGsim ends").toAscii() + ba.what());
        retVal = -3;
    }
    catch ( ... )
    {
        qErrnoWarning(retVal,"critical error, ECGsim ends");
        retVal = -4;
    }

    return retVal;
}




//    PCircAdapt      circadapt( argc, argv );
//    int             retVal = 0;
//    QString         filename;
//    QStringList     args( circadapt.arguments() );


//    // This enables creation of objects at run-time
//    qRegisterMetaType<TCirculation>( "TCirculation" );
////    qRegisterMetaType<THeart>( "THeartState" );
////    qRegisterMetaType<TResistance>( "TResistanceState" );
////    qRegisterMetaType<TTube>( "TTubeState" );


//    // Needed to get the filename to open under OSx when the file is to be opened due to
//    // double clicking on it
//#ifdef Q_WS_MAC
//    PEventListener  eventListener;

//    circadapt.installEventFilter( &eventListener );
//#endif

//    if ( args.size() > 0 )
//    {
//        for ( int i = 0; i < args.size(); i++ )
//        {
//            if ( args[i].contains( CASE_FILE_EXT ) )
//            {
//                QFileInfo casefile( args.at( i ).toLatin1() );

//                if ( casefile.exists() )
//                {
//                    // We have a case file to load
//                    filename = casefile.filePath();
//                }
//            }
//        }
//    }

//    // Under OSx a window application that is opened by double clicking on a, for this application, registered file
//    // the file name to open isn't added to the commandline. The application is told by the operating system to
//    // open the file by sending a signal to the application. The eventlistener is installed to get the filename to
//    // open. Before the event can be recieved the events must be processed.
//#ifdef Q_WS_MAC
//    QApplication::processEvents();
//    if ( filename.isEmpty() && !eventListener.fileName().isEmpty() )
//    {
//        filename = eventListener.fileName();
//    }
//#endif

//    try {
//        PCircAdaptMainwindow mainWindow( filename );

//#ifdef Q_WS_MAC
//        eventListener.setMainWin( &mainWindow );
//#endif

//        mainWindow.show();

//        retVal = circadapt.exec();

//#ifdef Q_WS_MAC
//        mainWindow.removeEventFilter( &eventListener );
//#endif
//    }
//    catch( const char * str )
//    {
//        qErrnoWarning( retVal, QString( "critical Error %s" ).arg( str ).toAscii() );
//        retVal = -1;
//    }
//    catch( QtConcurrent::Exception &var)
//    {
//        qErrnoWarning( retVal, QString( "critical error due to QT exception, CircAdapt ends: " ).toAscii() + var.what() );
//        retVal = -2;
//    }
//    catch(const std::bad_alloc &ba)
//    {
//        qErrnoWarning( retVal, QString( "critical error due to memory allocation error, CircAdapt ends" ).toAscii() + ba.what() );
//        retVal = -3;
//    }
//    catch ( ... )
//    {
//        qErrnoWarning( retVal, "critical error, CircAdapt ends" );
//        retVal = -4;
//    }

//    return retVal;
//}

//#ifdef Q_WS_MAC
//bool PEventListener::eventFilter( QObject* obj, QEvent* event )
//{
//    if ( event->type() == QEvent::FileOpen )
//    {
//        QString filename = static_cast<QFileOpenEvent*>(event)->file();

//        if ( mMainWindowPtr )
//        {
//            // Application is running and the window is already showing. Open the file
//            if ( QApplication::activeModalWidget() == NULL )
//            {
//                mMainWindowPtr->loadDefaultCase( filename );
//            }
//            else
//            {
//                return true; // ignore file open events while a modal dialog is displayed
//            }
//        }
//        else {
//            // Application is running but the window isn't showing yet. store the name of
//            // the file so it can be given to the window class to open.
//            mFileName = filename;
//        }
//    }
//    else
//    {
//        // Give it on to the standard event processing
//        QObject::eventFilter( obj, event );
//    }

//    return false;
//}
//#endif
