/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: pCircAdaptAboutdialog.cpp 1019 2014-01-21 11:10:25Z eelco $
 *
 */

#include <QStringList>
#include <QUrl>
#include <QDesktopServices>

#include "PCircAdaptaboutdialog.h"
#include <time.h>

#include "PeacsSupport_global.h"


#define QUOTE_STR(x) #x
#define BUILD_YEAR_STR( x, y, z ) QUOTE_STR( x y z )


#ifdef CIRCADAPT_YEAR_LOW_RC
    #ifdef CIRCADAPT_YEAR_HIGH_RC
        #define CIRCADAPT_YEAR_STR BUILD_YEAR_STR( CIRCADAPT_YEAR_LOW_RC, -, CIRCADAPT_YEAR_HIGH_RC )
    #else
        #define CIRCADAPT_YEAR_STR QUOTE_STR( CIRCADAPT_YEAR_LOW_RC )
    #endif
#else
    #ifdef CIRCADAPT_YEAR_RC
        #define CIRCADAPT_YEAR_STR QUOTE_STR( CIRCADAPT_YEAR_RC )
    #else
        #ifdef CIRCADAPT_YEAR_HIGH_RC
            #define CIRCADAPT_YEAR_STR QUOTE_STR( CIRCADAPT_YEAR_HIGH_RC )
        #else
            #define CIRCADAPT_YEAR_STR ""
        #endif
    #endif
#endif


using namespace Peacs::CircAdapt;


PCircAdaptAboutDialog::PCircAdaptAboutDialog(QWidget *parent) :
    QDialog(parent)
{
    ui.setupUi( this );

    mLabelLinkCatcherPtr = new PLabelLinkCatcher(this);
    if ( mLabelLinkCatcherPtr != NULL )
    {
        ui.label->installEventFilter( mLabelLinkCatcherPtr );
        ui.label_2->installEventFilter( mLabelLinkCatcherPtr );
    }


    QString txt( "<p>CircAdapt Simulator version: " + QApplication::applicationVersion() + "<br>" );
    txt.append( QString("Compilation date: ") + QString(__DATE__) + QString("  ") + QString(__TIME__) + "</p>");
#ifdef CIRCADAPT_PRERELEASE
    txt.append("<p><b> - This version is a pre-release -</b></p>");
#endif
    txt.append( QString("<p> StITPro ") + CIRCADAPT_YEAR_STR + ". All rights reserved.</p>");
    txt.append( "<p>The CircAdapt Simulator is an interactive simulation tool " \
                "that can be used for education of cardiovascular physiology. " \
                "It is based on the CircAdapt model of cardiovascular system " \
                "dynamics developed at Maastricht University. For more information " \
                "visit <a href=\"http://www.circadapt.org\">www.circadapt.org</a>.</p>" \
                "<p>This software is distributed in the hope that it will " \
                "be useful but WITHOUT ANY WARRANTY; without even the " \
                "implied warranty of MERCHANTABILITY or FITNESS FOR A " \
                "PARTICULAR PURPOSE. See the GNU " \
                "General Public License for more details.</p>" \
                "<p>This software was developed with Qt and is distributed with its " \
                "corresponding libraries. See <a href=\"http://www.qt-project.org\">" \
                "www.qt-project.org</a> for more details.</p>" );
#ifdef SUPPORT_VERSION
    txt.append( "<p>This software uses the PeacsSupport code library developed " \
                "and copyrighted by Peacs. See <a href=\"http://www.peacs.nl\">" \
                "www.peacs.nl</a>.<br>PeacsSupport classes version: ");
    txt.append( Peacs::Support::PSupportInfo::getSupportVersionStr() );
    txt.append( "</p>" );
#endif
    txt.append( "<p>Installer is created with <a href=\"http://bitrock.com\">" \
                "BitRock InstallBuilder</a>.</p>" );

    ui.licenseText->setWordWrap( true );        // Also set in the UI edittor, but that flag sometimes resets itself
    ui.licenseText->setText( txt );
}


PCircAdaptAboutDialog::~PCircAdaptAboutDialog()
{
    if ( mLabelLinkCatcherPtr != NULL )
    {
        ui.label->removeEventFilter( mLabelLinkCatcherPtr );
        ui.label_2->removeEventFilter( mLabelLinkCatcherPtr );

        delete mLabelLinkCatcherPtr;
        mLabelLinkCatcherPtr = NULL;
    }
}


void PCircAdaptAboutDialog::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui.retranslateUi( this );
        break;
    default:
        break;
    }
}


bool PCircAdaptAboutDialog::PLabelLinkCatcher::eventFilter( QObject *obj, QEvent *event )
{
    if ( ( (obj == dynamic_cast<PCircAdaptAboutDialog*>(parent())->ui.label) ||
           (obj == dynamic_cast<PCircAdaptAboutDialog*>(parent())->ui.label_2) ) &&
         event->type() == QEvent::MouseButtonPress )
    {
        if (obj == dynamic_cast<PCircAdaptAboutDialog*>(parent())->ui.label)
        {
            QUrl url("http://www.peacs.nl");
//            QUrl url("http://www.peacs.nl/CircAdapt.php");
            QDesktopServices::openUrl( url );
        }
        else
        {
            QUrl url("http://www.circadapt.org");
            QDesktopServices::openUrl( url );
        }

        return true;
    } else {
        // standard event processing
        return QObject::eventFilter(obj, event);
    }
}
