/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: atria.h 789 2013-07-18 13:22:28Z peter $
 *
 */

#ifndef _ATRIA_H_
#define _ATRIA_H_

#include "defs.h"
#include "chamber.h"
#include "circelement.h"



using namespace Peacs::Support;
namespace Peacs
{

    namespace CircAdapt
    {

        class TAtria;

        /**
        Class for muscular chamber. Inherits from MCompliance.
        */
        class SAtria: public SCircElement
        {

        public:
            /** @brief Default Constructor */
            SAtria();

            /** @brief Constructor */
            SAtria(const QString &name);

            SAtria(const SAtria & SAtria);

            virtual ~SAtria() {;}


            virtual const SAtria& operator = (const SAtria &ventr);

            void init();

            /** @brief copy the sepcification parameters from the user class to the transfer class
              */
            void copyUserSpecifications(const TAtria &atria);

            SCompliance & rightAtrium() { return mRigtA; }

            SCompliance & leftAtrium() { return mLeftA; }

            const SChamber & getRightAtrium() const { return mRigtA; }

            const SChamber & getLeftAtrium() const { return mLeftA; }

            inline double getVolume() const { return mLeftA.getVolume() + mLeftA.getWallVolume() +
                                                     mRigtA.getVolume() + mRigtA.getWallVolume(); }

            void adjustForSackPressure(double pressure);

            /// Method to initialize cavity pressure and cross-section
            void initAtrialMechanics(double tCycleAct, double tCycleRef);

            void updateAtrialMechanics (double deltaT);

            /// Implement method to initialize adaptation variables
            void initAdaptation();

            /// Implement method to update adaptation variables
            void updateAdaptation();

            /// Method to perform adaptation
            void adapt (bool adaptWV, bool adaptSL, bool adaptWA, int nrAdaptUpdates);


            //==================== file functions ===================

            /** @brief returns the default version number */
            int getDefaultVersion(void) const;

            /** @brief Returns the class name */
            const char *getClassName(void) const;

            /// Virtual implementation of method to write specification to file
            virtual bool writeXml(QDomNode &parentNode) const;

            /// Virtual implementation of method to read specification from file
            virtual bool readXml(const QDomNode &parentNode);


        private:
            SChamber mLeftA;
            SChamber mRigtA;
        };
    }
}
#endif
