/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: circelement.cpp 776 2013-07-11 20:33:36Z eelco $
 *
 */

#include <QDebug>
#include "circelement.h"


using namespace Peacs::CircAdapt;


const QString CIRCELEMENTNAME     = "Undefined";

// Constructor
SCircElement::SCircElement():
    PXmlFile(),
    mElementName(CIRCELEMENTNAME),
    mLoadedVersion(-1)
{

}

SCircElement::SCircElement(const QString &name):
    PXmlFile(),
    mElementName(name),
    mLoadedVersion(-1)
{
}

SCircElement::SCircElement (const SCircElement &element):
    PXmlFile(element),
    mElementName(element.mElementName),
    mLoadedVersion(element.mLoadedVersion)
{

}

const SCircElement & SCircElement::operator = (const SCircElement &element)
{
    if ( this != &element)
    {
        mElementName = element.mElementName;
        mLoadedVersion = element.mLoadedVersion;
    }
    return *this;
}


bool SCircElement::readXml(const QDomNode &node)
{
    mElementName = getNodeName(node);
    mLoadedVersion = getNodeVersion(node);
    if ( mElementName.isEmpty() )
    {
        qDebug().nospace() << "no element name" ;
    }

    return ( !mElementName.isEmpty() &&
             readSpecification(node) &&
             readState(node) );
}

bool SCircElement::writeXml(QDomNode &parentNode) const
{
    bool ok = false;
    QDomNode myNode = createXmlEntry(parentNode,getClassName(),getDefaultVersion(),mElementName);
    if ( !myNode.isNull())
    {
        ok  = writeSpecification(myNode);
        ok &= writeState(myNode);
    }
    return ok;
}


