/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: circelement.h 776 2013-07-11 20:33:36Z eelco $
 *
 */
#ifndef _CIRCELEMENT_H_
#define _CIRCELEMENT_H_

#include <QString>
//#include "circsettings.h"


#include "pxmlfile.h"


using namespace Peacs::Support;
namespace Peacs
{

    namespace CircAdapt
    {
        /**
        Abstract base class for circulation elements.
        */
        class SCircElement : public PXmlFile
        {

        public:
            /* @brief Default Constructor */
            SCircElement();

            /* @brief Constructor */
            SCircElement (const QString &name);

            SCircElement (const SCircElement &element);

            virtual ~SCircElement() { ; }

            virtual void init() = 0;

            virtual int getDefaultVersion(void) const = 0;

            /** @brief Returns the class name */
            virtual const char *getClassName(void) const = 0;

            /** @returns the name of the CircAdapt element */
            inline const QString & getName() const { return mElementName; }

            /** @returns the name of the CircAdapt element */
            inline void setName(const QString & name) { mElementName = name; }

            virtual bool operator == (const SCircElement &element) { return mElementName == element.mElementName; }

            virtual bool operator != (const SCircElement &element) { return mElementName != element.mElementName; }

            virtual const SCircElement & operator = (const SCircElement &element);


            /// Virtual implementation of method to write specification to file
            virtual bool writeXml(QDomNode &parentNode) const;

            /// Virtual implementation of method to read specification from file
            virtual bool readXml(const QDomNode &parentNode);

            /// Abstract method to write specification to file
            virtual bool writeSpecification (QDomNode &/*parentNode*/) const { return true;}

            /// Abstract method to read specification from file
            virtual bool readSpecification (const QDomNode &/*parentNode*/) { return true; }

            /// Abstract method to write state to file
            virtual bool writeState (QDomNode &/*parentNode*/) const  { return true; }

            /// Abstract method to read state from file
            virtual bool readState (const QDomNode &/*parentNode*/) { return true; }

        private:
            // Element name
            QString mElementName;

            int mLoadedVersion;

        };
    }
}

#endif
