/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: circsettings.h 776 2013-07-11 20:33:36Z eelco $
 *
 */

#ifndef _CIRCSETTINGS_DEF_
#define _CIRCSETTINGS_DEF_

#include <QString>

/*---------------------------------------------------------------+
| circsettings.h                                                 |
| Include file for simulation settings for heart and circulation.|
|                                                                |
| Nico Kuijpers and Theo Arts                                    |
| August 29, 2008                                                |
| Maastricht University                                          |
+---------------------------------------------------------------*/

/*----- Use namespace std for cin, cout, QDebug().nospace(), string, etc. -----*/
using namespace std;

/*----- Define type for simulation time (microseconds) -----*/
typedef int Time;

/*----- Default settings for simulation control parameters -----*/
const Time CIRCSIMTIMESTEP     =     100;  // Simulation time step
const Time CIRCADAPTTIMESTEP   =    1000;  // Adaptation time step
const Time CIRCMONTIMESTEP     =    1000;  // Monitor time step
const int  CIRCTOTALHEARTBEATS =      50;  // Total number of heart beats

/*----- Maximum number of characters in char* string (C) -----*/
const int MAXNAMLEN = 128;

/*----- Value is not defined -----*/
const int NOTDEFINED = -1;

/*----- Specification parameters for circulation -----*/
const double BLOODDENSITY        = 1050.0;       // Density of blood
const double SACKPRESSADAPT      = 500.0;        // Sack adaptation parameter
const double SACKSTIFFNESS       = 10.0;         // Pericardial sack stiffness
const double MITRALLEAKPAR       = 5e-4;         // Mitral leak parameter
const double MITRALLEAKFACTOR    = 0.2;          // Mitral leak factor
const double MITRALLEAKMIN       = 1e-6;         // Minimum mitral leak
const double TRICUSPIDLEAKPAR    = 5e-4;         // Tricuspid leak parameter
const double TRICUSPIDLEAKFACTOR = 0.2;          // Tricuspid leak factor
const double TRICUSPIDLEAKMIN    = 1e-6;         // Minimum tricuspid leak
const double VALVEPMARGIN        = 10.0;         // Threshold for valve closure
const double VALVEVMARGINFACTOR  = 0.01;         // Factor valve velocity thresh
const double SEPTUMVENTRICLETAU  = 0.01;         // Tau for TriSeg computation
const double LREFDEFAULT         = 0.0329;       // LRef for TriSeg computation

/*----- Criterium for adaptation -----*/
// const double ERRFLOWCRITERIUM    = 0.05;         // Criterium for stationary flow
// DEBUG
const double ERRFLOWCRITERIUM    = 0.01;         // Criterium for stationary flow

/*----- Flags for adaptation to exercise -----*/
const bool ADAPT_WV = true;  // Adaptation of wall volume
const bool ADAPT_SL = true;  // Adaptation of sarcomere length
const bool ADAPT_WA = true;  // Adaptation of wall area



const double DEFAULT_VALVE_LENGTH      = 0.0201;
const double DEFAULT_VALVE_MAX_AOPEN   = 5.0000e-004; // open valve
const double DEFAULT_VALVE_MIN_ALEAK   = 5.0000e-010; // no leak

#endif
