/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: flowduct.h 776 2013-07-11 20:33:36Z eelco $
 *
 */

#ifndef _FLOWDUCT_H_
#define _FLOWDUCT_H_


#include "circelement.h"
#include "compliance.h"

/*---------------------------------------------------------------+
| Class: FlowDuct                                                |
| Representation of a flow channel between compliances.          |
| Inherits from CircElement.                                     |
| Valve and Resistance inherit from FlowDuct.                    |
|                                                                |
| Nico Kuijpers and Theo Arts                                    |
| September 12, 2008                                             |
| Maastricht University                                          |
|                                                                |
| Sum of flow over heart beat added to determine stability       |
| criterion for adaptation.                                      |
| Nico Kuijpers                                                  |
| July 7, 2010                                                   |
| Maastricht University                                          |
+---------------------------------------------------------------*/

namespace Peacs
{

    namespace CircAdapt
    {
    class TFlowDuct;
        /**
        Class for flow channel between compliances. Inherits from CircElement.
        */

        class SFlowDuct: public SCircElement {

        public:

            /// Constructor
            SFlowDuct(SCompliance &prox, SCompliance &dist );

            SFlowDuct (SCompliance &prox, SCompliance &dist,const QString &name);

            SFlowDuct (const SFlowDuct & flowduct);

            /// Destructor
            virtual ~SFlowDuct() { ; }

            virtual void init();


            /** @brief copy the sepcification parameters from the user class to the transfer class
              */
            void copyUserSpecifications(const TFlowDuct &) {;}


            bool isValid() const;

            const SFlowDuct & operator = (const SFlowDuct &flowduct);

            /** @brief returns the default version number */
            int getDefaultVersion(void) const;

            /** @brief Returns the class name */
            const char *getClassName(void) const;

            /// Method to get flow
            inline const double & getFlow() const { return mQ; }

            void setConnectedCompliance(SCompliance & prox, SCompliance& dist);

            const SCompliance & getProximal( )const { return mProximal; }

            const SCompliance & getDistal( )const { return mDistal; }


        //    const double & getFlow() const

            /// Virtual method to initialize adaptation variables
            virtual void initAdaptation() {}

            /// Virtual method to update adaptation variables
            virtual void updateAdaptation() { }


            //==================== file functions ===================

            /// Abstract method to write state to file
            bool writeState (QDomNode &parentNode) const;

            /// Abstract method to read state from file
            bool readState (const QDomNode &node);


        protected:
            // Configuration parameters
            SCompliance &mProximal;  // Pointer to proximate compliance

            SCompliance &mDistal;  // Pointer to distal compliance

            // Simulation state variables
            double mQ;          // Flow
        };
    }
}

#endif
