/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: resistance.h 776 2013-07-11 20:33:36Z eelco $
 *
 */

#ifndef _RESISTANCE_H_
#define _RESISTANCE_H_


#include "flowduct.h"

namespace Peacs
{

    namespace CircAdapt
    {
    class TResistance;
        /**
        Class for microcirculation. Inherits from FlowDuct.
        */
        class SResistance: public SFlowDuct {

        public:
            enum resistance_type { UNDEFINED_RESISTANCE=0,SYSTEMIC =1, PULMONARY=2, SYSTEMIC_CRANIAL=3};

            /* @brief Default Constructor */
            SResistance(resistance_type type,SCompliance &prox, SCompliance &dist );

            SResistance(resistance_type type, SCompliance &prox, SCompliance &dist, const QString &name);

            SResistance (const SResistance &res);

            /// Destructor
            virtual ~SResistance() { ;}

            void init();

            /** @brief copy the sepcification parameters from the user class to the transfer class
              */
            void copyUserSpecifications(const TResistance &res);


            resistance_type getResistanceType() const { return mResistanceType; }

            const SResistance& operator = (const SResistance &comp);

            double getResistance() const {return mR; }

            /// Method to control blood pressure
            void controlBloodPressure (double p0, double q0);

            /// Method to set flow
            void setFlow (double flow);

            /// Method to update flow
            void updateFlow();

            void initAdaptation() { ;}

            //==================== file functions ===================

            /** @brief returns the default version number */
            int getDefaultVersion(void) const;

            /** @brief Returns the class name */
            const char *getClassName(void) const;

            /// Abstract method to write specification to file
            bool writeSpecification (QDomNode &parentNode) const;

            /// Abstract method to read specification from file
            bool readSpecification (const QDomNode &node);

            /// Method to set default parameters for systemic peripheral resistance (LRp)
            void setParLRp ();

            /// Method to set default parameters for pulmonary peripheral resistance (RRp)
            void setParRRp ();

        private:
            // Specification parameters
            double mR; // Peripheral resistance

            resistance_type mResistanceType;
        };
    }
}

#endif
