/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: tokens.h 776 2013-07-11 20:33:36Z eelco $
 *
 */

#ifndef _TOKENS_DEF_
#define _TOKENS_DEF_


/************************************************************************/
/*                                                                      */
/* tokens.h:  Definition of tokens to parse configuration file for the  */
/*            simulation of cardiac electrical activity.                */
/*                                                                      */
/* Nico Kuijpers, August 23, 2000                                       */
/* Eindhoven University of Technology                                   */
/*                                                                      */
/* Last update: May 10, 2007                                            */
/************************************************************************/


/*----- General purpose tokens -----*/
#define T_PLUS                   0
#define T_MINUS                  1
#define T_TIMES                  2
#define T_SLASH                  3
#define T_ARROW                  4
#define T_COMMA                  5
#define T_LPAR                   6
#define T_RPAR                   7
#define T_LBRACE                 8
#define T_RBRACE                 9
#define T_PERIOD                10
#define T_COLON                 11
#define T_INTEGER               12
#define T_NUMBER                13
#define T_STRING                14
#define T_SAME                  15
#define T_GARBAGE               16
#define T_EOF                   17

/*----- Tokens specific for cardiac electrical activity -----*/
#define T_TISSUE                18
#define T_TISSUE_TYPE           19
#define T_TISSUE_DIMENSIONS     20
#define T_TISSUE_CONFIG         21
#define T_NODE_DIMENSIONS       22
#define T_EDGE_DIMENSIONS       23
#define T_RANDOMSEED            24
#define T_LENGTH                25
#define T_WIDTH                 26
#define T_HEIGHT                27
#define T_RADIUS                28
#define T_MINIMUM               29
#define T_MAXIMUM               30
#define T_AVERAGE               31
#define T_VARIANCE              32
#define T_STIMNODESA            33
#define T_STIMNODESB            34
#define T_STIMNODESC            35
#define T_SEMIDEADNODES         36
#define T_FULLDEADNODES         37
#define T_REFRACNODES           38
#define T_REENTRYNODES          39
#define T_MONITORNODES          40
#define T_SINUSNODES            41
#define T_ACCURATENODES         42
#define T_ANODALNODES           43
#define T_CATHODALNODES         44
#define T_MEMBRANECAPACITANCE   45
#define T_MEMBRANERATIO         46
#define T_LONGINTCONDUCT        47
#define T_TRANSINTCONDUCT       48
#define T_TRANSMURINTCONDUCT    49
#define T_LONGEXTCONDUCT        50
#define T_TRANSEXTCONDUCT       51
#define T_TRANSMUREXTCONDUCT    52
#define T_GAPJUNCRATIO          53
#define T_REFRACPERIOD          54
#define T_MEMPOTREST            55
#define T_INTPOTREST            56
#define T_EXTPOTREST            57
#define T_LAYER                 58

#endif
