#-------------------------------------------------
#
# This file is part of the Peacs support package
#
# Copyright (C) 2010 - 2012 Peacs (www.peacs.nl)
#
# The Peacs Support package is free software: you can redistribute it
# and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# The Peacs support classes are distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with the Peacs support package.  If not, see
# <http://www.gnu.org/licenses/>.
#
# Project:             PeacsSupport
# Qt version:          4.7.x
# Compiler:            MinGW, gcc 4.4.0
# Target platforms:    Windows, Mac OS-X
#
# This file can be used to include the source of this project into an
# application.
# Customization must be used to include only those files that are needed
# for the resulting project
#
# Subversion info:
#
# $Id: PeacsSupport.pri 1182 2014-03-25 12:59:03Z eelco $
#
#-------------------------------------------------


# Default a source library structure is assumed to be used. This setting can be overruled by specify one of the
# following CONFIG options to qmake:
#
# * PS_STATIC_LIB   - use a static library
# * PS_DYNAMIC_LIB  - use a dynamic library
#
# When more options are specified the PS_LIBRARY overrules all
#
# The option PS_LIBRARY only influences the install build fase
#
#

# Include the file with the version information
!include(PeacsSupportVersion.pri):warning( "Peacs support build version information not found" )

CONFIG(PS_USE_HELP) {
    message( "Help classes will be included" )
    CONFIG += help
}

CONFIG(PS_USE_OPENGL) {
    QT += opengl    # Make sure that openGL support for Qt is enabled
}

QT += xml           # Make sure that openGL support for Qt is enabled for PXmlFile

# Don't add the sources to the PeacsSupport project, because that project has got its own
# project files
!contains( TARGET, PeacsSupport ) {
    # Try to get the path to the PeacsSupport classes from the system environment
    exists( $$(PEACS_SUPPORT_PATH)/file/*.h ) {
        PEACSSUPPORTPATH = $$(PEACS_SUPPORT_PATH)
    }

    isEmpty( PEACSSUPPORTPATH ) {
        # Try to find the sources at the specified location reletive to the current project location
        exists( $${_PRO_FILE_PWD_}/PeacsSupport/file/*.h ) {
            PEACSSUPPORTPATH = ./PeacsSupport
        }
    }

    isEmpty( PEACSSUPPORTPATH ) {
        # Try to find the sources at the specified location reletive to the location of the current file being parsed
        exists( $${PWD}/file/*.h ) {
            PEACSSUPPORTPATH = $$PWD
        }
    }

    !isEmpty( PEACSSUPPORTPATH ) {
        # PeacsSupport classes found. Now we can use them

        SUPPORT_TARGET_NAME = PeacsSupport
        CONFIG(PS_STATIC_LIB):SUPPORT_TARGET_NAME = $${SUPPORT_TARGET_NAME}_s           # Static library
        CONFIG(debug, debug|release):SUPPORT_TARGET_NAME = $${SUPPORT_TARGET_NAME}_d    # Debug with "_d"

        # A static library must be used
        CONFIG(PS_STATIC_LIB) {
            DEFINES += PEACSSUPPORT_LIBRARY_STATIC  # Don't declare any DLL export/import directive

            LIBS += $${PEACSSUPPORTPATH}/lib/$${QMAKE_PREFIX_STATICLIB}$${SUPPORT_TARGET_NAME}.$${QMAKE_EXTENSION_STATICLIB}

        } else {
            # A dynamic library must be used
            CONFIG(PS_DYNAMIC_LIB) {

                LIBS += $${PEACSSUPPORTPATH}/lib/$${QMAKE_PREFIX_STATICLIB}$${SUPPORT_TARGET_NAME}.$${QMAKE_EXTENSION_STATICLIB}
                DEFINES += PEACSSUPPORT_LIBRARY_STATIC  # Don't declare any DLL export/import directive

            # No library will be used, just the sources directly
            } else {

                # Don't declare any DLL export/import directive
                DEFINES += PEACSSUPPORT_LIBRARY_STATIC

                # Add the sources
                SOURCES += \
                    $${PEACSSUPPORTPATH}/PeacsSupport_global.cpp \
                    $${PEACSSUPPORTPATH}/support/pvector.cpp \
                    $${PEACSSUPPORTPATH}/support/PStatistics.cpp \
                    $${PEACSSUPPORTPATH}/support/pplane.cpp \
                    $${PEACSSUPPORTPATH}/support/pmatrix.cpp \
                    $${PEACSSUPPORTPATH}/support/PIVector.cpp \
                    $${PEACSSUPPORTPATH}/support/pgraphgeometry.cpp \
                    $${PEACSSUPPORTPATH}/support/pgeometry.cpp \
                    $${PEACSSUPPORTPATH}/support/pfunctions.cpp \
                    $${PEACSSUPPORTPATH}/support/pface.cpp \
                    $${PEACSSUPPORTPATH}/support/pedge.cpp \
                    $${PEACSSUPPORTPATH}/support/pCoordinateSystem.cpp \
                    $${PEACSSUPPORTPATH}/support/pbox.cpp \
                    $${PEACSSUPPORTPATH}/support/P3DVertices.cpp \
                    $${PEACSSUPPORTPATH}/support/p3dvector.cpp \
                    $${PEACSSUPPORTPATH}/support/PSettings.cpp \
                    $${PEACSSUPPORTPATH}/file/pfile.cpp \
                    $${PEACSSUPPORTPATH}/file/pxmlfile.cpp

                CONFIG(PS_USE_SIGNAL_WIDGETS) {
                    SOURCES += \
                        $${PEACSSUPPORTPATH}/widgets/PGraphicsSignalView.cpp \
                        $${PEACSSUPPORTPATH}/widgets/PGrSceneBase.cpp \
                        $${PEACSSUPPORTPATH}/widgets/PGraphMatrixScene.cpp \
                        $${PEACSSUPPORTPATH}/widgets/PGrItemBase.cpp \
                        $${PEACSSUPPORTPATH}/widgets/PSignalData.cpp \
                        $${PEACSSUPPORTPATH}/widgets/PSignalsData.cpp \
                        $${PEACSSUPPORTPATH}/widgets/PSignalPlotBase.cpp \
                        $${PEACSSUPPORTPATH}/widgets/PSignalPlot.cpp \
                        $${PEACSSUPPORTPATH}/widgets/PSignalDraw.cpp \
                        $${PEACSSUPPORTPATH}/widgets/PSimpleTextItem.cpp \
                        $${PEACSSUPPORTPATH}/widgets/PSignalMarker.cpp \
                        $${PEACSSUPPORTPATH}/widgets/PMarkerLine.cpp \
                        $${PEACSSUPPORTPATH}/widgets/PMarkerEllipse.cpp \
                        $${PEACSSUPPORTPATH}/widgets/PSignalMarkers.cpp \
                        $${PEACSSUPPORTPATH}/widgets/PWidgetDefines.cpp \
                        $${PEACSSUPPORTPATH}/widgets/PXYPlot.cpp
                }
            }
        }

        INCLUDEPATH += \
            $${PEACSSUPPORTPATH} \
            $${PEACSSUPPORTPATH}/file/ \
            $${PEACSSUPPORTPATH}/support/

            CONFIG(PS_USE_SIGNAL_WIDGETS) {
                INCLUDEPATH += \
                    $${PEACSSUPPORTPATH}/widgets/
            }

        HEADERS += \
            $${PEACSSUPPORTPATH}/PeacsSupport_global.h \
            $${PEACSSUPPORTPATH}/support/pvector.h \
            $${PEACSSUPPORTPATH}/support/PStatistics.h \
            $${PEACSSUPPORTPATH}/support/pplane.h \
            $${PEACSSUPPORTPATH}/support/pmatrix.h \
            $${PEACSSUPPORTPATH}/support/PIVector.h \
            $${PEACSSUPPORTPATH}/support/pgraphgeometry.h \
            $${PEACSSUPPORTPATH}/support/pgeometry.h \
            $${PEACSSUPPORTPATH}/support/pfunctions.h \
            $${PEACSSUPPORTPATH}/support/pface.h \
            $${PEACSSUPPORTPATH}/support/pedge.h \
            $${PEACSSUPPORTPATH}/support/pCoordinateSystem.h \
            $${PEACSSUPPORTPATH}/support/pbox.h \
            $${PEACSSUPPORTPATH}/support/P3DVertices.h \
            $${PEACSSUPPORTPATH}/support/P3DVector.h \
            $${PEACSSUPPORTPATH}/support/PSettings.h \
            $${PEACSSUPPORTPATH}/file/pfile.h \
            $${PEACSSUPPORTPATH}/file/pxmlfile.h

        CONFIG(PS_USE_SIGNAL_WIDGETS) {
            HEADERS += \
                $${PEACSSUPPORTPATH}/widgets/PGraphicsSignalView.h \
                $${PEACSSUPPORTPATH}/widgets/PGrSceneBase.h \
                $${PEACSSUPPORTPATH}/widgets/PGraphMatrixScene.h \
                $${PEACSSUPPORTPATH}/widgets/PGrItemBase.h \
                $${PEACSSUPPORTPATH}/widgets/PSignalData.h \
                $${PEACSSUPPORTPATH}/widgets/PSignalsData.h \
                $${PEACSSUPPORTPATH}/widgets/PSignalPlotBase.h \
                $${PEACSSUPPORTPATH}/widgets/PSignalPlot.h \
                $${PEACSSUPPORTPATH}/widgets/PSignalDraw.h \
                $${PEACSSUPPORTPATH}/widgets/PSimpleTextItem.h \
                $${PEACSSUPPORTPATH}/widgets/PSignalMarker.h \
                $${PEACSSUPPORTPATH}/widgets/PMarkerLine.h \
                $${PEACSSUPPORTPATH}/widgets/PMarkerEllipse.h \
                $${PEACSSUPPORTPATH}/widgets/PSignalMarkers.h \
                $${PEACSSUPPORTPATH}/widgets/PWidgetDefines.h \
                $${PEACSSUPPORTPATH}/widgets/PXYPlot.h
        }
    } else{
        error( "No 'Peacs Support' files found. Add the path to the library to the 'PEACS_SUPPORT_PATH' environment variable." )
    }
}
