/*
 *  This file is part of the Peacs support package
 *
 *  Copyright (C) 2010 - 2012 Peacs (www.peacs.nl)
 *
 *  The Peacs Support package is free software: you can redistribute it
 *  and/or modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  The Peacs support classes are distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the Peacs support package.  If not, see
 *  <http://www.gnu.org/licenses/>.
 *
 *    Project:              Peacs Support
 *    Qt version:           4.7.x
 *    Compiler:             QMake, MinGW, gcc 4.4.0
 *    Target platforms:     Windows, Mac OS-X
 *
 *    Subversion info:
 *
 *    $Id: pfile.cpp 415 2012-02-26 20:49:06Z eelco $
 *
 */

#include "pfile.h"
#include <QDataStream>


using namespace Peacs::Support;


PFile::PFile( void ) :
    mLoadedClassName()
{
}


PFile::PFile( const PFile& source ) :
    mLoadedClassName( source.mLoadedClassName )
{
}


const PFile& PFile::operator=( const PFile& source )
{
    if ( this != &source )
    {
        mLoadedClassName = source.mLoadedClassName;
    }

    return *this;
}


int PFile::readHead( QDataStream& stream, const QString &className  )
{
    QString name    = className;
    qint32  version = (qint32)READ_HEADER_FAILED;

    if ( className.isEmpty() )
    {
        name = getClassName();
    }

    if ( isOk( stream ) )
    {
        stream >> mLoadedClassName;

        if ( isOk( stream ) )
        {
            if ( mLoadedClassName.isEmpty( ) )
            {
                // Sometimes there was a misterius space that made the reading
                // fail. Retry the action was the solution
                stream >> mLoadedClassName;
            }

            if ( isOk( stream ) && mLoadedClassName == name )
            {
                qint32  tmpVer  = (qint32)READ_HEADER_FAILED;

                stream >> tmpVer;

                if ( isOk( stream ) )
                {
                    // Check if all read information is correct for this class
                    version = tmpVer;
                }
            }
        }
    }

    return (int)version;
}


bool PFile::writeHead( QDataStream& stream , const QString &className, int versionNr ) const
{
    QString name = className;

    if ( className.isEmpty() )
    {
        name = getClassName();
    }

    if ( versionNr == USE_DEFUALT_VERSION )
    {
        versionNr = getDefaultVersion();
    }

    stream << name;

    if ( isOk( stream ) )
    {
        // Always save the case under the latest release version
        stream << ( qint32 ) versionNr;
    }

    return isOk( stream );
}


const QString& PFile::getLoadedClassName( void ) const
{
    return mLoadedClassName;
}
