/*
 *  This file is part of the Peacs support package
 *
 *  Copyright (C) 2010 - 2012 Peacs (www.peacs.nl)
 *
 *  The Peacs Support package is free software: you can redistribute it
 *  and/or modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  The Peacs support classes are distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the Peacs support package.  If not, see
 *  <http://www.gnu.org/licenses/>.
 *
 *    Project:              Peacs Support
 *    Creation date:
 *    Qt version:           4.6.3
 *    Qt creator:           2.0.0
 *    Compiler:             MinGW, gcc 4.4.0
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: pxmlfile.cpp 1092 2014-02-16 21:22:55Z peter $
 *
 */


#include <QDebug>
#include "pxmlfile.h"

const char * XML_NAMEATTR       = "name";
const char * XML_VERSIONATTR    = "version";
const char * XML_DATEATTR       = "date";



using namespace Peacs::Support;

PXmlFile::PXmlFile( )
{

}

PXmlFile::PXmlFile(const PXmlFile &/*xmlFile*/)
{

}

QDomNode PXmlFile::createXmlEntry(QDomNode &parentNode, const QString &tagName) const
{

    QDomElement idElement(parentNode.ownerDocument().createElement(tagName));
    return parentNode.appendChild(idElement);
}


QDomNode PXmlFile::createXmlEntry(QDomNode &parentNode, const QString &tagName, int versionNr) const
{

    QDomElement idElement(parentNode.ownerDocument().createElement(tagName));
    idElement.setAttribute(XML_VERSIONATTR,versionNr);
    return parentNode.appendChild(idElement);
}


QDomNode PXmlFile::createXmlEntry(QDomNode &parentNode, const QString &tagName, int versionNr, const QString &name) const
{
    QDomElement idElement(parentNode.ownerDocument().createElement(tagName));
    idElement.setAttribute(XML_NAMEATTR,name);
    idElement.setAttribute(XML_VERSIONATTR,versionNr);
    return parentNode.appendChild(idElement);
}


bool PXmlFile::addValue(QDomNode &parentNode, const QString &tagName, const QString &value) const
{
    QDomNode node;
    node = parentNode.appendChild( parentNode.ownerDocument().createElement( tagName ) );
    node.appendChild( node.ownerDocument().createTextNode( value ) );
    return !node.isNull();
}


bool PXmlFile::addValue(QDomNode &parentNode, const QString &tagName, double value) const
{
    QDomNode node;
    node = parentNode.appendChild( parentNode.ownerDocument().createElement( tagName ) );
    node.appendChild( node.ownerDocument().createTextNode( QString::number(value) ) );
    return !node.isNull();
}

bool PXmlFile::addValue(QDomNode &parentNode, const QString &tagName, int value) const
{
    QDomNode node;
    node = parentNode.appendChild( parentNode.ownerDocument().createElement( tagName ) );
    node.appendChild( node.ownerDocument().createTextNode( QString::number(value) ) );
    return !node.isNull();
}

bool PXmlFile::addValue(QDomNode &parentNode, const QString &tagName, const PIVector &value) const
{
    QDomNode node;
    node = parentNode.appendChild( parentNode.ownerDocument().createElement( tagName ) );
    QString str;
    if ( !value.isEmpty() )
    {
        for (int i=0; i < value.size(); i++)
        {
            str += QString::number(value.at(i));
            if ( i < value.size() - 1)
            {
                str +=  ", ";
            }
        }
    }
    node.appendChild( node.ownerDocument().createTextNode( str ) );
    return !node.isNull();
}

bool PXmlFile::addValue(QDomNode &parentNode, const QString &tagName, const PVector &value) const
{
    QDomNode node;
    node = parentNode.appendChild( parentNode.ownerDocument().createElement( tagName ) );
    QString str;
    if ( !value.isEmpty() )
    {
        for (int i=0; i < value.size(); i++)
        {
            str += QString::number(value.at(i));
            if ( i < value.size() - 1)
            {
                str +=  ", ";
            }
        }
    }
    node.appendChild( node.ownerDocument().createTextNode( str ) );
    return !node.isNull();
}



int PXmlFile::readInt(const QDomNode &node, const QString &tagName, int defaultValue) const
{
    QDomNodeList elements = node.toElement().elementsByTagName(tagName);
    int value = defaultValue;
    bool ok;
    if ( elements.size() == 1 )
    {
         value = elements.at(0).toElement().text().toInt(&ok);
         if ( !ok )
             value = defaultValue;
    }
    else
    {
        qDebug().nospace() << "PXmlFile::readvalue error: could not find field with tagname " << tagName;
    }
    return value;
}


bool PXmlFile::readBool(const QDomNode &node, const QString &tagName, bool defaultValue) const
{
    QDomNodeList elements = node.toElement().elementsByTagName(tagName);
    bool value = defaultValue;
    bool ok;
    if ( elements.size() == 1 )
    {
         int val = elements.at(0).toElement().text().toInt(&ok);
         if ( !ok )
         {
             value = defaultValue;
         }
         else
         {
             value = val != 0;
         }
    }
    else
    {
        qDebug().nospace() << "PXmlFile::readvalue error: could not find field with tagname " << tagName;
    }
    return value;
}


double PXmlFile::readValue(const QDomNode &node, const QString &tagName, double defaultValue) const
{
    QDomNodeList elements = node.toElement().elementsByTagName(tagName);
    double value = defaultValue;
    bool ok;
    if ( elements.size() == 1 )
    {
         value = elements.at(0).toElement().text().toDouble(&ok);
         if ( !ok )
             value = defaultValue;
    }
    else
    {
        qDebug().nospace() << "PXmlFile::readvalue error: could not find field with tagname " << tagName << " nr nodes " << elements.size();
    }
    return value;
}

double PXmlFile::readValue(QDomNode &node, const QString &tagName, double defaultValue) const
{
    QDomNodeList elements = node.toElement().elementsByTagName(tagName);
    double value = defaultValue;
    bool ok;
    if ( elements.size() == 1 )
    {
         value = elements.at(0).toElement().text().toDouble(&ok);
         if ( !ok )
             value = defaultValue;
    }
    else
    {
        qDebug().nospace() << "PXmlFile::readvalue error: could not find field with tagname " << tagName << " nr nodes " << elements.size();
    }
    return value;
}


QString PXmlFile::readString(const QDomNode &node, const QString &tagName, const QString &defaultValue) const
{
    QDomNodeList elements = node.toElement().elementsByTagName(tagName);
    QString value = defaultValue;
    if ( elements.size() == 1 )
    {
         value = elements.at(0).toElement().text();
    }
    else
    {
        qDebug().nospace() << "PXmlFile::readvalue error: could not find field with tagname " << tagName;
    }
    return value;
}

PIVector PXmlFile::readIVector(const QDomNode &node, const QString &tagName, int defaultSize) const
{
    QDomNodeList elements = node.toElement().elementsByTagName(tagName);
    PIVector vector(defaultSize);
    if ( elements.size() == 1 )
    {
         QString value = elements.at(0).toElement().text();
         QStringList strList(value.split(','));
         vector.resize(strList.size());
         bool ok;
         for (int i=0; i < strList.size(); i++)
         {
             int a= strList[i].toInt(&ok);
             if (ok)
             {
                vector[i] = a;
             }
         }
    }
    else
    {
        qDebug().nospace() << "PXmlFile::readvalues error: could not find field with tagname " << tagName;
    }
    return vector;
}


PVector PXmlFile::readVector(const QDomNode &node, const QString &tagName, int defaultSize) const
{
    QDomNodeList elements = node.toElement().elementsByTagName(tagName);
    PVector vector(defaultSize);
    if ( elements.size() == 1 )
    {
         QString value = elements.at(0).toElement().text();
         QStringList strList(value.split(','));
         vector.resize(strList.size());
         bool ok;
         for (int i=0; i < strList.size(); i++)
         {
             double a= strList[i].toDouble(&ok);
             if ( ok )
             {
                vector[i] = a;
             }
         }
    }
    else
    {
        qDebug().nospace() << "PXmlFile::readvalues error: could not find field with tagname " << tagName;
    }
    return vector;
}
QString PXmlFile::getNodeName(const QDomNode &node) const
{
    return node.toElement().attribute(XML_NAMEATTR);
}

int PXmlFile::getNodeVersion(const QDomNode &node) const
{
    QString str = node.toElement().attribute(XML_VERSIONATTR);
    bool ok;
    int version = str.toInt(&ok);
    return version;
}
