/*
 *  This file is part of the Peacs support package
 *
 *  Copyright (C) 2010 - 2012 Peacs (www.peacs.nl)
 *
 *  The Peacs Support package is free software: you can redistribute it
 *  and/or modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  The Peacs support classes are distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the Peacs support package.  If not, see
 *  <http://www.gnu.org/licenses/>.
 *
 *    Project:              Peacs Support
 *    Creation date:
 *    Qt version:           4.6.3
 *    Qt creator:           2.0.0
 *    Compiler:             MinGW, gcc 4.4.0
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: pxmlfile.h 970 2013-12-25 16:44:23Z peter $
 *
 */



#ifndef _PXMLFILE_H
#define _PXMLFILE_H

#include <QDomNode>
#include <QString>
#include <QStringList>
#include <QXmlAttributes>
#include <QDomDocument>
#include "PeacsSupport_global.h"
#include "PIVector.h"
#include "pvector.h"

namespace Peacs
{
    namespace Support
    {

        /** \class PXmlFile pfile.h "file/pfile.h"
            @brief Mainly a virtual function class
         */

        class PEACSSUPPORTSHARED_EXPORT PXmlFile
        {
        protected:

            /** @brief flag that indicated that the read file (class) header is wrong or was failed to read
              * used in readHead(...)
              */
            static const int READ_HEADER_FAILED = -1;

            /** @brief Constructor used by inherted classes.
             */
            PXmlFile();

            /** @brief Constructor used by inherted classes.
             */
            PXmlFile(const PXmlFile & xmlFile);


            /** @brief destructor */
            virtual ~PXmlFile() {}

            virtual int getDefaultVersion(void) const = 0;

            /** @brief Returns the class name */
            virtual const char *getClassName(void) const = 0;

            /** @brief Create a Xml entry
             *       function called when writimg class info to the XML
             *  @param parentNode  The node to which the new nodes is added
             *  @param tagName the name of the tag in the XML file
             *  @return the new node added to the parent one
             */
            QDomNode createXmlEntry(QDomNode &parentNode,const QString &tagName) const;

            /** @brief Create a Xml entry
             *       function called when writimg class info to the XML
             *  @param parentNode  The node to which the new nodes is added
             *  @param tagName the name of the tag in the XML file
             *  @param versionNr The version number is added as an attribute
             *  @return the new node added to the parent one
             */
            QDomNode createXmlEntry(QDomNode &parentNode,const QString &tagName , int versionNr) const;

            /** @brief Create a Xml entry
             *       function called when writimg class info to the XML
             *  @param parentNode  The node to which the new nodes is added
             *  @param tagName the name of the tag in the XML file
             *  @param versionNr The version number is added as an attribute
             *  @param name The name is added as an attribute
             *  @return the new node added to the parent one
             */
            QDomNode createXmlEntry(QDomNode &parentNode,const QString &tagName , int versionNr, const QString &name) const;

            /** @brief add an element with value
             *  @param parentNode  The node to which a new element with the value is added
             *  @param tagName the name of the tag in the XML file
             *  @param value the value of the tag field
             *  @return true if is succesfull added
             */
            bool addValue(QDomNode &parentNode, const QString &tagName ,double value) const;

            /** @brief add an element with value
             *  @param parentNode  The node to which a new element with the value is added
             *  @param tagName the name of the tag in the XML file
             *  @param value the value of the tag field
             *  @return true if is succesfull added
             */
            bool addValue(QDomNode &parentNode, const QString &tagName ,int value) const;

            /** @brief add an element with value
             *  @param parentNode  The node to which a new element with the value is added
             *  @param tagName the name of the tag in the XML file
             *  @param value the value of the tag field
             *  @return true if is succesfull added
             */
            bool addValue(QDomNode &parentNode, const QString &tagName ,const QString &value) const;


            /** @brief add an element with value
             *  @param parentNode  The node to which a new element with the value is added
             *  @param tagName the name of the tag in the XML file
             *  @param value the value of the tag field
             *  @return true if is succesfull added
             */
            bool addValue(QDomNode &parentNode, const QString &tagName ,const PIVector &value) const;


            /** @brief add an element with value
             *  @param parentNode  The node to which a new element with the value is added
             *  @param tagName the name of the tag in the XML file
             *  @param value the value of the tag field
             *  @return true if is succesfull added
             */
            bool addValue(QDomNode &parentNode, const QString &tagName ,const PVector &value) const;


            /** @brief read an integer from the siblings node with tagname
             *  @param node  The parent node from which the value is searched
             *  @param tagName the name of the tag in the XML file
             *  @param defaultValue used in case the field is not found
             *  @return the value read from the node
             */
            int readInt(const QDomNode &node, const QString &tagName,int defaultValue = 0) const;

            /** @brief read an integer from the siblings node with tagname
             *  @param node  The parent node from which the value is searched
             *  @param tagName the name of the tag in the XML file
             *  @param defaultValue used in case the field is not found
             *  @return the value read from the node
             */
            bool readBool(const QDomNode &node, const QString &tagName,bool defaultValue = false) const;


            /** @brief read an double from the siblings node with tagname
             *  @param node  The parent node from which the value is searched
             *  @param tagName the name of the tag in the XML file
             *  @param defaultValue used in case the field is not found
             *  @return the value read from the node
             */
            double readValue(const QDomNode &node, const QString &tagName,double defaultValue=0.0) const;


            /** @brief read an double from the siblings node with tagname
             *  @param node  The parent node from which the value is searched
             *  @param tagName the name of the tag in the XML file
             *  @param defaultValue used in case the field is not found
             *  @return the value read from the node
             */
            double readValue(QDomNode &node, const QString &tagName,double defaultValue=0.0) const;


            /** @brief read an double from the siblings node with tagname
             *  @param node  The parent node from which the value is searched
             *  @param tagName the name of the tag in the XML file
             *  @param defaultValue used in case the field is not found
             *  @return the value read from the node
             */
            PIVector readIVector(const QDomNode &node, const QString &tagName,int defaultSize = 0) const;

            /** @brief read an double from the siblings node with tagname
             *  @param node  The parent node from which the value is searched
             *  @param tagName the name of the tag in the XML file
             *  @param defaultValue used in case the field is not found
             *  @return the value read from the node
             */
            PVector readVector(const QDomNode &node, const QString &tagName,int defaultSize = 0) const;


            /** @brief read an integer from the siblings node with tagname
             *  @param node  The parent node from which the value is searched
             *  @param tagName the name of the tag in the XML file
             *  @param defaultValue used in case the field is not found
             *  @return the value read from the node
             */
            QString readString(const QDomNode &node, const QString &tagName,const QString &defaultValue=QString()) const;


            QString getNodeName( const QDomNode &node ) const;

            int getNodeVersion( const QDomNode &node ) const;


        public:
            //----------------------- File stream functions -------------------
            /** @brief Pure virtual function Read all avaliable for a specific
             * class from a file. Before calling this function the readHead(...)
             * function must have been called or must be the first function
             * called when reading from stream
             *
             *  @see bool readHead(QTextStream& stream);
             *  @see virtual bool write(QDatastream &ds)const =0;
             *  @param stream the (file) data stream to read from
             */
            virtual bool readXml(const QDomNode &node) = 0;

            /** @brief Pure virtual function write all avaliable class data
             *      to a file. This function must be called
             *      before the write(...) function is called or must be
             *      the first function called when writimg to stream
             *  @see virtual bool read(QDatastream &ds)=0;
             *  @param stream the (file) data stream to write to
             */
            virtual bool writeXml(QDomNode &parentNode)const = 0;


            //------------------------ XML support functions -------------------

        public:

            /**
             * @brief Checks the status of the stream for neing OK
             * @param extraOk extra flag to check and use in result
             * @param stream the stream to check and return the new flag value
             * @return true if the status of the stream is OK and the extraOk
             * flag is true, otherwise false
             */
            inline bool isOk(bool& extraOk, const QTextStream& stream) const {
                return extraOk = extraOk && (stream.status() == QTextStream::Ok);
            }

            /**
             * @brief Checks the status of the stream for neing OK
             * @param stream the stream to check
             * @return true if the status of the stream is OK, otherwise false
             */
            inline bool isOk(const QTextStream& stream) const {
                return stream.status() == QTextStream::Ok;
            }
        };
    }
}

#endif // _PFILE_H


