/*
 *  This file is part of the Peacs support package
 *
 *  Copyright (C) 2010 - 2012 Peacs (www.peacs.nl)
 *
 *  The Peacs Support package is free software: you can redistribute it
 *  and/or modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  The Peacs support classes are distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the Peacs support package.  If not, see
 *  <http://www.gnu.org/licenses/>.
 *
 *    Project:              Peacs Support
 *    Qt version:           4.7.x
 *    Compiler:             QMake, MinGW, gcc 4.4.0
 *    Target platforms:     Windows, Mac OS-X
 *
 *    Subversion info:
 *
 *    $Id: PStatistics.cpp 616 2013-02-04 19:20:57Z peter $
 *
 */


#include "PStatistics.h"
#include <QList>


using namespace Peacs::Support;


PStatistics::PStatistics( const PVector &v, doStatisticsTypes doStatistics ) :
    mMin( 0 ),
    mMax( 0 ),
    mMinI( 0 ),
    mMaxI( 0 ),
    mMean( 0 ),
    mStd( 0 ),
    mVec( v )
{
    compStatistics( doStatistics );
}


void PStatistics::compStatistics( doStatisticsTypes doStatistics )
{
    if ( !mVec.isEmpty() )
    {
        const int vecSize = mVec.N();

        if ( doStatistics & (doMin | doMax | doMean | doStd | doSum) )
        {
            int             i;
            const double*    vecData = mVec.data();      // Get the raw data array to work with
            double          mean    = (double)vecData[0];
            double          sqSum   = 0;
            double           tmp;

            if ( doStatistics & doMin )
            {
                mMin    = vecData[0];
                mMinI   = 0;
            }

            if ( doStatistics & doMax )
            {
                mMax    = vecData[0];
                mMaxI   = 0;
            }

            for ( i = 1; i < vecSize; i++ )
            {
                mean += vecData[i];

                if ( (doStatistics & doMin) && (vecData[i] < mMin) )
                {
                    mMin    = vecData[i];
                    mMinI   = i;
                }

                if ( (doStatistics & doMax) && (mMax < vecData[i]) )
                {
                    mMaxI   = i;
                    mMax    = vecData[i];
                }
            }

            if ( doStatistics & (doMean | doStd) )
            {
                mMean = mean / (double)vecSize;
            }

            if ( doStatistics & doStd )
            {
                for ( i = 0; i < vecSize; i++ )
                {
                    tmp     = (vecData[i] - mMean);
                    sqSum   += (tmp * tmp);
                }

                mStd = (double)sqrt( sqSum / ((double)vecSize) );
            }

            if ( doStatistics & doSum )
            {
                mSum = mean;
            }
        }

        if ( doStatistics & doMedian )
        {
            PVector median( mVec );

            median.sort();

            if ( vecSize % 2 )
            {
                mMedian = median[vecSize / 2];
            }
            else
            {
                mMedian = (median[vecSize / 2 - 1] + median[vecSize / 2]) / 2;
            }
        }
    }
    else
    {
        mMean   = 0;
        mMedian = 0;
        mStd    = 0;
        mMin    = 0;
        mMax    = 0;
        mSum    = 0;
        mMaxI   = -1;
        mMinI   = -1;
    }
}
