/*
 *  This file is part of the Peacs support package
 *
 *  Copyright (C) 2010 - 2012 Peacs (www.peacs.nl)
 *
 *  The Peacs Support package is free software: you can redistribute it
 *  and/or modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  The Peacs support classes are distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the Peacs support package.  If not, see
 *  <http://www.gnu.org/licenses/>.
 *
 *    Project:              Peacs Support
 *    Qt version:           4.7.x
 *    Compiler:             QMake, MinGW, gcc 4.4.0
 *    Target platforms:     Windows, Mac OS-X
 *
 *    Subversion info:
 *
 *    $Id: pbox.cpp 1196 2014-04-15 10:20:27Z eelco $
 *
 */


#include "pbox.h"
#include "pfunctions.h"


using namespace Peacs::Support;


/*PBox::PBox() :
    mVertices( P3DVertices() ),
    mMaxBox(),
    mMinBox()
{
}
*/

PBox::PBox( const P3DVertices & vertices ) :
    mVertices( vertices ),
    mMaxBox(),
    mMinBox()
{
    update();
}


void PBox::update()
{
    if ( !mVertices.isEmpty() )
    {
        mMaxBox = mVertices.at( 0 );
        mMinBox = mVertices.at( 0 );

        for ( int i = 1; i < mVertices.size(); i++ )
        {
            mMaxBox = pMax( mMaxBox, mVertices.at( i ) );
            mMinBox = pMin( mMinBox, mVertices.at( i ) );
        }
    }
    else
    {
        mMaxBox.toZeros();
        mMinBox.toZeros();
    }
}


bool PBox::insideBox(const P3DVector &pos) const
{
    return ( pos.x() >= mMinBox.x() && pos.x() <= mMaxBox.x() &&
             pos.y() >= mMinBox.y() && pos.y() <= mMaxBox.y() &&
             pos.z() >= mMinBox.z() && pos.z() <= mMaxBox.z() );
}
