/*
 *  This file is part of the Peacs support package
 *
 *  Copyright (C) 2010 - 2012 Peacs (www.peacs.nl)
 *
 *  The Peacs Support package is free software: you can redistribute it
 *  and/or modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  The Peacs support classes are distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the Peacs support package.  If not, see
 *  <http://www.gnu.org/licenses/>.
 *
 *    Project:              Peacs Support
 *    Qt version:           4.7.x
 *    Compiler:             QMake, MinGW, gcc 4.4.0
 *    Target platforms:     Windows, Mac OS-X
 *
 *    Subversion info:
 *
 *    $Id: pedge.cpp 415 2012-02-26 20:49:06Z eelco $
 *
 */


#include "pedge.h"


using namespace Peacs::Support;


//=== Constructors ============================================================

PEdge::PEdge()
{
    mEdge[A] = 0;
    mEdge[B] = 0;
}


PEdge::PEdge(int a,int b)
{
    mEdge[A] = a;
    mEdge[B] = b;
}


PEdge::PEdge(const PEdge &edge)
{
    mEdge[A] = edge[A];
    mEdge[B] = edge[B];
}


//=== Operators ==============================================================

const PEdge &PEdge::operator =(const PEdge& edge)
{
    if ( this != &edge )
    {
        mEdge[A] = edge.mEdge[A];
        mEdge[B] = edge.mEdge[B];
    }

    return *this;
}


const PEdge &PEdge::invert(const PEdge& edge)
{
    mEdge[A] = edge.mEdge[B];
    mEdge[B] = edge.mEdge[A];

    return *this;
}

const PEdge &PEdge::invert(void)
{
    int a = mEdge[A];

    mEdge[A] = mEdge[B];
    mEdge[B] = a;

    return *this;
}


PEdge PEdge::inverted(void) const
{
    return PEdge( mEdge[B], mEdge[A] );
}


bool PEdge::operator ==(const PEdge& edge) const
{
    return ( mEdge[A] == edge.mEdge[A] &&
             mEdge[B] == edge.mEdge[B] );
}


bool PEdge::operator !=(const PEdge& edge) const
{
    return ( mEdge[A] != edge.mEdge[A] ||
             mEdge[B] != edge.mEdge[B] );
}


//=== Access Functions =======================================================

#ifndef QT_NO_DEBUG

int & PEdge::operator[] ( edges_t index )
{
    Q_ASSERT( index == PEdge::A || index == PEdge::B );     // Test just to be sure

    return mEdge[index];
}


const int & PEdge::operator[] ( edges_t index ) const
{
    Q_ASSERT( index == PEdge::A || index == PEdge::B );     // Test just to be sure

    return mEdge[index];
}

#endif
