/*
 *  This file is part of the Peacs support package
 *
 *  Copyright (C) 2010 - 2012 Peacs (www.peacs.nl)
 *
 *  The Peacs Support package is free software: you can redistribute it
 *  and/or modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  The Peacs support classes are distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the Peacs support package.  If not, see
 *  <http://www.gnu.org/licenses/>.
 *
 *    Project:              Peacs Support
 *    Qt version:           4.7.x
 *    Compiler:             QMake, MinGW, gcc 4.4.0
 *    Target platforms:     Windows, Mac OS-X
 *
 *    Subversion info:
 *
 *    $Id: PMarkerEllipse.cpp 967 2013-12-23 19:22:31Z eelco $
 *
 */

#include "PMarkerEllipse.h"

#include <math.h>


using namespace Peacs::Support;


PMarkerEllipse::PMarkerEllipse() :
    PSignalMarker(),
    mRx( 1.0 ),     // Just a default value bigger then 0
    mRy( 1.0 ),
    mCenter()
{
}


PMarkerEllipse::PMarkerEllipse( int id, const QString & label ) :
    PSignalMarker( id, label ),
    mRx( 1.0 ),     // Just a default value bigger then 0
    mRy( 1.0 ),
    mCenter()
{
}


PMarkerEllipse::PMarkerEllipse( int id, const QString & label, bool show, const QPen & pen ) :
    PSignalMarker( id, label, show, pen ),
    mRx( 1.0 ),     // Just a default value bigger then 0
    mRy( 1.0 ),
    mCenter()
{
}


PMarkerEllipse::PMarkerEllipse( const PMarkerEllipse & source ) :
    PSignalMarker( source ),
    mRx( source.mRx ),
    mRy( source.mRy ),
    mCenter( source.mCenter )
{
}


const PMarkerEllipse & PMarkerEllipse::operator =( const PMarkerEllipse & source )
{
    if ( this != &source )
    {
        PSignalMarker::operator =( source );

        mRx     = source.mRx;
        mRy     = source.mRy;
        mCenter = source.mCenter;
    }

    return *this;
}


const QPainterPath & PMarkerEllipse::translate( const QTransform & translation, bool forceTransformation )
{
    if ( forceTransformation || mMarker.isEmpty() )
    {
        mMarker = QPainterPath();   // Clear the current info

        mMarker.addEllipse( translation.map( mCenter ), mRx, mRy );
    }

    return mMarker;
}


QPainterPath PMarkerEllipse::translated( const QTransform & translation )
{
    QPainterPath path;

    path.addEllipse( translation.map( mCenter ), mRx, mRy );

    return path;
}


bool PMarkerEllipse::isEmpty( void ) const
{
    // Noting to check for. Assume there is always data, thus return false
    return false;
}


void PMarkerEllipse::clear( void )
{
    mCenter = QPointF();

    PSignalMarker::clear();
}


bool PMarkerEllipse::isMarkerAtPos( const QPointF & pos, double range ) const
{
    QLineF line( mCenter, pos );

    return fabs( line.length() ) <= range;
}
