/*
 *  This file is part of the Peacs support package
 *
 *  Copyright (C) 2010 - 2012 Peacs (www.peacs.nl)
 *
 *  The Peacs Support package is free software: you can redistribute it
 *  and/or modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  The Peacs support classes are distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the Peacs support package.  If not, see
 *  <http://www.gnu.org/licenses/>.
 *
 *    Project:              Peacs Support
 *    Qt version:           4.7.x
 *    Compiler:             QMake, MinGW, gcc 4.4.0
 *    Target platforms:     Windows, Mac OS-X
 *
 *    Subversion info:
 *
 *    $Id: PMarkerLine.cpp 743 2013-06-26 14:37:20Z eelco $
 *
 */

#include "PMarkerLine.h"

#include <math.h>


using namespace Peacs::Support;


PMarkerLine::PMarkerLine() :
    PSignalMarker()
{
}


PMarkerLine::PMarkerLine( int id, const QString & label ) :
    PSignalMarker( id, label )
{
}


PMarkerLine::PMarkerLine( int id, const QString & label, bool show, const QPen & pen ) :
    PSignalMarker( id, label, show, pen )
{
}


PMarkerLine::PMarkerLine( const PMarkerLine & source ) :
    PSignalMarker( source )
{
}


bool PMarkerLine::adjustForNewExtent( qreal minX, qreal maxX, qreal minY, qreal maxY )
{
    bool updated = false;

    if ( !isEmpty() )
    {
        if ( mFirst.x() == mLast.x() )
        {
            if ( mFirst.y() < mLast.y() )
            {
                if ( minY < maxY )
                {
                    mFirst.setY( minY );
                    mLast.setY( maxY );
                }
                else
                {
                    mFirst.setY( maxY );
                    mLast.setY( minY );
                }
            }
            else
            {
                if ( minY < maxY )
                {
                    mFirst.setY( maxY );
                    mLast.setY( minY );
                }
                else
                {
                    mFirst.setY( minY );
                    mLast.setY( maxY );
                }
            }

            updated = true;
        }
        else if ( mFirst.y() == mLast.y() )
        {
            if ( mFirst.x() < mLast.x() )
            {
                if ( minX < maxX )
                {
                    mFirst.setX( minX );
                    mLast.setX( maxX );
                }
                else
                {
                    mFirst.setX( maxX );
                    mLast.setX( minX );
                }
            }
            else
            {
                if ( minX < maxX )
                {
                    mFirst.setX( maxX );
                    mLast.setX( minX );
                }
                else
                {
                    mFirst.setX( minX );
                    mLast.setX( maxX );
                }
            }

            updated = true;
        }
    }

    return updated;
}


const QPainterPath & PMarkerLine::translate( const QTransform & translation, bool forceTransformation )
{
    if ( forceTransformation || mMarker.isEmpty() )
    {
        mMarker = QPainterPath( translation.map( mFirst ) );

        mMarker.lineTo( translation.map( mLast ) );
    }

    return mMarker;
}


QPainterPath PMarkerLine::translated( const QTransform & translation )
{
    QPainterPath path( translation.map( mFirst ) );

    path.lineTo( translation.map( mLast ) );

    return path;
}


bool PMarkerLine::isEmpty( void ) const
{
    return mFirst == mLast;
}


void PMarkerLine::clear( void )
{
    mFirst  = QPointF();
    mLast   = QPointF();

    PSignalMarker::clear();
}


bool PMarkerLine::isMarkerAtPos( const QPointF & pos, double range ) const
{
    bool yes = false;

    if ( mFirst != mLast )
    {
        if ( mFirst.x() == mLast.x() )
        {
            // It is a vertical line
            yes = fabs( pos.x() - mFirst.x() ) <= range;
        }
        else if ( mFirst.y() == mLast.y() )
        {
            // It is a horizontal line
            yes = fabs( pos.y() - mFirst.y() ) <= range;
        }
        else
        {
            // It is a sloping line. Check the length of the normal of the marker line to the
            // given point.
            QLineF l1( mFirst, mLast );
            QLineF l2( mFirst, pos );
            qreal  angle = l1.angleTo( l2 );

            yes = (fabs( sin( angle ) ) * l2.length()) <= range;
        }
    }

    return yes;
}
