/*
 *  This file is part of the Peacs support package
 *
 *  Copyright (C) 2010 - 2012 Peacs (www.peacs.nl)
 *
 *  The Peacs Support package is free software: you can redistribute it
 *  and/or modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  The Peacs support classes are distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the Peacs support package.  If not, see
 *  <http://www.gnu.org/licenses/>.
 *
 *    Project:              Peacs Support
 *    Qt version:           4.7.x
 *    Compiler:             QMake, MinGW, gcc 4.4.0
 *    Target platforms:     Windows, Mac OS-X
 *
 *    Subversion info:
 *
 *    $Id: PSignalMarker.cpp 930 2013-11-27 18:56:36Z eelco $
 *
 */


#include "PSignalMarker.h"


using namespace Peacs::Support;


int UNINITIALYZED_MARKER_DATA = INT_MAX;


PSignalMarker::PSignalMarker( void ) :
    mMarker(),
    mId( UNINITIALYZED_MARKER_DATA-- ), // Make sure all marker data object can be uniquly identified
    mLabel(),
    mShow( false ),
    mPen()
{
}


PSignalMarker::PSignalMarker( int id, const QString & label ) :
    mMarker(),
    mId( id ),
    mLabel( label ),
    mShow( false ),
    mPen()
{
}


PSignalMarker::PSignalMarker( int id, const QString & label, bool show, const QPen & pen ) :
    mMarker(),
    mId( id ),
    mLabel( label ),
    mShow( show ),
    mPen( pen )
{
}


PSignalMarker::PSignalMarker( const PSignalMarker & source ) :
    mMarker( source.mMarker ),
    mId( source.mId ),
    mLabel( source.mLabel ),
    mShow( source.mShow ),
    mPen( source.mPen )
{
}


const PSignalMarker &PSignalMarker::operator =( const PSignalMarker & source )
{
    if ( this != &source )
    {
        mMarker     = source.mMarker;
        mId         = source.mId;
        mLabel      = source.mLabel;
        mShow       = source.mShow;
        mPen        = source.mPen;
    }

    return *this;
}


bool PSignalMarker::operator ==( const PSignalMarker & source ) const
{
    return (mId == source.mId) && (mLabel == source.mLabel);
}


bool PSignalMarker::operator !=( const PSignalMarker & source ) const
{
    return (mId != source.mId) || (mLabel != source.mLabel);
}


const QPainterPath & PSignalMarker::translate( const QTransform & translation, bool forceTransformation )
{
    Q_UNUSED( translation )
    Q_UNUSED( forceTransformation )

    return mMarker;
}


QPainterPath PSignalMarker::translated( const QTransform & translation )
{
    Q_UNUSED( translation )

    return mMarker;
}


bool PSignalMarker::isEmpty( void ) const
{
    return false;
}


bool PSignalMarker::isMarkerAtPos( const QPointF & pos, double range ) const
{
    Q_UNUSED( pos )
    Q_UNUSED( range )

    // This class has noting to check against because there is no information in data coordinate system
    // Derived classes should be used to do the real checking
    return false;
}
