/*
 *  This file is part of the Peacs support package
 *
 *  Copyright (C) 2010 - 2012 Peacs (www.peacs.nl)
 *
 *  The Peacs Support package is free software: you can redistribute it
 *  and/or modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  The Peacs support classes are distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the Peacs support package.  If not, see
 *  <http://www.gnu.org/licenses/>.
 *
 *    Project:              Peacs Support
 *    Qt version:           4.7.x
 *    Compiler:             QMake, MinGW, gcc 4.4.0
 *    Target platforms:     Windows, Mac OS-X
 *
 *    Subversion info:
 *
 *    $Id: PSignalMarkers.cpp 1113 2014-02-19 21:03:28Z eelco $
 *
 */


#include "PSignalMarkers.h"


using namespace Peacs::Support;


PSignalMarkers::PSignalMarkers( void ) :
    mData(),
    mSelectedIndex( NO_MARKER_FOUND )
{
}


PSignalMarkers::PSignalMarkers( int size ) :
    mData( size ),
    mSelectedIndex( NO_MARKER_FOUND )
{
}


PSignalMarkers::~PSignalMarkers( void )
{
    clear();
}


void PSignalMarkers::clear( void )
{
    if ( mData.count() > 0 )
    {
        PSignalMarker** markerPtrs = mData.data();

        for ( int i = 0; i < mData.count(); i++ )
        {
            delete markerPtrs[i];
            markerPtrs[i] = NULL;
        }

        mData.clear();
    }

    mSelectedIndex = NO_MARKER_FOUND;
}


bool PSignalMarkers::setSize( int size )
{
    if ( size < mData.size() )
    {
        // We have to clear memory
        for ( int index = size; index < mData.size(); ++index )
        {
            delete mData[ index ];
            mData[ index ] = NULL;
        }
        mData.resize( size );
    }
    else if ( size > mData.size() )
    {
        // We have to add new pointer memory initialized with 0 (NULL)
        int oldSize = mData.size();

        mData.resize( size );

        for ( int index = oldSize; index < size; ++index )
        {
            mData[ index ] = NULL;
        }
    }

    if ( mSelectedIndex >= size )
    {
        mSelectedIndex = NO_MARKER_FOUND;
    }

    return mData.size() == size;
}


int PSignalMarkers::findMarker( const QPointF & pos, double range, bool onlyVisible ) const
{
    int markerIndex = NO_MARKER_FOUND;

    if ( mData.count() > 0 )
    {
        PSignalMarker * const * markerPtrs = mData.constData();

        for ( int i = 0; (i < mData.count()) && (markerIndex == NO_MARKER_FOUND); i++ )
        {
            if ( (!onlyVisible || markerPtrs[i]->mustShow()) &&
                 markerPtrs[i]->isMarkerAtPos( pos, range ) )
            {
                markerIndex = i;
            }
        }
    }

    return markerIndex;
}
