/*
 *  This file is part of the Peacs support package
 *
 *  Copyright (C) 2010 - 2012 Peacs (www.peacs.nl)
 *
 *  The Peacs Support package is free software: you can redistribute it
 *  and/or modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  The Peacs support classes are distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the Peacs support package.  If not, see
 *  <http://www.gnu.org/licenses/>.
 *
 *    Project:              Peacs Support
 *    Qt version:           4.7.x
 *    Compiler:             QMake, MinGW, gcc 4.4.0
 *    Target platforms:     Windows, Mac OS-X
 *
 *    Subversion info:
 *
 *    $Id: PSignalPlot.cpp 1161 2014-03-12 12:21:14Z eelco $
 *
 */


#include "psignalplot.h"
#include "pfunctions.h"
#include "PWidgetDefines.h"

#include <QDebug>


using namespace Peacs::Support;


PSignalPlot::PSignalPlot( bool xAxisShowsIndex, double zoomT0, double zoomT1, int column, int row ) :
    PSignalPlotBase( xAxisShowsIndex, zoomT0, zoomT1, column, row )
{
}


PSignalPlot::~PSignalPlot()
{
}


//void PSignalPlot::init()
//{
//    PSignalPlotBase::init();
//}


void PSignalPlot::paintForeGround( QPainter * painter, const QRectF & updateRect )
{
    QRectF  paintRect( getGraphRect() );
    QPointF itemPos( pos() );

    paintRect.translate( itemPos );
    paintRect = paintRect.intersected( updateRect );

    painter->save();

    if ( isSignalClipping() )
    {
        painter->setClipping( true );
        painter->setClipRect( paintRect ); // do not draw outside the draw area
    }

    // Only draw the markers and selection rectangle when painting to the screen
    if ( !isPrinting() )
    {
        double          pixelSize       = getPixelSize();
        QRectF          objectRect;
        PSignalMarker * markerPtr       = NULL;
        QPainterPath    markerPath;

        for ( int markerNr = mMarkers.count() - 1; markerNr >= 0; markerNr-- )
        {
            markerPtr = mMarkers[ markerNr ];

            if ( markerPtr->mustShow() && !markerPtr->isEmpty() )
            {
                painter->setPen( markerPtr->getPen() );

                if ( markerPtr->getMarker().isEmpty() )
                {
                    markerPtr->translate( getGraphTransform() );
                }

                markerPath = markerPtr->getMarker();

                markerPath.translate( itemPos );

                objectRect = markerPath.boundingRect();

                if ( 0 == objectRect.width() )
                {
                    // Most time the width of a line is zero. In that case the intersection will fail. Just add a width
                    objectRect.setWidth( 1.0 );
                }

                if ( paintRect.intersects( objectRect ) )
                {
                    painter->drawPath( markerPath );
                }
            }
        }

        if ( getSelectRect().isValid() )
        {
            painter->setPen( QPen( QBrush( getSelectRectColor() ), pixelSize ) );
            painter->drawRect( mapRectToScene( getSelectRect() ) );
        }
    }

    painter->restore();

    PSignalPlotBase::paintForeGround( painter, updateRect );
}


void PSignalPlot::paint ( QPainter * painter,
                          const QStyleOptionGraphicsItem * option,
                          QWidget * widget )
{
    Q_UNUSED( widget );

    double  pixelSize   = getPixelSize();
    bool    brushSet    = false;

    // Create or update the polygon information according the given signal data
    mSignals.CreateOrUpdatePolygons( getGraphTransform(), pixelSize );

    painter->save();
    if ( isSignalClipping() )
    {
        painter->setClipping( true );
        painter->setClipRect( getGraphRect().intersected( option->exposedRect ) ); // do not draw outside the draw area
    }

    for ( int sigNr = 0; sigNr < mSignals.count(); sigNr++ )
    {
        const PSignalData & constSigData = mSignals.at( sigNr );

        if ( constSigData.mustShow() && (constSigData.getDataPoly().count() > 0) )
        {
            QPen pen( constSigData.getPen() );

            if ( isPrinting() && (constSigData.getPrintPen().style() != Qt::NoPen) )
            {
                pen = constSigData.getPrintPen();
            }
            pen.setWidthF( 1.3 * pixelSize );
            painter->setPen( pen );

            if ( isPrinting() )
            {
                if ( constSigData.hasPrintBrush() )
                {
                    // If there is a print brush defined, set it ...
                    painter->setBrush( constSigData.getPrintBrush() );
                    brushSet = true;
                }
                else if ( constSigData.hasBrush() )
                {
                    // If there is a brush defined, set it ...
                    painter->setBrush( constSigData.getBrush() );
                    brushSet = true;
                }
            }
            else if ( constSigData.hasBrush() )
            {
                // If there is a brush defined, set it ...
                painter->setBrush( constSigData.getBrush() );
                brushSet = true;
            }

            if ( brushSet )
            {
                QPainterPath path;

                path.addPolygon( constSigData.getDataPoly() );

                painter->drawPath( path );
            }
            else
            {
                painter->drawPolyline( constSigData.getDataPoly() );
            }

            if ( constSigData.hasBrush() || (isPrinting() && constSigData.hasPrintBrush()) )
            {
                // ... and reset it again
                painter->setBrush( QBrush() );
                brushSet = false;
            }
        }
    }

    painter->restore();

    // Call the base implementation to reset the stop mouse move handling flag
    PSignalPlotBase::paint( painter, option, widget );
}
