/*
 *  This file is part of the Peacs support package
 *
 *  Copyright (C) 2010 - 2012 Peacs (www.peacs.nl)
 *
 *  The Peacs Support package is free software: you can redistribute it
 *  and/or modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  The Peacs support classes are distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the Peacs support package.  If not, see
 *  <http://www.gnu.org/licenses/>.
 *
 *    Project:              Peacs Support
 *    Qt version:           4.7.x
 *    Compiler:             QMake, MinGW, gcc 4.4.0
 *    Target platforms:     Windows, Mac OS-X
 *
 *    Subversion info:
 *
 *    $Id: PSimpleTextItem.cpp 902 2013-11-01 14:55:29Z eelco $
 *
 */


#include "PSimpleTextItem.h"

//#include <QBrush>
//#include <QPainter>

//#include <QDebug>


using namespace Peacs::Support;


PSimpleTextItem::PSimpleTextItem( const QString & text, int column, int row ) :
    QGraphicsTextItem( text ),
    PGrItemBase( column, row ) // The height is set according the text height and not according the matrix element size
{
    if ( text.contains( '<' ) )
    {
        // We assume that this is HTML text
        setHtml( text );
    }
    else
    {
        // We assume that this is plain text
        setPlainText( text );
    }
}


bool PSimpleTextItem::setBounds( qreal width, qreal height )
{
    bool    changed     = false;

    if ( (rotation() == 90) || (rotation() == -90) )
    {
        if ( height != textWidth() )
        {
            width = height;
        }
        else
        {
            width = 0.0;
        }
    }
    else if ( width == textWidth() )
    {
        width = 0.0;
    }

    if ( 0.0 != width )
    {
        if ( width > textWidth() )
        {
            setTextWidth( width );             // Make sure that align center works
        }

        // Get the new default rect from the base class, i.e. the text bouding rectangle
        mBounds = QGraphicsTextItem::boundingRect();

        mBounds.setWidth( width );           // Set just to be sure

        prepareGeometryChange();                // Notify the view of the new bounds (update will be called if needed)

        changed = true;
    }

    return changed;
}


QRectF PSimpleTextItem::boundingRect( void ) const
{
    if ( mBounds.isEmpty() )
    {
        return QGraphicsTextItem::boundingRect();
    }
    else
    {
        return mBounds;
    }
}


QSizeF PSimpleTextItem::getPreferredSize( void )
{
    // This call changes the actual size of the internal bounding box. Use setBounds(...) after calling this function
    adjustSize();

    // Use the transformation to take rotation, etc. in account.
    return sceneTransform().mapRect( QGraphicsTextItem::boundingRect() ).size();
}


// Only here for testing. Default commented out
//void PSimpleTextItem::paint( QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget )
//{
////    if ( getRow() == 2 )
////    {
//        QBrush bgBrush( Qt::cyan );
//        painter->fillRect( boundingRect(), bgBrush );
////    }

////    qDebug() << "Draw for item at [" << getColumn() << getRow() << "] at position [" << pos() << "]\t" << getBounds();

//    QGraphicsTextItem::paint( painter, option, widget );
//}
