/*
 *  This file is part of the Peacs support package
 *
 *  Copyright (C) 2010 - 2012 Peacs (www.peacs.nl)
 *
 *  The Peacs Support package is free software: you can redistribute it
 *  and/or modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  The Peacs support classes are distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the Peacs support package.  If not, see
 *  <http://www.gnu.org/licenses/>.
 *
 *    Project:              Peacs Support
 *    Qt version:           4.7.x
 *    Compiler:             QMake, MinGW, gcc 4.4.0
 *    Target platforms:     Windows, Mac OS-X
 *
 *    Subversion info:
 *
 *    $Id: PSimpleTextItem.h 902 2013-11-01 14:55:29Z eelco $
 *
 */


#ifndef PSIMPLETEXTITEM_H
#define PSIMPLETEXTITEM_H


#include <QGraphicsTextItem>

#include "PGrItemBase.h"
#include "PeacsSupport_global.h"


namespace Peacs
{
    namespace Support
    {
        /** @class PSimpleTextItem PSimpleTextItem.h "widgets/PSimpleTextItem.h"
          * @brief The class can be used to display simple texts in a Peacs graphics view
          */
        class PEACSSUPPORTSHARED_EXPORT PSimpleTextItem : public QGraphicsTextItem, public PGrItemBase
        {

        public:

            /** @brief Only present to be used in the type() function
              * @see PGrItemBase
              */
            enum { Type = PGrItemBase::PSimpleTextItemType };


            /** @brief Returns the type of graphics item.
              * Enable the use of qgraphicsitem_cast with this item.
              * @see PGrItemBase
              */
            inline int type( void ) const { return Type; }


            /** @brief constructor
              *
              * @param text the text to show in this item
              * @param column the column at which this item will reside in the matrix (default -1)
              * @param row the row at which this item will reside in the matrix (default -1)
              *
              * @see PPlotBase for information about column and row
              */
            PSimpleTextItem( const QString & text, int column = -1, int row = -1 );


            /** @brief Sets the bounding (virtual) rectangle used when drawing
              * This internally used virtual drawing board bound will be used to draw within.
              *
              * @param width The new width to use.
              * @param height The new height to use.
              * @return true if the bounding rectangle has changed
              */
            bool setBounds( qreal width, qreal height );


            /** @brief returns the outer bounds of the item as a rectangle.
              * All painting must be restricted to inside an item's bounding rect.
              */
            QRectF boundingRect( void ) const;


            /** @brief
              *
              * @note This call changes the actual size of the internal bounding box. Use setBounds(...)
              * after calling this function
              */
            QSizeF getPreferredSize( void );


            /** Only here for testing. Default commented out
              */
            //void paint( QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget );
        };
    }
}


#endif // PSIMPLETEXTITEM_H
