/*
 *  This file is part of the Peacs support package
 *
 *  Copyright (C) 2010 - 2012 Peacs (www.peacs.nl)
 *
 *  The Peacs Support package is free software: you can redistribute it
 *  and/or modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  The Peacs support classes are distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the Peacs support package.  If not, see
 *  <http://www.gnu.org/licenses/>.
 *
 *    Project:              Peacs Support
 *    Qt version:           4.7.x
 *    Compiler:             QMake, MinGW, gcc 4.4.0
 *    Target platforms:     Windows, Mac OS-X
 *
 *    Subversion info:
 *
 *    $Id: PXYPlot.h 1113 2014-02-19 21:03:28Z eelco $
 *
 */


#ifndef __PXYPLOT_H
#define	__PXYPLOT_H


#include "PeacsSupport_global.h"
#include "PSignalPlot.h"


namespace Peacs
{
    namespace Support
    {
        /** @class PXYPlot PXYPlot.h "widgets/PXYPlot.h"
          * @brief The view window in which the ECGs are displayed
          */
        class PEACSSUPPORTSHARED_EXPORT PXYPlot : public PSignalPlot
        {
//            Q_OBJECT

        public:

            /** @brief Only present to be used in the type() function
              * @see PGrItemBase
              */
            enum { Type = PGrItemBase::PXYPlotType };


            /** @brief Returns the type of graphics item.
              * Enable the use of qgraphicsitem_cast with this item.
              * @see PGrItemBase
              */
            inline int type( void ) const { return Type; }


            /** @brief constructor
              *
              * @param column the column at which this item will reside in the matrix (default -1)
              * @param row the row at which this item will reside in the matrix (default -1)
              *
              * @see PGrItemBase for information about column and row
              */
            PXYPlot( int column = -1, int row = -1 );


            /** @brief destructor: Nothing to do */
            virtual ~PXYPlot();


            /** @brief initializes the class with the data provided before
              * This function must be called before displaying the desired data. It
              * will initialize local members and in a derived class implementation
              * specific data can be set, like labels.
              */
            virtual void init( void );


            /** @brief compute the time and amplitude values from the given graphicsItem coordinates
              * @param pos the graphicsItem coordinates
              */
            inline QPointF toXY( const QPointF & pos ) { return toTimeAmplitude( pos );/*QPointF( toX( pos.x() ), toY( pos.y() ) );*/ }


            /** @brief compute the time and amplitude values to the graphicsItem coordinates
             * @param t the time
             * @param ampl the amplitude
             */
            inline QPointF toItemCoor( double t, double ampl ) { return PSignalPlotBase::toItemCoor( t, ampl );/*QPointF( toItemX( t ), toItemY( ampl ) );*/ }


            /** @brief Wrapper function for PSignalPlotBase::toAmplitude
              * Convenient function to maintain logical naming compared to this class name
              *
              * @see PSignalPlotBase::toAmplitude
              */
            inline double toY( double pos ) const { return toAmplitude( pos ); }


            /** @brief Wrapper function for PSignalPlotBase::toItemAmplitude
              * Convenient function to maintain logical naming compared to this class name
              *
              * @see PSignalPlotBase::toItemAmplitude
              */
            inline double toItemY( double signalY ) const { return toItemAmplitude( signalY ); }


            /** @brief Wrapper function for PSignalPlotBase::toTime
              * Convenient function to maintain logical naming compared to this class name
              *
              * @see PSignalPlotBase::toTime
              */
            inline double toX( double pos ) const { return toTime( pos ); }


            /** @brief Wrapper function for PSignalPlotBase::toItemTime
              * Convenient function to maintain logical naming compared to this class name
              *
              * @see PSignalPlotBase::toItemTime
              */
            inline double toItemX( double signalX ) const { return toItemTime( signalX ); }


            virtual void calcAmplitudeScale( bool up = true );


            virtual void calcTimeScale( bool up = true );
        };
    }
}

#endif	/* __PXYPLOT_H */
