/*
 *  This file is part of the Peacs support package
 *
 *  Copyright (C) 2010 - 2012 Peacs (www.peacs.nl)
 *
 *  The Peacs Support package is free software: you can redistribute it
 *  and/or modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  The Peacs support classes are distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the Peacs support package.  If not, see
 *  <http://www.gnu.org/licenses/>.
 *
 *    Project:              Peacs Support
 *    Qt version:           4.7.x
 *    Compiler:             QMake, MinGW, gcc 4.4.0
 *    Target platforms:     Windows, Mac OS-X
 *
 *    Subversion info:
 *
 *    $Id: psignalplot.h 1157 2014-03-12 10:47:06Z eelco $
 *
 */

#ifndef __PSIGNALPLOT_H__
#define	__PSIGNALPLOT_H__


#include <QStyleOptionGraphicsItem>

#include "psignalplotBase.h"


namespace Peacs
{
    namespace Support
    {
        /** @class PSignalPlot PSignalPlot.h "widgets/PSignalPlot.h"
          * @brief The graphics scene item that can show a graph with content
          */
        class PEACSSUPPORTSHARED_EXPORT PSignalPlot : public PSignalPlotBase
        {
            Q_OBJECT

        public:

            /** @brief Only present to be used in the type() function
              * @see PGrItemBase
              */
            enum { Type = PGrItemBase::PSignalPlotType };


            /** @brief Returns the type of graphics item.
              * Enable the use of qgraphicsitem_cast with this item.
              * @see PGrItemBase
              */
            inline int type( void ) const { return Type; }


            /** @brief constructor
              *
              * @param xAxisShowsIndex If true, the X-axis shows index data, i.e. starting at 0 with intervals of 1. @see mXAxisShowsIndex
              * @param zoomT0 Specified the initial starting point, i.e. left extent of the X-axis
              * @param zoomT1 Specified the initial end point, i.e. right extent of the X-axis
              * @param column the column at which this item will reside in the matrix (default -1)
              * @param row the row at which this item will reside in the matrix (default -1)
              *
              * @see PGrItemBase for information about column and row
              */
            PSignalPlot( bool xAxisShowsIndex, double zoomT0, double zoomT1, int column = -1, int row = -1 );


            /** @brief destructor: Nothing to do
              */
            virtual ~PSignalPlot();


//            /** @brief initializes the class with the data provided before
//              * This function must be called before displaying the desired data. It
//              * will initialize local members and in a derived class implementation
//              * specific data can be set, like labels.
//              */
//            virtual void init();


            /** @brief In this class no real function is implemented, override to add needed function
              * Override this function in derived classes to enable drawing data on top of the background
              * and signal information
              *
              * @param painter the painter see Qt documentation
              * @param updateRect the rectangle, in scene coordinates, of the rectangle to (re-)draw
              */
            virtual void paintForeGround( QPainter * painter, const QRectF & updateRect );


            /** paint the grid and ECG signals
              * @param painter see PGraphicsSignalView
              * @param option used to determine the scale
              * @param widget not used
              */
            virtual void paint( QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget = 0 );


//        protected:

//            /** @brief Implement this function to adjust the position of the marker for the given position
//              * This function will be called by the mouse click handling function to indicate a user action
//              * at the specified position in the graph. The given positions will be in data coordinates
//              * system and graph coordinate system to enable maximum control.
//              * Derived, application specific classes should implement the correct marker behaviour for this.
//              *
//              * This function should not be called here but in a derived class
//              *
//              * @param dataPos The position to adjust with. This position is in data coordinate system
//              * @param graphPos The position to adjust with. This position is in graph coordinate system
//              * @param mouseReleased If true, this function is being called from the mouse release event, otherwise the mouse move event
//              *
//              * @return true if the position has changed
//              */
//            virtual bool adjustMarkerPosition( const QPointF & dataPos, const QPointF & graphPos, bool mouseReleased )
//                { Q_UNUSED(dataPos) Q_UNUSED(graphPos) Q_UNUSED(mouseReleased) Q_ASSERT( false ); return false; }
        };
    }
}

#endif	/* __PSIGNALPLOT_H__ */

