/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id$
 *
 */


//#include <QDebug>

#include "PCircSignalScene.h"
#include "PSignalPlotBase.h"

using namespace Peacs::CircAdapt;
using namespace Peacs::Support;


PCircSignalScene::PCircSignalScene( int columns, int rows, QObject * parentPtr ) :
    PGraphMatrixScene( columns, rows, parentPtr )
{
}


void PCircSignalScene::autoScale( void )
{
//qDebug() << "PCircSignalScene::autoScales called";

    // The horizontal oriented graph items should always show tha same Y-axis scale. So determaine
    // the maximum needed extent for the horizontal items and set the extent accordingly
    double                  minY    = 0;
    double                  maxY    = 0;
    PSignalPlotBase *       sigplot = NULL;

    Q_ASSERT( matrixItems().size() == 9 );

    // The first item in the first row is located at the 4th position. The first one in the second row
    // at the 8th position. Etc.
    for ( int y = 0; y < 3; ++y )
    {
        minY = 0;
        maxY = 0;

        for ( int x = 0; x < 3; ++x )
        {
            sigplot = dynamic_cast<PSignalPlotBase *>( matrixItems()[ (y * 3) + x ] );

            if ( NULL != sigplot )
            {
                double tmpMinY = sigplot->getMinYData();
                double tmpMaxY = sigplot->getMaxYData();

                if ( minY > tmpMinY )
                {
                    minY = tmpMinY;
                }

                if ( maxY < tmpMaxY )
                {
                    maxY = tmpMaxY;
                }

                if ( 2 == x )
                {
                    // Let the scale start at zero of not negative
                    if ( 0.0 < minY )
                    {
                        minY = 0.0;
                    }
                    if ( 0.0 > maxY )
                    {
                        maxY = 0.0;
                    }

                    if ( minY != maxY )
                    {
                        // Only one item needs to be set. It will inform the others
                        sigplot->setSignalAmpl( minY, maxY, false );
                    }
                }
            }
        }
    }
}


void PCircSignalScene::scaleAmplitude( QObject * itemPtr )
{
    if ( itemPtr && dynamic_cast<PSignalPlotBase *>( itemPtr ) )
    {
        // For the amplitude we use the same scale for all horizontal items, so use the row number
        PSignalPlotBase *   basePtr         = static_cast<PSignalPlotBase *>( itemPtr );
        PSignalPlotBase *   sigplot         = NULL;
        double              maxSigAmpl      = basePtr->getMaxSigAmpl();
        double              minSigAmpl      = basePtr->getMinSigAmpl();
        double              amplitudeStep   = basePtr->getAmplitudeStep();
        double              amplitudeRange  = basePtr->getAmplitudeRange();

        if ( Widget::NOTUSED != basePtr->getRow() )
        {
            for ( int x = 0; x < 3; ++x )
            {
                sigplot = dynamic_cast<PSignalPlotBase *>( matrixItems()[ (basePtr->getRow() * 3) + x ] );

                sigplot->changeAmplScale( maxSigAmpl, minSigAmpl, amplitudeStep, amplitudeRange );
            }
        }
    }
}


void PCircSignalScene::scaleTime( QObject * itemPtr )
{
    if ( itemPtr && dynamic_cast<PSignalPlotBase *>( itemPtr ) )
    {
        // For the time we use the same scale for all vertical items, so use the column number
        PSignalPlotBase *   basePtr     = static_cast<PSignalPlotBase *>( itemPtr );
        PSignalPlotBase *   sigplot     = NULL;
        double              maxTime     = basePtr->getZoomT1();
        double              minTime     = basePtr->getZoomT0();
        double              timeStep    = basePtr->getTimeStep();
        double              timeRange   = basePtr->getTimeRange();

        if ( Widget::NOTUSED != basePtr->getColumn() )
        {
            for ( int y = 0; y < 3; ++y )
            {
                sigplot = dynamic_cast<PSignalPlotBase *>( matrixItems()[ (y * 3) + basePtr->getColumn() ] );

                sigplot->changeTimeScale( maxTime, minTime, timeStep, timeRange );
            }
        }
    }
}
