/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id$
 *
 */

#ifndef PCIRCSIGNALSCENE_H
#define PCIRCSIGNALSCENE_H


#include "PGraphMatrixScene.h"


namespace Peacs
{
    namespace CircAdapt
    {
        /** @class PCircSignalScene PCircSignalScene.h "dataViews/PCircSignalScene.h"
          * @brief Is used maintain all time related signal items in a matrix shape
          */
        class PCircSignalScene : public Peacs::Support::PGraphMatrixScene
        {

        public:

            /** @brief Constructor
              *
              * @param columns Specifies the number of columns shown in this scene
              * @param rows Specifies the number of rows shown in this scene
              * @param parentPtr Pointer to the parent for this scene object
              */
            PCircSignalScene( int columns, int rows, QObject * parentPtr = 0 );


            /** @brief Forces all items to rescale themself
              * This function is called after the autoScaleSlot() slot is triggered by a graph item (plot or draw)
              * and when the update action in a graph item (by calling autoScaleSlot() function of that class) has
              * returned true, indicating a rescal is needed.
              * Default implementation will tell all items to rescale themself according the items signal data
              * min/max values.
              * This function overrides this to link the horizontal and vertical positioned items in the desired way
              */
            void autoScale( void );


        protected:

            /** @brief Handles the signals from graph items inticating an amplitude rescale is perfomed
              * This function is the default implementation of handling the amplitude scaling at a graph item. When
              * an item changes its amplitude scale, it will emit a signal to inform the view. The view is
              * responsible to take the appropriate actions with regards to the other items in the view, i.e.
              * rescale the amplitude of those items the same or just some of them.
              * Default implementation does noting. This implementation handles time scaling in an
              * application specific way
              *
              * @param itemPtr Points to the item initiating the signal
              * @see PGrSceneBase::scaleAmplitudeSlot
              */
            void scaleAmplitude( QObject * itemPtr );


            /** @brief Handles the signals from graph items inticating a time rescale is perfomed
              * This function is the default implementation of handling the time scaling at a graph item. When
              * an item changes its time scale, it will emit a signal to inform the view. The view is
              * responsible to take the appropriate actions with regards to the other items in the view, i.e.
              * rescale the time of those items the same or just some of them.
              * Default implementation does noting. This implementation handles time scaling in an
              * application specific way
              *
              * @param itemPtr Points to the item initiating the signal
              * @see PGrSceneBase::scaleTimeSlot
              */
            void scaleTime( QObject * itemPtr );
        };
    }
}

#endif // PCIRCSIGNALSCENE_H
