/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id$
 *
 */

#ifndef PPVSCENE_H
#define PPVSCENE_H


#include "PGraphMatrixScene.h"



namespace Peacs
{
    namespace CircAdapt
    {
        /** @class PPvScene PPvScene.h "dataViews/PPvScene.h"
          * @brief Is used maintain all PV-loop related items in a matrix shape
          */
        class PPvScene : public Peacs::Support::PGraphMatrixScene
        {

        public:

            /** @brief Constructor
              *
              * @param columns Specifies the number of columns shown in this scene
              * @param rows Specifies the number of rows shown in this scene
              * @param parentPtr Pointer to the parent for this scene object
              */
            PPvScene( int columns, int rows, QObject * parentPtr = 0 );


            /** @brief Forces all items to rescale themself
              * This function is called after the autoScaleSlot() slot is triggered by a graph item (plot or draw)
              * and when the update action in a graph item (by calling autoScaleSlot() function of that class) has
              * returned true, indicating a rescal is needed.
              * Default implementation will tell all items to rescale themself according the items signal data
              * min/max values.
              * This function overrides this to link the horizontal and vertical positioned items in the desired way
              */
            void autoScale( void );
        };
    }
}

#endif // PPVSCENE_H
