/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: pparamview.cpp 923 2013-11-11 14:03:15Z eelco $
 *
 */
#include "tcirculation.h"
#include "pparamview.h"
#include "pvector.h"
#include "PSettings.h"

#include "PCircAdaptOptionsDialog.h"

//#include <QDebug>
#include <QClipboard>

using namespace Peacs::CircAdapt;


PParamView::PParamView(UCirculation &data, MSimulateCirculation &simClass, QWidget *parent) :
    QWidget( parent ),
    mData( data ),
    mSimCirc( simClass ),
    mUi(),
    mBusy( false ),
    mDoUpdate( false ),
    mTmpValue( 0.0 )
{
    mUi.setupUi( this );

    mUi.mitralis_insuff->setToolTip( "% of reference valve area" );
    mUi.mitralis_stenosisspinbox->setToolTip( "% of reference valve area" );
    mUi.aorta_insuff->setToolTip( "% of reference valve area" );
    mUi.aorta_stenosisSpinbox->setToolTip( "% of reference valve area" );
    mUi.tricus_insuff->setToolTip( "% of reference valve area" );
    mUi.tricus_stenosisSpinbox->setToolTip( "% of reference valve area" );
    mUi.pulmvalve_insuff->setToolTip( "% of reference valve area" );
    mUi.pulmvalve_stenosisSpinbox->setToolTip( "% of reference valve area" );

    mUi.restActiveLabel->setAutoFillBackground(true);
    mUi.exerciseActiveLabel->setAutoFillBackground(true);
    mUi.transitionActiveLabel->setAutoFillBackground(true);

    updateAllSettings( int(POptionsDialog::PARAM_TABS_CHANGED) );
}


PParamView::~PParamView(  )
{
}



void PParamView::specsHaveBeenCopied()
{
    mDoUpdate = false;
}


void PParamView::updateAllSettings( int changed )
{
    // For the meaning of the parameter POptionsDialog::mChangedItems
    POptionsDialog::ChangedOptionsType changedType( changed );

    if ( changedType.testFlag( POptionsDialog::PARAM_TABS_CHANGED ) )
    {
        // This is the only part of read settings that has to be read afer the
        // options have been changed
        Support::PSettings  pSetting;
        QSettings &         settings    = pSetting.getSettings();
        bool                valB;
        QString             valStr      = settings.value( PARAMETER_TABS_SELECTED, POptionsDialog::PARAM_TAB_ID_GEN ).toString();

        if ( valStr == POptionsDialog::PARAM_TAB_ID_VENTR )
        {
            mUi.general->setCurrentIndex( 1 );
        }
        else if ( valStr == POptionsDialog::PARAM_TAB_ID_ATRIA )
        {
            mUi.general->setCurrentIndex( 2 );
        }
        else if ( valStr == POptionsDialog::PARAM_TAB_ID_VALVES )
        {
            mUi.general->setCurrentIndex( 3 );
        }
        else if ( valStr == POptionsDialog::PARAM_TAB_ID_VASC )
        {
            mUi.general->setCurrentIndex( 4 );
        }
        else if ( valStr == POptionsDialog::PARAM_TAB_ID_SHUNT )
        {
            mUi.general->setCurrentIndex( 5 );
        }
        else if ( valStr == POptionsDialog::PARAM_TAB_ID_ADAPT )
        {
            mUi.general->setCurrentIndex( 6 );
        }
        else
        {
            mUi.general->setCurrentIndex( 0 );
        }

        // The first tab will always be enabled
        valB = settings.value( PARAMETER_TABS_VENTR, true ).toBool();
        mUi.general->setTabEnabled( 1, valB );
        valB = settings.value( PARAMETER_TABS_ATRIA, true ).toBool();
        mUi.general->setTabEnabled( 2, valB );
        valB = settings.value( PARAMETER_TABS_VALVES, true ).toBool();
        mUi.general->setTabEnabled( 3, valB );
        valB = settings.value( PARAMETER_TABS_VASC, true ).toBool();
        mUi.general->setTabEnabled( 4, valB );
        valB = settings.value( PARAMETER_TABS_SHUNT, true ).toBool();
        mUi.general->setTabEnabled( 5, valB );
        valB = settings.value( PARAMETER_TABS_ADAPT, true ).toBool();
        mUi.general->setTabEnabled( 6, valB );
    }
}


void PParamView::Initialize( void )
{
    // TODO We must do something with the adaptation mode here and in the model after a reset

    updateInfo();
}


void PParamView::updateInfo( void )
{
    if ( !mBusy && !mData.resistances().isEmpty() && !mDoUpdate )
    {
        mBusy = true;

        // General parameters
        int beattime = qMax( 0, qRound( 60000.0 / (double)mData.getHeartbeatTime() ) );
        mUi.pressureControlCheckbox->setChecked( mData.pressureControl() );
        if ( mUi.heartRateDial->value() != beattime )
        {
            mUi.heartRateDial->setValue( beattime );
        }

        mUi.pressureControlCheckbox->setChecked(    mData.pressureControl() );
        mUi.meanAPSpinBox->setValue(                mData.toPressure( mData.meanArterialPressure() ) );
        mUi.COSpinBox->setValue(                    mData.toFlow(mData.venousReturn()) );

        mUi.meanAPSpinBox->setEnabled( mData.pressureControl() );
        mUi.COSpinBox->setEnabled( mData.pressureControl() );

        // Ventricles parameters
        double avdelay = mData.heart().getVentricles().getActivationDelay() * 1000.0;
        mUi.AVdelaySpinBox->setValue(               avdelay );
        mUi.AVdelayLabel->setText( QString( "%1" ).arg( qRound( mData.getHeartbeatTime() * AVDELAY_RELATIVEFROMRATE + avdelay ) ) );


        mUi.LVpassiveStifnessSpinBox->setValue(     mData.getHeart().getVentricles().getLeftVentricle().getRelSfPas() * 100.0 );
        mUi.LVcontractilitySpinBox->setValue(       mData.getHeart().getVentricles().getLeftVentricle().getRelSfAct() * 100.0 );
        mUi.LVMassSpinBox->setValue(                mData.getHeart().getVentricles().getLeftVentricle().getRelWallVolume() * 100.0 );

        mUi.SeptalPassiveStifnessSpinBox->setValue( mData.getHeart().getVentricles().getSeptum().getRelSfPas() * 100.0 );
        mUi.SeptalContractilitySpinBox->setValue(   mData.getHeart().getVentricles().getSeptum().getRelSfAct() * 100.0 );
        mUi.SeptalMassSpinBox->setValue(            mData.getHeart().getVentricles().getSeptum().getRelWallVolume() * 100.0 );

        mUi.RVpassiveStifnessSpinBox->setValue(     mData.getHeart().getVentricles().getRightVentricle().getRelSfPas() * 100.0 );
        mUi.RVcontractilitySpinBox->setValue(       mData.getHeart().getVentricles().getRightVentricle().getRelSfAct() * 100.0 );
        mUi.RVMassSpinBox->setValue(                mData.getHeart().getVentricles().getRightVentricle().getRelWallVolume() * 100.0 );


        // Atria parameters
        mUi.LApassiveStifnessSpinBox->setValue(     mData.getHeart().getAtria().getLeftAtrium().getRelSfPas() * 100.0 );
        mUi.LAcontractilitySpinBox->setValue(       mData.getHeart().getAtria().getLeftAtrium().getRelSfAct() * 100.0 );
        mUi.LAMassSpinBox->setValue(                mData.getHeart().getAtria().getLeftAtrium().getRelWallVolume() * 100.0 );

        mUi.RApassiveStifnessSpinBox->setValue(     mData.getHeart().getAtria().getRightAtrium().getRelSfPas() * 100.0 );
        mUi.RAcontractilitySpinBox->setValue(       mData.getHeart().getAtria().getRightAtrium().getRelSfAct() * 100.0 );
        mUi.RAMassSpinBox->setValue(                mData.getHeart().getAtria().getRightAtrium().getRelWallVolume() * 100.0 );


        // Values parameters
        mUi.mitralis_insuff->setValue(              100.0 * mData.getHeart().getValveLAv().getAreaLeakMin() / mData.getHeart().getValveLAv().getAreaOpenMax() );
        mUi.mitralis_stenosisspinbox->setValue(     100.0 * mData.getHeart().getValveLAv().getStenosis() );
        mUi.aorta_insuff->setValue(                 100.0 * mData.getHeart().getValveLArt().getAreaLeakMin() / mData.getHeart().getValveLArt().getAreaOpenMax() );
        mUi.aorta_stenosisSpinbox->setValue(        100.0 * mData.getHeart().getValveLArt().getStenosis() );
        mUi.tricus_stenosisSpinbox->setValue(       100.0 * mData.getHeart().getValveRAv().getStenosis() );
        mUi.tricus_insuff->setValue(                100.0 * mData.getHeart().getValveRAv().getAreaLeakMin() / mData.getHeart().getValveRAv().getAreaOpenMax() );
        mUi.pulmvalve_stenosisSpinbox->setValue(    100.0 * mData.getHeart().getValveRArt().getStenosis() );
        mUi.pulmvalve_insuff->setValue(             100.0 * mData.getHeart().getValveRArt().getAreaLeakMin() / mData.getHeart().getValveRArt().getAreaOpenMax() );


        // Vasculature parameters
        mUi.systemicArtStiffCoeffSpinBox->setValue(     mData.aorta().stiffnessWall() );
        mUi.systemicVenousStiffCoeffSpinBox->setValue(  mData.systemicVein().stiffnessWall() );

        mUi.pulmArtStiffCoeffSpinBox->setValue(     mData.pulmonaryArtery().stiffnessWall());
        mUi.pulmVenousStiffCoeffSpinBox->setValue(  mData.pulmonaryVeins().stiffnessWall());
        mUi.pulmPresDrop->setValue(                 mData.toPressure(mData.getPulmDrop()));


        // Shunts parameters
        mUi.ASDDiameter->setValue(                  sqrt( mData.heart().valveASD().getAreaOpenMax() * 4.0 / M_PI) * 1000.0 );
        mUi.VSDDiameter->setValue(                  sqrt( mData.heart().valveVSD().getAreaOpenMax() * 4.0 / M_PI) * 1000.0 );
        mUi.ductDiameter->setValue(                 sqrt( mData.heart().valveDUCT().getAreaOpenMax() * 4.0 / M_PI) * 1000.0 );


        // ADAPT
        QColor  activeColor(  Qt::red  );
        QColor  passiveColor( Qt::gray );
        bool    inAdapt     = mData.getAdaptStatus() != NOADAPTATION;  //mUi.startAdapt->isChecked();
        bool    doRest      = mUi.adaptRest->isChecked();
        bool    doExercise  = mUi.adaptExercise->isChecked();

        switch ( mData.getAdaptStatus() )
        {
        case RestAdaptation:
            mUi.restActiveLabel->setPalette(        activeColor  );
            mUi.exerciseActiveLabel->setPalette(    passiveColor );
            mUi.transitionActiveLabel->setPalette(  passiveColor );

            if ( !mUi.startAdapt->isChecked() )
            {
                mUi.startAdapt->setChecked( true );
            }
            break;
        case ExerciseAdapation:
            mUi.restActiveLabel->setPalette(        passiveColor );
            mUi.exerciseActiveLabel->setPalette(    activeColor  );
            mUi.transitionActiveLabel->setPalette(  passiveColor );

            if ( !mUi.startAdapt->isChecked() )
            {
                mUi.startAdapt->setChecked( true );
            }
            break;
        case TransitionAdaptation:
            mUi.restActiveLabel->setPalette(        passiveColor );
            mUi.exerciseActiveLabel->setPalette(    passiveColor );

            if ( !mUi.startAdapt->isChecked() )
            {
                mUi.startAdapt->setChecked( true );
            }
            mUi.transitionActiveLabel->setPalette(  activeColor  );
            break;
        default:
            mUi.restActiveLabel->setPalette(        passiveColor );
            mUi.exerciseActiveLabel->setPalette(    passiveColor );
            mUi.transitionActiveLabel->setPalette(  passiveColor );

            if ( mUi.startAdapt->isChecked() )
            {
                mUi.startAdapt->setChecked( false );
            }
            break;
        }

        mUi.homeostaticGroup->setEnabled( !inAdapt );
        mUi.adatptHeartRate->setText( QString( "heart rate: %1 bpm" ).arg( beattime ) );

        mUi.restRate->setEnabled(       doRest );
        mUi.restCO->setEnabled(         doRest );
        mUi.exerciseRate->setEnabled(   doExercise );
        mUi.exerciseCO->setEnabled(     doExercise );

        mUi.startAdapt->setEnabled( mUi.adaptRest->isChecked() || mUi.adaptExercise->isChecked() );

        // Reset the temporary values now that the real values are processed and know again
        mTmpValue = 0.0;

        mBusy = false;
    }
}

void PParamView::on_heartRateDial_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        emit( updateHeartRateInterval( qRound(60000.0 / arg1 ) ) );
    }
}


void PParamView::on_mitralis_stenosisspinbox_editingFinished()
{
    if ( !mBusy )
    {
        double val = mUi.mitralis_stenosisspinbox->value() / 100.0;
        mData.heart().valveLAv().setStenosis(val);
        mDoUpdate = true;
        TCirculation specs;
        specs.copyUserSpecifications(mData);
        emit( updateSpecs(specs) );
    }
}


void PParamView::on_aorta_stenosisSpinbox_editingFinished()
{
    if ( !mBusy )
    {
        double val = mUi.aorta_stenosisSpinbox->value()/100.0;
        mData.heart().valveLArt().setStenosis(val);
        mDoUpdate = true;

        TCirculation specs;
        specs.copyUserSpecifications(mData);
        emit( updateSpecs(specs) );
    }
}


void PParamView::on_pressureControlCheckbox_clicked(bool checked)
{
    if ( !mBusy )
    {
        mBusy = true;

        mData.setPressureControl(checked);
        mDoUpdate = true;

        TCirculation specs;
        specs.copyUserSpecifications(mData);
        emit( updateSpecs(specs) );
        mBusy = false;
    }
}


void PParamView::on_ASDDiameter_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        double A = M_PI *(arg1 *arg1) / 4000000.0; // to convert to m^2
        mData.heart().valveASD().setAreaLeakMin( A );
        mData.heart().valveASD().setAreaOpenMax( A );
        mDoUpdate = true;

        TCirculation specs;
        specs.copyUserSpecifications(mData);
        emit( updateSpecs(specs) );
    }
}

void PParamView::on_VSDDiameter_valueChanged(double arg1)
{

    if ( !mBusy )
    {
        double A = M_PI *(arg1 *arg1) / 4000000.0; // to convert to m^2
        mData.heart().valveVSD().setAreaLeakMin( A );
        mData.heart().valveVSD().setAreaOpenMax( A );
        mDoUpdate = true;

        TCirculation specs;
        specs.copyUserSpecifications(mData);
        emit( updateSpecs( specs ) );    }
}

void PParamView::on_ductDiameter_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        double A = M_PI *(arg1 *arg1) / 4000000.0; // to convert to m^2
        mData.heart().valveDUCT().setAreaLeakMin( A );
        mData.heart().valveDUCT().setAreaOpenMax( A );
        mDoUpdate = true;

        TCirculation specs;
        specs.copyUserSpecifications(mData);
        emit( updateSpecs(specs) );
    }
}

void PParamView::on_pulmPresDrop_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        mData.setPulmDrop(mData.fromPressure(arg1));
        mDoUpdate = true;

        TCirculation specs;
        specs.copyUserSpecifications(mData);
        emit( updateSpecs(specs) );
    }
}


void PParamView::on_COSpinBox_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        mData.setVenousReturn( mData.fromFlow( arg1 ) );
        mDoUpdate = true;
        TCirculation specs;
        specs.copyUserSpecifications( mData );
        emit( updateSpecs( specs ) );
    }
}


void PParamView::on_meanAPSpinBox_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        mData.setMeanArterialPressure( mData.fromPressure( arg1 ) );
        mDoUpdate = true;
        TCirculation specs;
        specs.copyUserSpecifications( mData );
        emit( updateSpecs( specs ) );
    }
}





void PParamView::on_tricus_insuff_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        double maxValue = mData.heart().valveRAv().getAreaOpenMax();
        mData.heart().valveRAv().setAreaLeakMin( arg1 * maxValue / 100.0 );
        mDoUpdate = true;
        TCirculation specs;
        specs.copyUserSpecifications(mData);
        emit( updateSpecs(specs) );
    }
}


void PParamView::on_aorta_insuff_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        double maxValue = mData.heart().valveLArt().getAreaOpenMax();
        mData.heart().valveLArt().setAreaLeakMin( arg1 * maxValue / 100.0 );
        mDoUpdate = true;
        TCirculation specs;
        specs.copyUserSpecifications(mData);
        emit( updateSpecs(specs) );
    }
}

void PParamView::on_mitralis_insuff_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        double maxValue = mData.heart().valveLAv().getAreaOpenMax();
        mData.heart().valveLAv().setAreaLeakMin( arg1 * maxValue / 100.0 );
        mDoUpdate = true;
        TCirculation specs;
        specs.copyUserSpecifications(mData);
        emit( updateSpecs(specs) );
    }
}

void PParamView::on_pulmvalve_insuff_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        double maxValue = mData.heart().valveRArt().getAreaOpenMax();
        mData.heart().valveRArt().setAreaLeakMin( arg1 * maxValue / 100.0 );
        mDoUpdate = true;
        TCirculation specs;
        specs.copyUserSpecifications(mData);
        emit( updateSpecs(specs) );
    }
}


void PParamView::on_aorta_stenosisSpinbox_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        double val = arg1 / 100.0;
        mData.heart().valveLArt().setStenosis(val);
        mDoUpdate = true;
        TCirculation specs;
        specs.copyUserSpecifications(mData);
        emit( updateSpecs(specs) );
    }
}

void PParamView::on_mitralis_stenosisspinbox_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        double val = arg1 / 100.0;
        mData.heart().valveLAv().setStenosis(val);
        mDoUpdate = true;

        TCirculation specs;
        specs.copyUserSpecifications(mData);
        emit( updateSpecs(specs) );
    }
}



void PParamView::on_pulmvalve_stenosisSpinbox_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        double val = arg1 / 100.0;
        mData.heart().valveRArt().setStenosis(val);
        mDoUpdate = true;

        TCirculation specs;
        specs.copyUserSpecifications(mData);
        emit( updateSpecs(specs) );
    }
}

void PParamView::on_tricus_stenosisSpinbox_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        double val = arg1 / 100.0;
        mData.heart().valveRAv().setStenosis(val);
        mDoUpdate = true;

        TCirculation specs;
        specs.copyUserSpecifications(mData);
        emit( updateSpecs(specs) );
    }
}


void PParamView::on_startAdapt_clicked(bool checked)
{
    if ( checked )
    {

        int nTransitionBeats = mUi.nrBeats->value();
        if ( !mUi.transitionCheckBox->isChecked() )
        {
            nTransitionBeats = 0;
        }

        mSimCirc.updateAdaptCirc(mData.fromFlow(mUi.restCO->value()),
                                 mData.fromFlow(mUi.exerciseCO->value()),
                                 qRound(60000.0 / mUi.restRate->value()),
                                 qRound(60000.0 / mUi.exerciseRate->value()),
                                 nTransitionBeats,
                                 mUi.nrRestCyclesSpinBox->value(),
                                 mUi.nrExerciseCyclesSpinBox->value(),
                                 mUi.adaptRest->isChecked(),
                                 mUi.adaptExercise->isChecked());
    }
    else
    {
        mSimCirc.stopAdaptCirc();
    }
}


void PParamView::updateAdapt()
{
    if ( mUi.startAdapt->isChecked())
    {
        int nTransitionBeats = mUi.nrBeats->value();
        if ( !mUi.transitionCheckBox->isChecked() )
        {
            nTransitionBeats = 0;
        }

        mSimCirc.updateAdaptCirc(mData.fromFlow(mUi.restCO->value()),
                                 mData.fromFlow(mUi.exerciseCO->value()),
                                 qRound(60000.0 / mUi.restRate->value()),
                                 qRound(60000.0 / mUi.exerciseRate->value()),
                                 nTransitionBeats,
                                 mUi.nrRestCyclesSpinBox->value(),
                                 mUi.nrExerciseCyclesSpinBox->value(),
                                 mUi.adaptRest->isChecked(),
                                 mUi.adaptExercise->isChecked());
    }
}

void PParamView::on_adaptExercise_clicked()
{
    if ( !mBusy )
    {
       updateAdapt();
    }
}

void PParamView::on_adaptRest_clicked()
{
    if ( !mBusy )
    {
        updateAdapt();
    }
}

void PParamView::on_exerciseRate_valueChanged(double arg1)
{
    if ( !mBusy )
    {
       updateAdapt();
    }
}

void PParamView::on_exerciseCO_valueChanged(double arg1)
{
    if ( !mBusy )
    {
       updateAdapt();
    }
}

void PParamView::on_restCO_valueChanged(double arg1)
{
    if ( !mBusy )
    {
       updateAdapt();
    }
}

void PParamView::on_restRate_valueChanged(double arg1)
{
    if ( !mBusy )
    {
       updateAdapt();
    }
}

void PParamView::on_nrBeats_valueChanged(int arg1)
{
    if ( !mBusy )
    {
       updateAdapt();
    }
}


void PParamView::on_nrRestCyclesSpinBox_valueChanged(int arg1)
{
    if ( !mBusy )
    {
       updateAdapt();
    }
}

void PParamView::on_nrExerciseCyclesSpinBox_valueChanged(int arg1)
{
    if ( !mBusy )
    {
       updateAdapt();
    }
}


void PParamView::on_pulmArtStiffCoeffSpinBox_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        mData.pulmonaryArtery().setStiffnessWall( arg1 );
        mDoUpdate = true;
        TCirculation specs;
        specs.copyUserSpecifications( mData );
        emit( updateSpecs( specs ) );
    }
}

void PParamView::on_pulmVenousStiffCoeffSpinBox_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        mData.pulmonaryVeins().setStiffnessWall( arg1 );
        mDoUpdate = true;
        TCirculation specs;
        specs.copyUserSpecifications( mData );
        emit( updateSpecs( specs ) );
    }
}

void PParamView::on_systemicArtStiffCoeffSpinBox_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        mData.aorta().setStiffnessWall( arg1 );
        mDoUpdate = true;
        TCirculation specs;
        specs.copyUserSpecifications( mData );
        emit( updateSpecs( specs ) );
    }
}

void PParamView::on_systemicVenousStiffCoeffSpinBox_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        mData.systemicVein().setStiffnessWall( arg1 );
        mDoUpdate = true;
        TCirculation specs;
        specs.copyUserSpecifications( mData );
        emit( updateSpecs( specs ) );
    }
}


////////////// TAB: Ventricles.

////////////// TAB: Ventricles. Items: Passive stiffness

void PParamView::on_chkVentriclesPassStiff_clicked( bool checked )
{
    mUi.SeptalPassiveStifnessSpinBox->setEnabled( !checked );
    mUi.RVpassiveStifnessSpinBox->setEnabled( !checked );
}


void PParamView::on_LVpassiveStifnessSpinBox_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        TCirculation specs;

        if ( mUi.chkVentriclesPassStiff->isChecked() )
        {
            double diff;

            if ( mTmpValue == 0.0 )
            {
                diff        = arg1 - mData.getHeart().getVentricles().getLeftVentricle().getRelSfPas() * 100.0;
                mTmpValue   = arg1;
            }
            else
            {
                diff        = arg1 - mTmpValue;
                mTmpValue   = arg1;
            }

            mUi.SeptalPassiveStifnessSpinBox->setValue( mUi.SeptalPassiveStifnessSpinBox->value() + diff );
            mData.heart().ventricles().septum().setRelSfPas( mUi.SeptalPassiveStifnessSpinBox->value() / 100  );

            mUi.RVpassiveStifnessSpinBox->setValue( mUi.RVpassiveStifnessSpinBox->value() + diff );
            mData.heart().ventricles().rightVentricle().setRelSfPas( mUi.RVpassiveStifnessSpinBox->value() / 100  );
        }

        mData.heart().ventricles().leftVentricle().setRelSfPas( arg1 / 100  );

        mDoUpdate = true;

        specs.copyUserSpecifications( mData );

        emit( updateSpecs( specs ) );
    }
}

void PParamView::on_SeptalPassiveStifnessSpinBox_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        mData.heart().ventricles().septum().setRelSfPas( arg1 / 100  );

        mDoUpdate = true;
        TCirculation specs;
        specs.copyUserSpecifications( mData );
        emit( updateSpecs( specs ) );
    }
}


void PParamView::on_RVpassiveStifnessSpinBox_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        mData.heart().ventricles().rightVentricle().setRelSfPas( arg1 / 100  );

        mDoUpdate = true;
        TCirculation specs;
        specs.copyUserSpecifications( mData );
        emit( updateSpecs( specs ) );
    }
}




// TAB: Ventricles. Items: Contractility

void PParamView::on_chkVentriclesContr_clicked( bool checked )
{
    mUi.SeptalContractilitySpinBox->setEnabled( !checked );
    mUi.RVcontractilitySpinBox->setEnabled( !checked );
}


void PParamView::on_LVcontractilitySpinBox_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        TCirculation specs;

        if ( mUi.chkVentriclesContr->isChecked() )
        {
            double diff;

            if ( mTmpValue == 0.0 )
            {
                diff        = arg1 - mData.getHeart().getVentricles().getLeftVentricle().getRelSfAct() * 100.0;
                mTmpValue   = arg1;
            }
            else
            {
                diff        = arg1 - mTmpValue;
                mTmpValue   = arg1;
            }

            mUi.SeptalContractilitySpinBox->setValue( mUi.SeptalContractilitySpinBox->value() + diff );
            mData.heart().ventricles().septum().setRelSfAct( mUi.SeptalContractilitySpinBox->value() / 100  );

            mUi.RVcontractilitySpinBox->setValue( mUi.RVcontractilitySpinBox->value() + diff );
            mData.heart().ventricles().rightVentricle().setRelSfAct( mUi.RVcontractilitySpinBox->value() / 100  );
        }

        mData.heart().ventricles().leftVentricle().setRelSfAct( arg1 / 100.0 );

        mDoUpdate = true;

        specs.copyUserSpecifications( mData );

        emit( updateSpecs( specs ) );
    }
}


void PParamView::on_SeptalContractilitySpinBox_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        mData.heart().ventricles().septum().setRelSfAct( arg1 / 100.0 );
        mDoUpdate = true;
        TCirculation specs;
        specs.copyUserSpecifications( mData );
        emit( updateSpecs( specs ) );
    }
}


void PParamView::on_RVcontractilitySpinBox_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        mData.heart().ventricles().rightVentricle().setRelSfAct( arg1 / 100.0 );
        mDoUpdate = true;
        TCirculation specs;
        specs.copyUserSpecifications( mData );
        emit( updateSpecs( specs ) );
    }
}





// TAB: Ventricles. Items: Mass

void PParamView::on_chkVentriclesMass_clicked( bool checked )
{
    mUi.SeptalMassSpinBox->setEnabled( !checked );
    mUi.RVMassSpinBox->setEnabled( !checked );
}


void PParamView::on_LVMassSpinBox_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        TCirculation specs;

        if ( mUi.chkVentriclesMass->isChecked() )
        {
            double diff;

            if ( mTmpValue == 0.0 )
            {
                diff        = arg1 - mData.getHeart().getVentricles().getLeftVentricle().getRelWallVolume() * 100.0;
                mTmpValue   = arg1;
            }
            else
            {
                diff        = arg1 - mTmpValue;
                mTmpValue   = arg1;
            }

            mUi.SeptalMassSpinBox->setValue( mUi.SeptalMassSpinBox->value() + diff );
            mData.heart().ventricles().septum().setRelWallVolume( mUi.SeptalMassSpinBox->value() / 100  );

            mUi.RVMassSpinBox->setValue( mUi.RVMassSpinBox->value() + diff );
            mData.heart().ventricles().rightVentricle().setRelWallVolume( mUi.RVMassSpinBox->value() / 100  );
        }

        mData.heart().ventricles().leftVentricle().setRelWallVolume( arg1 / 100.0 );

        mDoUpdate = true;

        specs.copyUserSpecifications( mData );

        emit( updateSpecs( specs ) );
    }
}


void PParamView::on_SeptalMassSpinBox_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        mData.heart().ventricles().septum().setRelWallVolume( arg1 / 100.0 );
        mDoUpdate = true;
        TCirculation specs;
        specs.copyUserSpecifications( mData );
        emit( updateSpecs( specs ) );
    }
}


void PParamView::on_RVMassSpinBox_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        mData.heart().ventricles().rightVentricle().setRelWallVolume( arg1 / 100.0 );
        mDoUpdate = true;
        TCirculation specs;
        specs.copyUserSpecifications( mData );
        emit( updateSpecs( specs ) );
    }
}






/////////////ATRIA////////////////////////

// TAB: Atria. Items: Passive stiffness

void PParamView::on_chkAtriaPassStiff_clicked( bool checked )
{
    mUi.RApassiveStifnessSpinBox->setEnabled( !checked );
}


void PParamView::on_LApassiveStifnessSpinBox_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        TCirculation specs;

        if ( mUi.chkAtriaPassStiff->isChecked() )
        {
            double diff;

            if ( mTmpValue == 0.0 )
            {
                diff        = arg1 - mData.getHeart().getAtria().getLeftAtrium().getRelSfPas() * 100.0;
                mTmpValue   = arg1;
            }
            else
            {
                diff        = arg1 - mTmpValue;
                mTmpValue   = arg1;
            }

            mUi.RApassiveStifnessSpinBox->setValue( mUi.RApassiveStifnessSpinBox->value() + diff );
            mData.heart().atria().rightAtrium().setRelSfPas( mUi.RApassiveStifnessSpinBox->value() / 100  );
        }

        mData.heart().atria().leftAtrium().setRelSfPas( arg1 / 100  );

        mDoUpdate = true;

        specs.copyUserSpecifications( mData );

        emit( updateSpecs( specs ) );
    }
}


void PParamView::on_RApassiveStifnessSpinBox_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        mData.heart().atria().rightAtrium().setRelSfPas( arg1 / 100  );

        mDoUpdate = true;
        TCirculation specs;
        specs.copyUserSpecifications( mData );
        emit( updateSpecs( specs ) );
    }
}





// TAB: Atria. Items: Contractility

void PParamView::on_chkAtriaContr_clicked( bool checked )
{
    mUi.RAcontractilitySpinBox->setEnabled( !checked );
}


void PParamView::on_LAcontractilitySpinBox_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        TCirculation specs;

        if ( mUi.chkAtriaContr->isChecked() )
        {
            double diff;

            if ( mTmpValue == 0.0 )
            {
                diff        = arg1 - mData.getHeart().getAtria().getLeftAtrium().getRelSfAct() * 100.0;
                mTmpValue   = arg1;
            }
            else
            {
                diff        = arg1 - mTmpValue;
                mTmpValue   = arg1;
            }

            mUi.RAcontractilitySpinBox->setValue( mUi.RAcontractilitySpinBox->value() + diff );
            mData.heart().atria().rightAtrium().setRelSfAct( mUi.RAcontractilitySpinBox->value() / 100  );
        }

        mData.heart().atria().leftAtrium().setRelSfAct( arg1 / 100.0 );

        mDoUpdate = true;

        specs.copyUserSpecifications( mData );

        emit( updateSpecs( specs ) );
    }
}


void PParamView::on_RAcontractilitySpinBox_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        mData.heart().atria().rightAtrium().setRelSfAct( arg1 / 100.0 );

        mDoUpdate = true;
        TCirculation specs;
        specs.copyUserSpecifications( mData );
        emit( updateSpecs( specs ) );
    }
}





// TAB: Atria. Items: Mass

void PParamView::on_chkAtriaMass_clicked( bool checked )
{
    mUi.RAMassSpinBox->setEnabled( !checked );
}


void PParamView::on_LAMassSpinBox_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        TCirculation specs;

        if ( mUi.chkAtriaMass->isChecked() )
        {
            double diff;

            if ( mTmpValue == 0.0 )
            {
                diff        = arg1 - mData.getHeart().getAtria().getLeftAtrium().getRelWallVolume() * 100.0;
                mTmpValue   = arg1;
            }
            else
            {
                diff        = arg1 - mTmpValue;
                mTmpValue   = arg1;
            }

            mUi.RAMassSpinBox->setValue( mUi.RAMassSpinBox->value() + diff );
            mData.heart().atria().rightAtrium().setRelWallVolume( mUi.RAMassSpinBox->value() / 100  );
        }

        mData.heart().atria().leftAtrium().setRelWallVolume( arg1 / 100.0 );

        mDoUpdate = true;

        specs.copyUserSpecifications( mData );

        emit( updateSpecs( specs ) );
    }
}


void PParamView::on_RAMassSpinBox_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        mData.heart().atria().rightAtrium().setRelWallVolume( arg1 / 100.0 );
        mDoUpdate = true;
        TCirculation specs;
        specs.copyUserSpecifications( mData );
        emit( updateSpecs( specs ) );
    }
}





void PParamView::on_AVdelaySpinBox_valueChanged(double arg1)
{
    if ( !mBusy )
    {
        mData.heart().ventricles().setActivationDelay(arg1 / 1000.0);
        mDoUpdate = true;
        TCirculation specs;
        specs.copyUserSpecifications( mData );
        emit( updateSpecs( specs ) );

    }
}

void PParamView::copyToClipBoard(void)
{
    QPixmap     pixmap( width(), height() );
    QClipboard* cb = QApplication::clipboard();

    cb->setPixmap( pixmap.grabWidget(this), QClipboard::Clipboard );
}


