/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: pparamview.h 923 2013-11-11 14:03:15Z eelco $
 *
 */

#ifndef __PPARAMVIEW_H
#define __PPARAMVIEW_H

#include <QWidget>

#include "ucirculation.h"
#include "msimulatecirculation.h"
#include "ui_PParamView.h"


namespace Peacs
{
    namespace CircAdapt
    {
        class PParamView : public QWidget
        {
            Q_OBJECT

        public:

            /** @brief Initialyzer constructor
              *
              * @param data Reference to the user circulation data store object
              */
            explicit PParamView( UCirculation & data, MSimulateCirculation & simClass, QWidget * parent = 0 );


            /** @brief
              */
            virtual ~PParamView( );


        signals:

            /** @brief
              */
            void updateHeartRateInterval( int );

            /** @brief
              */
            void updateSpecs( const TCirculation & specs );


        public slots:

            /** @brief Is used to initialyze the content
              * This function will call updateInfo() to perform the initial synchronisation. Also 'stops' the adaptation mode
              */
            void Initialize( void );


            /** @brief Is used to synchronize the shown parameter with the parameters in the simulation model
              */
            void updateInfo( void );


            /** @brief
              */
            void specsHaveBeenCopied();

            void copyToClipBoard(void);

            /** @brief slot for the options dialog
              * Reads info from the settings file
              * For the meaning of the parameter @see POptionsDialog::mChangedItems
              *
              * @param changed indicated what options have changed. This is a 32 bit-flag value
              */
            void updateAllSettings( int changed );


        private slots:

            /** @brief
              */
            void on_mitralis_insuff_valueChanged(double arg1);


            /** @brief
              */
            void on_aorta_insuff_valueChanged(double arg1);


            /** @brief
              */
            void on_mitralis_stenosisspinbox_editingFinished();


            /** @brief
              */
            void on_aorta_stenosisSpinbox_editingFinished();


            /** @brief
              */
            void on_pressureControlCheckbox_clicked(bool checked);


            /** @brief
              */
            void on_aorta_stenosisSpinbox_valueChanged(double arg1);


            /** @brief
              */
            void on_mitralis_stenosisspinbox_valueChanged(double arg1);


            /** @brief
              */
            void on_ASDDiameter_valueChanged(double arg1);


            /** @brief
              */
            void on_VSDDiameter_valueChanged(double arg1);


            /** @brief
              */
            void on_ductDiameter_valueChanged(double arg1);


            /** @brief
              */
            void on_pulmPresDrop_valueChanged(double arg1);


            /** @brief
              */
            void on_COSpinBox_valueChanged(double arg1);


            /** @brief
              */
            void on_pulmArtStiffCoeffSpinBox_valueChanged(double arg1);


            /** @brief
              */
            void on_adaptExercise_clicked();


            /** @brief
              */
            void on_adaptRest_clicked();


            /** @brief
              */
            void on_tricus_insuff_valueChanged(double arg1);


            /** @brief
              */
            void on_pulmvalve_insuff_valueChanged(double arg1);


            /** @brief
              */
            void on_pulmvalve_stenosisSpinbox_valueChanged(double arg1);


            /** @brief
              */
            void on_tricus_stenosisSpinbox_valueChanged(double arg1);


            /** @brief
              */
            void on_heartRateDial_valueChanged(double arg1);


            /** @brief
              */
            void on_startAdapt_clicked(bool checked);


            /** @brief
              */
            void updateAdapt();


            /** @brief
              */
            void on_exerciseRate_valueChanged(double arg1);


            /** @brief
              */
            void on_exerciseCO_valueChanged(double arg1);


            /** @brief
              */
            void on_restCO_valueChanged(double arg1);


            /** @brief
              */
            void on_restRate_valueChanged(double arg1);


            /** @brief
              */
            void on_nrBeats_valueChanged(int arg1);


            /** @brief
              */
            void on_meanAPSpinBox_valueChanged(double arg1);


            /** @brief
              */
            void on_nrRestCyclesSpinBox_valueChanged(int arg1);


            /** @brief
              */
            void on_nrExerciseCyclesSpinBox_valueChanged(int arg1);


            /** @brief
              */
            void on_pulmVenousStiffCoeffSpinBox_valueChanged(double arg1);


            /** @brief
              */
            void on_systemicArtStiffCoeffSpinBox_valueChanged(double arg1);


            /** @brief
              */
            void on_systemicVenousStiffCoeffSpinBox_valueChanged(double arg1);




            // TAB: Ventricles. Items: Passive stiffness

            /** @brief
              */
            void on_chkVentriclesPassStiff_clicked( bool checked );


            /** @brief
              */
            void on_LVpassiveStifnessSpinBox_valueChanged(double arg1);


            /** @brief
              */
            void on_SeptalPassiveStifnessSpinBox_valueChanged(double arg1);


            /** @brief
              */
            void on_RVpassiveStifnessSpinBox_valueChanged(double arg1);




            // TAB: Ventricles. Items: Contractility

            /** @brief
              */
            void on_chkVentriclesContr_clicked( bool checked );


            /** @brief
              */
            void on_LVcontractilitySpinBox_valueChanged(double arg1);


            /** @brief
              */
            void on_SeptalContractilitySpinBox_valueChanged(double arg1);


            /** @brief
              */
            void on_RVcontractilitySpinBox_valueChanged(double arg1);




            // TAB: Ventricles. Items: Mass

            /** @brief
              */
            void on_chkVentriclesMass_clicked( bool checked );


            /** @brief
              */
            void on_LVMassSpinBox_valueChanged(double arg1);


            /** @brief
              */
            void on_SeptalMassSpinBox_valueChanged(double arg1);


            /** @brief
              */
            void on_RVMassSpinBox_valueChanged(double arg1);






            // TAB: Atria. Items: Passive stiffness

            /** @brief
              */
            void on_chkAtriaPassStiff_clicked( bool checked );


            /** @brief
              */
            void on_LApassiveStifnessSpinBox_valueChanged(double arg1);


            /** @brief
              */
            void on_RApassiveStifnessSpinBox_valueChanged(double arg1);




            // TAB: Atria. Items: Contractility

            /** @brief
              */
            void on_chkAtriaContr_clicked( bool checked );


            /** @brief
              */
            void on_LAcontractilitySpinBox_valueChanged(double arg1);


            /** @brief
              */
            void on_RAcontractilitySpinBox_valueChanged(double arg1);




            // TAB: Atria. Items: Mass

            /** @brief
              */
            void on_chkAtriaMass_clicked( bool checked );


            /** @brief
              */
            void on_LAMassSpinBox_valueChanged(double arg1);


            /** @brief
              */
            void on_RAMassSpinBox_valueChanged(double arg1);




            /** @brief
              */
            void on_AVdelaySpinBox_valueChanged(double arg1);

        private:

            /** @brief
              */
            UCirculation & mData;


            /** @brief
              */
            MSimulateCirculation & mSimCirc;


            /** @brief
              */
            Ui::PParamView mUi;


            /** @brief
              */
            bool mBusy;


            /** @brief
              */
            bool mDoUpdate;


            /** @brief This value is used when adapting the vetrical or atria tissue propperties
              * When quickly changing the tissue proppeerties, for example when using the scroll wheel,
              * the value change can be mangled up with because mData is a, more or less, global value that
              * can change without this class being notified. This results in different behaviour then
              * expected when the "All walls" checkbox is checked. The 2 or 3 spin boxes can get out of sync
              * This value is used to temporary store the value to use in a rapid sequence of changes so
              * sychronisation can be maintained
              */
            double mTmpValue;
        };
    }
}
#endif // __PPARAMVIEW_H
