/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: ppvview.cpp 1173 2014-03-21 14:12:14Z eelco $
 *
 */


#include "ppvview.h"
#include "pcircpvloop.h"
#include "PCircAdaptDefines.h"
#include "PSettings.h"
#include "PCircAdaptOptionsDialog.h"
#include "PPvScene.h"

//#include <QDebug>

using namespace Peacs::Support;
using namespace Peacs::CircAdapt;


PPVView::PPVView( UCirculation & data, QWidget * parent ):
    PGraphicsSignalView( parent ),
    mData( data )
{
    Support::PSettings  pSetting;
    QSettings &         settings    = pSetting.getSettings();
    QColor              bgColor( settings.value( SIGNAL_COLOR_BG, DEFAULTBACKGROUNDCOLOR ).value<QColor>() );
    PPvScene *          scenePtr    = new PPvScene( 1, 1, this );

    setBackGroundColor( bgColor );
    setScene( scenePtr );

//    scenePtr->setItemIndexMethod( QGraphicsScene::NoIndex );
////    scenePtr->setItemIndexMethod( QGraphicsScene::BspTreeIndex );

    initialize();
}


PPVView::~PPVView( void )
{
}


void PPVView::initialyzeItems( void )
{
    PPVloop * pvloop = new PPVloop( mData, 0, 0 );

//    pvloop->setLabelCurr( subSignal_type_str[ CURRENT_PLOT ] );
//    pvloop->setLabelInit( subSignal_type_str[ INITIAL_PLOT ] );
//    pvloop->setLabelSaved( subSignal_type_str[ SAVED_PLOT ]);

    static_cast<PGraphMatrixScene *>( scene() )->addItem( PGraphMatrixScene::matrixItem, pvloop );

    updateAllSettings( POptionsDialog::SIGNAL_FONT_CHANGED |
                       POptionsDialog::SIGNAL_COLOR_CHANGED |
                       POptionsDialog::GRID_CHANGED );
}


void PPVView::updateAllSettings( int changed )
{
    Support::PSettings                  pSetting;
    QSettings &                         settings    = pSetting.getSettings();
    POptionsDialog::ChangedOptionsType  changedType( changed );

    // For the meaning of the parameter POptionsDialog::mChangedItems
    if ( changedType.testFlag( POptionsDialog::SIGNAL_FONT_CHANGED ) )
    {
        QFont   labelFnt;
        QFont   scaleFnt;
        QFont   textFnt;

        labelFnt.fromString( settings.value( SIGNAL_FONT_LABEL, SIGNAL_FONT_LABEL_DEFAULT ).toString() );
        scaleFnt.fromString( settings.value( SIGNAL_FONT_SCALE, SIGNAL_FONT_SCALE_DEFAULT ).toString() );
        textFnt.fromString(  settings.value( SIGNAL_FONT_TEXT,  SIGNAL_FONT_TEXT_DEFAULT ).toString() );

        setFonts( textFnt, labelFnt, scaleFnt );
    }

    if ( changedType.testFlag( POptionsDialog::GRID_CHANGED ) )
    {
        bool ok;
        int  tmpI = settings.value( GRIDTYPE, GRIDTYPE_DEFAULT ).toInt( &ok );

        if ( ok )
        {
            setGrid( (Widget::Grid_type)tmpI );
        }
    }

    if ( changedType.testFlag( POptionsDialog::SIGNAL_COLOR_CHANGED ) )
    {
        QColor                  bgColor( settings.value( SIGNAL_COLOR_BG, DEFAULTBACKGROUNDCOLOR ).value<QColor>() );
        QList<QGraphicsItem*>   scene_items( scene()->items() );

        for ( int i = 0; i < scene_items.size() ; i++ )
        {
            PPVloop * pvplot = dynamic_cast<PPVloop *>( scene_items[i] );

            if ( pvplot != NULL )
            {
                pvplot->setBackGroundColor( bgColor );
            }
        }

        setBackGroundColor( bgColor );
    }
}


void PPVView::updateSceneForeGround()
{
    if ( !scene()->items().isEmpty() )
    {
        QList<QGraphicsItem*>   scene_items = scene()->items();
        PPVloop *               pvplot;

        for ( int i = 0; i < scene_items.size(); ++i )
        {
            pvplot = qobject_cast<PPVloop *>( (PPVloop *)scene_items[i] );

            if ( pvplot != NULL )
            {
                //pvplot->setCurrentTimeMarker( mData.getCurrentTimeMarker() );
                pvplot->setInitialTimeMarker( mData.getInitialTimeMarker() );
                pvplot->setSavedTimeMarker( mData.getSavedTimeMarker() );
            }
        }
    }

    scene()->update();
}


void PPVView::autoScale( void )
{
//qDebug() << "PPVView::autoScales called";

    static_cast<PGraphMatrixScene *>( scene() )->autoScale();
}
