/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: psignalview.cpp 932 2013-11-27 19:55:50Z eelco $
 *
 */


#include "psignalview.h"
#include "pcircsignal.h"
#include "PCircSignalDraw.h"
#include "PCircAdaptDefines.h"
#include "PSettings.h"
#include "PCircAdaptOptionsDialog.h"
#include "PSimpleTextItem.h"
#include "PCircSignalScene.h"

//#include <QDebug>

using namespace Peacs::Support;
using namespace Peacs::CircAdapt;

PSignalView::PSignalView( UCirculation &data, QWidget *parent ) :
    PGraphicsSignalView( parent ),
    mData( data )
{
    Support::PSettings  pSetting;
    QSettings &         settings    = pSetting.getSettings();
    QColor              bgColor( settings.value( SIGNAL_COLOR_BG, DEFAULTBACKGROUNDCOLOR ).value<QColor>() );
    PCircSignalScene *  scenePtr    = new PCircSignalScene( 4, 3, this );   // 4 columns used because the last columns spanns two columns

    setBackGroundColor( bgColor );

//    scenePtr->setItemIndexMethod( QGraphicsScene::NoIndex );
////    scenePtr->setItemIndexMethod( QGraphicsScene::BspTreeIndex );

    setScene( scenePtr );

    initialize();
}


PSignalView::~PSignalView( void )
{
}


void PSignalView::initialyzeItems( void )
{
    // The sequence in the following array defines the signal types shown from top
    // to bottom in the graphicsview. Items defined in the 'signal_type' enumeration must
    // be used
    signal_type             itemType[]      = { PRESSURE, VOLUME, FLOWVELOCITY };
    int                     itemCount       = 3;
    // The sequence in the following array defines the signal bub-types shown from
    // left to right. Items defined in the 'subSignal_type' enumeration must be used
    subSignal_type          itemSubType[]   = { INITIAL_PLOT, SAVED_PLOT, CURRENT_PLOT };
    int                     itemSubCount    = 3;
    PSimpleTextItem*        textItem        = NULL;
    PSignalPlotBase*        itemPtr         = NULL;
    PSignalPlotBase*        itemPtrs[ 3 ];  // Equal to itemSubCount
    Support::PSettings      pSetting;
    QSettings &             settings        = pSetting.getSettings();
    QColor                  bgColorGraph( settings.value( SIGNAL_COLOR_BG_GRAPH, DEFAULTBACKGROUNDCOLORGRAPH ).value<QColor>() );

    // First add the column headers
    for ( int subitem = 0; subitem < itemSubCount; subitem++ )
    {
        textItem = new PSimpleTextItem( "<center>" + subSignal_type_str[ itemSubType[ subitem ] ].toUpper() + "</center>",
                                        subitem,
                                        0 );

        if ( itemSubType[subitem] == CURRENT_PLOT )
        {
            textItem->setColumnSpan( 2 );
        }

        static_cast<PGraphMatrixScene *>( scene() )->addItem( PGraphMatrixScene::columnHeaderItem, textItem );
    }

    for ( int item = 0; item < itemCount; item++ )
    {
        // For each row add a row header
        textItem = new PSimpleTextItem( "<center>" + signal_type_str[ itemType[ item ] ] + "</center>",
                                        0,
                                        item );

        textItem->setRotation( -90 );
        static_cast<PGraphMatrixScene *>( scene() )->addItem( PGraphMatrixScene::rowHeaderItem, textItem );

        for ( int subitem = 0; subitem < itemSubCount; subitem++ )
        {
            if ( itemSubType[subitem] == CURRENT_PLOT )
            {
                itemPtr = new PCircSignalDraw( itemType[item],
                                               itemSubType[subitem],
                                               mData,
                                               0,
                                               3400,        // represents the number of samples we want to see
                                               subitem,
                                               item );

                itemPtr->setXfactor( 20 );
                itemPtr->setColumnSpan( 2 );
            }
            else
            {
                itemPtr = new PGraphicsCircSignal( itemType[item],
                                                   itemSubType[subitem],
                                                   mData,
                                                   0,
                                                   1700,    // represents the number of samples we want to see
                                                   subitem,
                                                   item );

                //itemPtr->setDoAdaptXAxisOnScroll( false );  // Disable use of the scroll-wheel to change X-axis range
                itemPtr->setLimmitAdaptXAxis( true );
                itemPtr->setBackGroundColor( bgColorGraph );
            }

            itemPtr->setXLabelFactor( CIRCMONTIMESTEP );

            itemPtrs[ subitem ] = itemPtr;

            // Only shown the Y-axis labels at the left side in the graphics view graphs
//            if ( itemSubType[subitem] != INITIAL_PLOT )
//            {
//                itemPtr->setShowYScaleTexts( false );
//            }

            // Only shown the X-axis labels at the bottom of the graphics view graphs
            if ( itemType[item] != FLOWVELOCITY )
            {
                itemPtr->setShowXScaleTexts( false );
            }

            static_cast<PGraphMatrixScene *>( scene() )->addItem( PGraphMatrixScene::matrixItem, itemPtr );
        }

        connect( itemPtrs[0], SIGNAL(showSignals(int)), itemPtrs[1], SLOT(updateShowSignals(int)) );
        connect( itemPtrs[0], SIGNAL(showSignals(int)), itemPtrs[2], SLOT(updateShowSignals(int)) );

        connect( itemPtrs[1], SIGNAL(showSignals(int)), itemPtrs[0], SLOT(updateShowSignals(int)) );
        connect( itemPtrs[1], SIGNAL(showSignals(int)), itemPtrs[2], SLOT(updateShowSignals(int)) );

        connect( itemPtrs[2], SIGNAL(showSignals(int)), itemPtrs[0], SLOT(updateShowSignals(int)) );
        connect( itemPtrs[2], SIGNAL(showSignals(int)), itemPtrs[1], SLOT(updateShowSignals(int)) );
    }

    // Make sure that all is initialyzed propperly
    updateAllSettings( POptionsDialog::SIGNAL_FONT_CHANGED |
                       POptionsDialog::SIGNAL_COLOR_CHANGED |
                       POptionsDialog::GRID_CHANGED );
}


void PSignalView::initializeView( void )
{
    signal_type             itemType[]      = { PRESSURE, VOLUME, FLOWVELOCITY };
    QList<QGraphicsItem*> & scene_items( static_cast<PGraphMatrixScene *>( scene() )->rowHeaderItems() );

    for ( int i = 0; i < scene_items.size() ; i++ )
    {
        PSimpleTextItem * itemPtr = dynamic_cast<PSimpleTextItem *>( scene_items[i] );

        if ( itemPtr != NULL )
        {
            Q_ASSERT( (0 <= itemPtr->getRow()) && (2 >= itemPtr->getRow()) );

            const QList < UCircAdaptSignal *> & mySignals   = mData.getSignals( itemType[ itemPtr->getRow() ] );

            if ( 0 < mySignals.count() )
            {
                QString unitStr = QString("[%1]").arg( mySignals[0]->getUnits() );

                if ( !itemPtr->toPlainText().contains( unitStr ) )
                {
                    QString text( "<center>" + signal_type_str[ itemType[ itemPtr->getRow() ] ] + " " + unitStr + "</center>" );

                    itemPtr->setHtml( text );
                }
            }
        }
    }

    static_cast<PGraphMatrixScene *>( scene() )->startUpdateTimer();

    PGraphicsSignalView::initializeView();
}


void PSignalView::updateAllSettings( int changed )
{
    Support::PSettings                  pSetting;
    QSettings &                         settings    = pSetting.getSettings();
    POptionsDialog::ChangedOptionsType  changedType( changed );

    // For the meaning of the parameter POptionsDialog::mChangedItems
    if ( changedType.testFlag( POptionsDialog::SIGNAL_FONT_CHANGED ) )
    {
        QFont   labelFnt;
        QFont   scaleFnt;
        QFont   textFnt;

        labelFnt.fromString( settings.value( SIGNAL_FONT_LABEL, SIGNAL_FONT_LABEL_DEFAULT ).toString() );
        scaleFnt.fromString( settings.value( SIGNAL_FONT_SCALE, SIGNAL_FONT_SCALE_DEFAULT ).toString() );
        textFnt.fromString(  settings.value( SIGNAL_FONT_TEXT,  SIGNAL_FONT_TEXT_DEFAULT ).toString() );

        setFonts( textFnt, labelFnt, scaleFnt );
    }

    if ( changedType.testFlag( POptionsDialog::GRID_CHANGED ) )
    {
        bool ok;
        int  tmpI = settings.value( GRIDTYPE, GRIDTYPE_DEFAULT ).toInt( &ok );

        if ( ok )
        {
            setGrid( (Widget::Grid_type)tmpI );
        }
    }

    if ( changedType.testFlag( POptionsDialog::SIGNAL_COLOR_CHANGED ) )
    {
        QColor                  bgColor( settings.value( SIGNAL_COLOR_BG, DEFAULTBACKGROUNDCOLOR ).value<QColor>() );
        QColor                  bgColorGraph( settings.value( SIGNAL_COLOR_BG_GRAPH, DEFAULTBACKGROUNDCOLORGRAPH ).value<QColor>() );
        QList<QGraphicsItem*>   scene_items( scene()->items() );

        for ( int i = 0; i < scene_items.size() ; i++ )
        {
            PGraphicsCircSignal * sigplot = dynamic_cast<PGraphicsCircSignal *>( scene_items[i] );

            if ( sigplot != NULL )
            {
                if ( sigplot->getSubType() == CURRENT_PLOT )
                {
                    sigplot->setBackGroundColor( bgColor );
                }
                else
                {
                    sigplot->setBackGroundColor( bgColorGraph );
                }
            }
        }

        setBackGroundColor( bgColor );
    }
}


void PSignalView::updateSceneForeGround()
{
//qDebug() << "updateSceneForeGround called";
    if ( !scene()->items().isEmpty() )
    {
        QList<QGraphicsItem*>   scene_items = scene()->items();
        PGraphicsCircSignal*    sigplot;

        for (int i = 0; i < scene_items.size(); ++i)
        {
            sigplot = qgraphicsitem_cast<PGraphicsCircSignal *>( scene_items[i] );

            if ( sigplot != NULL )
            {
                switch ( sigplot->getSubType() )
                {
                case CURRENT_PLOT:
                    sigplot->setTimeMarker( mData.getCurrentTimeMarker() );
                    break;
                case INITIAL_PLOT:
                    sigplot->setTimeMarker( mData.getInitialTimeMarker() );
                    break;
                case SAVED_PLOT:
                    sigplot->setTimeMarker( mData.getSavedTimeMarker() );
                    break;
                default:
                    Q_ASSERT_X( false, "Error", "Unknown sub-type specified in function PSignalView::updateSceneForeGround()" );
                    break;
                }
            }
        }
    }
}


void PSignalView::autoScale( void )
{
//qDebug() << "PSignalView::autoScales called";

    static_cast<PGraphMatrixScene *>( scene() )->autoScale();
}
