/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: psignalview.h 931 2013-11-27 19:03:51Z eelco $
 *
 */

#ifndef __PSIGNALVIEW_H
#define __PSIGNALVIEW_H


#include "pgraphicssignalview.h"
#include "ucirculation.h"


namespace Peacs
{
    namespace CircAdapt
    {
        class PSignalView : public Peacs::Support::PGraphicsSignalView
        {
            Q_OBJECT
        public:

            /** @brief TODO
              */
            explicit PSignalView( UCirculation & data, QWidget * parent = 0 );


            /** @brief TODO
              */
            virtual ~PSignalView( void );


            /** @brief Must be called to initialize this view with its items.
              * By calling this function the init() functions of all items contained
              * by this graphics view will be called. Thus initialization can be done
              * with any first data present
              * Implement this function in a derived class to add application specific
              * initialisation. Always call this base implementation
              */
            void initializeView( void );


        public slots:

            /** @brief slot for the options dialog
              * Reads info from the settings file
              * For the meaning of the parameter @see POptionsDialog::mChangedItems
              *
              * @param changed indicated what options have changed. This is a 32 bit-flag value
              */
            void updateAllSettings( int changed );


            /** @brief Is used to update 'foreground' information. In this case the time marker
              */
            void updateSceneForeGround();


            /** @brief Slot that will be passed on to the base implementation
              *
              * @see PGraphMatrixScene::autoScale()
              */
            void autoScale( void );


        protected:


            /** @brief This function will add the items to this view
              * This function will be called by the function initialize().
              */
            void initialyzeItems( void );


        private:            

            UCirculation & mData;
        };

    }
}


#endif // __PSIGNALVIEW_H
