/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: pvalueview.cpp 941 2013-12-01 13:09:05Z eelco $
 *
 */


#include <QClipboard>

#include "pvalueview.h"


using namespace Peacs::CircAdapt;


const int VAL_ID_CO             = 0;
const int VAL_ID_QS             = 1;
const int VAL_ID_QP             = 2;
const int VAL_ID_LAP            = 3;
const int VAL_ID_SBP_DBP_MAP    = 4;
const int VAL_ID_SVR            = 5;
const int VAL_ID_CVP            = 6;
const int VAL_ID_RAP            = 7;
const int VAL_ID_sPAP_dPAP_mPAP = 8;
const int VAL_ID_PVR            = 9;
const int VAL_ID_PVP            = 10;
//const int VAL_ID_Pulm_flow      = 11;
const int VAL_ID_ASD_flow       = 11;
const int VAL_ID_VSD_flow       = 12;
const int VAL_ID_PDA_flow       = 13;

const int COLUMN_0              = 0;
const int COLUMN_1              = 1;
const int COLUMN_2              = 2;


PValueView::PValueView(UCirculation &data, QWidget *parent):
    QWidget( parent ),
    mData(data),
    mUi(),
    mCo( 0.0 ),
    mQs( 0.0 ),
    mQp( 0.0 ),
    mPVP( 0.0 ),
    mMAP( 0.0 ),
    mCVP( 0.0 ),
    mLAP( 0.0 ),
    mRAP( 0.0 ),
    mPVR( 0.0 ),
    mSVR( 0.0 ),
    mPAP( 0.0 ),
//    mPulmFlow( 0.0 ),
    mASDFlow( 0.0 ),
    mVSDFlow( 0.0 ),
    mDuctFlow( 0.0 ),
    mTableItems()
{
    QStringList         headers;
    QTableWidgetItem *  tableItemPtr = NULL;

    mUi.setupUi( this );

    mUi.valuesList->setColumnCount( 3 );
    mUi.valuesList->setRowCount( 14 );
    mUi.valuesList->setSortingEnabled( false );
    mUi.valuesList->setEditTriggers( QAbstractItemView::NoEditTriggers );   // Disable edit the items

    headers.append( tr("Name") );
    headers.append( tr("Value") );
    headers.append( tr("Unit") );
    mUi.valuesList->setHorizontalHeaderLabels( headers );   // Headers are needed to enable correct sizing

    // VAL_ID_CO
    // Joost: CO => Qs:
    //- moet berekend worden als de gemiddelde flow door de instroomopening van het rechter atrium
    //- tooltip: mean blood flow through systemic capillaries
    tableItemPtr = new QTableWidgetItem( "CO" );
    tableItemPtr->setToolTip( "Cardiac output (Mean blood flow through aortic valve)" );
    mUi.valuesList->setItem( VAL_ID_CO, COLUMN_0, tableItemPtr );
    mTableItems.append( new QTableWidgetItem( ".." ) );
    mUi.valuesList->setItem( VAL_ID_CO, COLUMN_1, mTableItems.at( VAL_ID_CO ) );
    mUi.valuesList->setItem( VAL_ID_CO, COLUMN_2, new QTableWidgetItem( "l/min" ) );

    // VAL_ID_QS
    tableItemPtr = new QTableWidgetItem( "Qs" );
    tableItemPtr->setToolTip( "Systemic flow (mean blood flow through the systemic capillaries)" );
    mUi.valuesList->setItem( VAL_ID_QS, COLUMN_0, tableItemPtr );
    mTableItems.append( new QTableWidgetItem( ".." ) );
    mUi.valuesList->setItem( VAL_ID_QS, COLUMN_1, mTableItems.at( VAL_ID_QS ) );
    mUi.valuesList->setItem( VAL_ID_QS, COLUMN_2, new QTableWidgetItem( "l/min" ) );

    // VAL_ID_QP
    tableItemPtr = new QTableWidgetItem( "Qp" );
    tableItemPtr->setToolTip( "Pulmonary flow (mean blood flow through the pulmonary capillaries)" );
    mUi.valuesList->setItem( VAL_ID_QP, COLUMN_0, tableItemPtr );
    mTableItems.append( new QTableWidgetItem( ".." ) );
    mUi.valuesList->setItem( VAL_ID_QP, COLUMN_1, mTableItems.at( VAL_ID_QP ) );
    mUi.valuesList->setItem( VAL_ID_QP, COLUMN_2, new QTableWidgetItem( "l/min" ) );

    // VAL_ID_LAP
    tableItemPtr = new QTableWidgetItem( "LAP" );
    tableItemPtr->setToolTip( "Left atrial pressure" );
    mUi.valuesList->setItem( VAL_ID_LAP, COLUMN_0, tableItemPtr );
    mTableItems.append( new QTableWidgetItem( ".." ) );
    mUi.valuesList->setItem( VAL_ID_LAP, COLUMN_1, mTableItems.at( VAL_ID_LAP ) );
    mUi.valuesList->setItem( VAL_ID_LAP, COLUMN_2, new QTableWidgetItem( "mmHg" ) );

    // VAL_ID_SBP_DBP_MAP
    tableItemPtr = new QTableWidgetItem( "SBP / DBP / MAP" );
    tableItemPtr->setToolTip( "Systolic / diastolic / mean arterial blood pressure" );
    mUi.valuesList->setItem( VAL_ID_SBP_DBP_MAP, COLUMN_0, tableItemPtr );
    mTableItems.append( new QTableWidgetItem( ".." ) );
    mUi.valuesList->setItem( VAL_ID_SBP_DBP_MAP, COLUMN_1, mTableItems.at( VAL_ID_SBP_DBP_MAP ) );
    mUi.valuesList->setItem( VAL_ID_SBP_DBP_MAP, COLUMN_2, new QTableWidgetItem( "mmHg" ) );

    // VAL_ID_SVR
    tableItemPtr = new QTableWidgetItem( "SVR" );
    tableItemPtr->setToolTip( "Systemic vascular resistance" );
    mUi.valuesList->setItem( VAL_ID_SVR, COLUMN_0, tableItemPtr );
    mTableItems.append( new QTableWidgetItem( ".." ) );
    mUi.valuesList->setItem( VAL_ID_SVR, COLUMN_1, mTableItems.at( VAL_ID_SVR ) );
    mUi.valuesList->setItem( VAL_ID_SVR, COLUMN_2, new QTableWidgetItem( "mmHg/l/min (WU)" ) );

    // VAL_ID_CVP
    tableItemPtr = new QTableWidgetItem( "CVP" );
    tableItemPtr->setToolTip( "Central venous pressure" );
    mUi.valuesList->setItem( VAL_ID_CVP, COLUMN_0, tableItemPtr );
    mTableItems.append( new QTableWidgetItem( ".." ) );
    mUi.valuesList->setItem( VAL_ID_CVP, COLUMN_1, mTableItems.at( VAL_ID_CVP ) );
    mUi.valuesList->setItem( VAL_ID_CVP, COLUMN_2, new QTableWidgetItem( "mmHg" ) );

    // VAL_ID_RAP
    tableItemPtr = new QTableWidgetItem( "RAP" );
    tableItemPtr->setToolTip( "Right atrial pressure" );
    mUi.valuesList->setItem( VAL_ID_RAP, COLUMN_0, tableItemPtr );
    mTableItems.append( new QTableWidgetItem( ".." ) );
    mUi.valuesList->setItem( VAL_ID_RAP, COLUMN_1, mTableItems.at( VAL_ID_RAP ) );
    mUi.valuesList->setItem( VAL_ID_RAP, COLUMN_2, new QTableWidgetItem( "mmHg" ) );

    // VAL_ID_sPAP_dPAP_mPAP
    tableItemPtr = new QTableWidgetItem( "sPAP / dPAP / mPAP" );
    tableItemPtr->setToolTip( "Systolic / diastolic / mean pulmonary arterial pressure" );
    mUi.valuesList->setItem( VAL_ID_sPAP_dPAP_mPAP, COLUMN_0, tableItemPtr );
    mTableItems.append( new QTableWidgetItem( ".." ) );
    mUi.valuesList->setItem( VAL_ID_sPAP_dPAP_mPAP, COLUMN_1, mTableItems.at( VAL_ID_sPAP_dPAP_mPAP ) );
    mUi.valuesList->setItem( VAL_ID_sPAP_dPAP_mPAP, COLUMN_2, new QTableWidgetItem( "mmHg" ) );

    // VAL_ID_PVR
    tableItemPtr = new QTableWidgetItem( "PVR" );
    tableItemPtr->setToolTip( "Pulmonary vascular resistance" );
    mUi.valuesList->setItem( VAL_ID_PVR, COLUMN_0, tableItemPtr );
    mTableItems.append( new QTableWidgetItem( ".." ) );
    mUi.valuesList->setItem( VAL_ID_PVR, COLUMN_1, mTableItems.at( VAL_ID_PVR ) );
    mUi.valuesList->setItem( VAL_ID_PVR, COLUMN_2, new QTableWidgetItem( "mmHg/l/min (WU)" ) );

    // VAL_ID_PVP
    tableItemPtr = new QTableWidgetItem( "PVP" );
    tableItemPtr->setToolTip( "Pulmonary venous pressure" );
    mUi.valuesList->setItem( VAL_ID_PVP, COLUMN_0, tableItemPtr );
    mTableItems.append( new QTableWidgetItem( ".." ) );
    mUi.valuesList->setItem( VAL_ID_PVP, COLUMN_1, mTableItems.at( VAL_ID_PVP ) );
    mUi.valuesList->setItem( VAL_ID_PVP, COLUMN_2, new QTableWidgetItem( "mmHg" ) );

    // VAL_ID_Pulm_flow
//    tableItemPtr = new QTableWidgetItem( "Pulm flow" );
//    tableItemPtr->setToolTip( "Flow through pulmonary circulation" );
//    mUi.valuesList->setItem( VAL_ID_Pulm_flow, COLUMN_0, tableItemPtr );
//    mTableItems.append( new QTableWidgetItem( ".." ) );
//    mUi.valuesList->setItem( VAL_ID_Pulm_flow, COLUMN_1, mTableItems.at( VAL_ID_Pulm_flow ) );
//    mUi.valuesList->setItem( VAL_ID_Pulm_flow, COLUMN_2, new QTableWidgetItem( "l/min" ) );

    // VAL_ID_ASD_flow
    tableItemPtr = new QTableWidgetItem( "ASD flow" );
    tableItemPtr->setToolTip( "Mean blood flow (left to right) through atrial septal defect" );
    mUi.valuesList->setItem( VAL_ID_ASD_flow, COLUMN_0, tableItemPtr );
    mTableItems.append( new QTableWidgetItem( ".." ) );
    mUi.valuesList->setItem( VAL_ID_ASD_flow, COLUMN_1, mTableItems.at( VAL_ID_ASD_flow ) );
    mUi.valuesList->setItem( VAL_ID_ASD_flow, COLUMN_2, new QTableWidgetItem( "l/min" ) );

    // VAL_ID_VSD_flow
    tableItemPtr = new QTableWidgetItem( "VSD flow" );
    tableItemPtr->setToolTip( "Mean blood flow (left to right) through ventricular septal defect" );
    mUi.valuesList->setItem( VAL_ID_VSD_flow, COLUMN_0, tableItemPtr );
    mTableItems.append( new QTableWidgetItem( ".." ) );
    mUi.valuesList->setItem( VAL_ID_VSD_flow, COLUMN_1, mTableItems.at( VAL_ID_VSD_flow ) );
    mUi.valuesList->setItem( VAL_ID_VSD_flow, COLUMN_2, new QTableWidgetItem( "l/min" ) );

    // VAL_ID_PDA_flow
    tableItemPtr = new QTableWidgetItem( "PDA flow" );
    tableItemPtr->setToolTip( "Mean blood flow (left to right) through patent ductus arteriosus" );
    mUi.valuesList->setItem( VAL_ID_PDA_flow, COLUMN_0, tableItemPtr );
    mTableItems.append( new QTableWidgetItem( ".." ) );
    mUi.valuesList->setItem( VAL_ID_PDA_flow, COLUMN_1, mTableItems.at( VAL_ID_PDA_flow ) );
    mUi.valuesList->setItem( VAL_ID_PDA_flow, COLUMN_2, new QTableWidgetItem( "l/min" ) );
}


PValueView::~PValueView( void )
{
}


bool PValueView::readSettings( QSettings & settings )
{
    bool read = false;

    if ( settings.contains( CIRCADAPT_VALUEVIEW_PART ) )
    {
        QHeaderView * headerViewPtr = mUi.valuesList->horizontalHeader();

        if ( headerViewPtr )
        {
            read = headerViewPtr->restoreState( settings.value( CIRCADAPT_VALUEVIEW_PART, QByteArray() ).toByteArray() );
        }
    }

    return read;
}


bool PValueView::writeSettings( QSettings & settings )
{
    bool write = false;

    QHeaderView * headerViewPtr = mUi.valuesList->horizontalHeader();

    if ( headerViewPtr )
    {
        settings.setValue( CIRCADAPT_VALUEVIEW_PART, headerViewPtr->saveState() );
    }

    return write;
}


void PValueView::updateInfo()
{
    if ( this->isVisible() )
    {
        QTableWidgetItem            newItem;
        const UCircAdaptSignal &    aortaFlow( mData.heart().valveLArt().getFlow() );

        if ( (aortaFlow.getLastBeat() != NULL ) &&
             !aortaFlow.getLastBeat()->isEmpty() )
        {
            double          tmpValue;
            PStatistics     aortaPStat(  *mData.getAorta().getPressureAtEntrance().getLastBeat(), PStatistics::doMinMaxMean );
            //PStatistics     lvStat(     *mData.getHeart().getVentricles().getLeftVentricle().getPressure().getLastBeat(), PStatistics::doMax );
            PStatistics     cvStat(     *mData.getSystemicVein().getPressureAtEntrance().getLastBeat(), PStatistics::doMean );
            PStatistics     lapStat(    *mData.getHeart().getAtria().getLeftAtrium().getPressure().getLastBeat(), PStatistics::doMean );
            PStatistics     rapStat(    *mData.getHeart().getAtria().getRightAtrium().getPressure().getLastBeat(), PStatistics::doMean );

            PStatistics     venaCavaPStat(*mData.getSystemicVein().getPressureAtEntrance().getLastBeat(), PStatistics::doMean );

            PStatistics     PAPStat(    *mData.pulmonaryArtery().getPressureAtEntrance().getLastBeat(), PStatistics::doMinMaxMean );
            PStatistics     PVPStat(    *mData.getPulmonaryVeins().getPressureAtEntrance().getLastBeat(), PStatistics::doMean );

            const PVector * lastAortaFlow = aortaFlow.getLastBeat();
            //PStatistics     aortaFlowStat( *lastAortaFlow, PStatistics::doSum );
            double          toLperMin = 60.0 / ( ( (double)lastAortaFlow->N() * aortaFlow.getSampleTime() ) / 1000.0) ;

            PStatistics     pulmFlowStat( *mData.heart().valveRArt().getFlow().getLastBeat() , PStatistics::doSum );

            int rrpi = 0;
            int srpi = 0;

            for ( int i = 0; i < mData.getResistances().size(); i++ )
            {
                if ( mData.getResistances().at(i).getResistanceType() ==  SResistance::PULMONARY )
                {
                    rrpi= i;
                }
                if ( mData.getResistances().at(i).getResistanceType() ==  SResistance::SYSTEMIC)
                {
                    srpi= i;
                }
            }

            PStatistics srpFlowStat( *mData.getResistances()[srpi].getFlow().getLastBeat() , PStatistics::doSum );

            PStatistics aortaFlowStat( *mData.getHeart().getValveLArt().getFlow().getLastBeat(), PStatistics::doSum );

            PStatistics qsFlowStat( *mData.getHeart().getValveRVen().getFlow().getLastBeat(), PStatistics::doSum );

            PStatistics qpFlowStat( *mData.getHeart().getValveLVen().getFlow().getLastBeat(), PStatistics::doSum );

            PStatistics rrpFlowStat( *mData.getResistances()[rrpi].getFlow().getLastBeat() , PStatistics::doSum );

            PStatistics asdFlowStat( *mData.getHeart().getValveASD().getFlow().getLastBeat() , PStatistics::doSum );

            PStatistics vsdFlowStat( *mData.getHeart().getValveVSD().getFlow().getLastBeat() , PStatistics::doSum );

            PStatistics ductFlowStat( *mData.getHeart().getValveDUCT().getFlow().getLastBeat() , PStatistics::doSum );

            tmpValue = aortaFlowStat.getSum() * toLperMin;
            if ( mCo != tmpValue )
            {
                mCo = tmpValue;

                mTableItems.at( VAL_ID_CO )->setText( QString( "%1" ).arg( mCo, 5, 'f', 2 ) );
            }

            tmpValue = qsFlowStat.getSum() * toLperMin;
            if ( mQs != tmpValue )
            {
                mQs = tmpValue;

                mTableItems.at( VAL_ID_QS )->setText( QString( "%1" ).arg( mQs, 5, 'f', 2 ) );
            }

            tmpValue = qpFlowStat.getSum() * toLperMin;
            if ( mQp != tmpValue )
            {
                mQp = tmpValue;

                mTableItems.at( VAL_ID_QP )->setText( QString( "%1" ).arg( mQp, 5, 'f', 2 ) );
            }

            tmpValue = mData.toPressure( lapStat.getMean() );
            if ( mLAP != tmpValue )
            {
                mLAP = tmpValue;

                mTableItems.at( VAL_ID_LAP )->setText( QString( "%1" ).arg( mLAP, 5, 'f', 2 ) );
            }

            tmpValue = mData.toPressure( aortaPStat.getMean() );
            if ( mMAP != tmpValue )
            {
                mMAP = tmpValue;

                mTableItems.at( VAL_ID_SBP_DBP_MAP )->setText( QString( "%1 / %2 / %3" ).arg( mData.toPressure( aortaPStat.getMax() ), 5, 'f', 2 )
                                                                                        .arg( mData.toPressure( aortaPStat.getMin() ), 5, 'f', 2 )
                                                                                        .arg( mMAP, 5, 'f', 2 ) );
            }

            tmpValue = (mData.toPressure( aortaPStat.getMean() ) - mData.toPressure( venaCavaPStat.getMean() ) ) /  (srpFlowStat.getSum() * toLperMin);
            if ( mSVR != tmpValue )
            {
                mSVR = tmpValue;

                mTableItems.at( VAL_ID_SVR )->setText( QString( "%1" ).arg( mSVR, 5, 'f', 2 ) );
            }


            tmpValue = mData.toPressure( cvStat.getMean() );
            if ( mCVP != tmpValue )
            {
                mCVP = tmpValue;

                mTableItems.at( VAL_ID_CVP )->setText( QString( "%1" ).arg( mCVP, 5, 'f', 2 ) );
            }


            tmpValue = mData.toPressure( rapStat.getMean() );
            if ( mRAP != tmpValue )
            {
                mRAP = tmpValue;

                mTableItems.at( VAL_ID_RAP )->setText( QString( "%1" ).arg( mRAP, 5, 'f', 2 ) );
            }

            tmpValue = mData.toPressure( PAPStat.getMean() );
            if ( mPAP != tmpValue )
            {
                mPAP = tmpValue;

                mTableItems.at( VAL_ID_sPAP_dPAP_mPAP )->setText( QString( "%1 / %2 / %3" ).arg( mData.toPressure( PAPStat.getMax() ), 5, 'f', 2 )
                                                                                           .arg( mData.toPressure( PAPStat.getMin() ), 5, 'f', 2 )
                                                                                           .arg( mPAP, 5, 'f', 2 ) );
            }

            tmpValue = (mData.toPressure( PAPStat.getMean() ) - mData.toPressure( PVPStat.getMean() ) ) /  (rrpFlowStat.getSum() * toLperMin);
            if ( mPVR != tmpValue )
            {
                mPVR = tmpValue;

                mTableItems.at( VAL_ID_PVR )->setText( QString( "%1" ).arg( mPVR, 5, 'f', 2 ) );
            }


            tmpValue = mData.toPressure( PVPStat.getMean() );
            if ( mPVP != tmpValue )
            {
                mPVP = tmpValue;

                mTableItems.at( VAL_ID_PVP )->setText( QString( "%1" ).arg( mPVP, 5, 'f', 2 ) );
            }


//            tmpValue = pulmFlowStat.getSum() * toLperMin;
//            if ( mPulmFlow != tmpValue )
//            {
//                mPulmFlow = tmpValue;

//                mTableItems.at( VAL_ID_Pulm_flow )->setText( QString( "%1" ).arg( mPulmFlow, 5, 'f', 2 ) );
//            }


            tmpValue = qRound(100.0 * asdFlowStat.getSum() * toLperMin) / 100.0;
            if ( mASDFlow != tmpValue )
            {
                mASDFlow = tmpValue;
                mTableItems.at( VAL_ID_ASD_flow )->setText( QString( "%1" ).arg( mASDFlow, 5, 'f', 2 ) );
            }

            tmpValue = qRound(100.0 * vsdFlowStat.getSum() * toLperMin) / 100.0;
            if ( mVSDFlow != tmpValue )
            {
                mVSDFlow = tmpValue;

                mTableItems.at( VAL_ID_VSD_flow )->setText( QString( "%1" ).arg( mVSDFlow, 5, 'f', 2 ) );
            }

            tmpValue = qRound(100.0 * ductFlowStat.getSum() * toLperMin) / 100.0;
            if ( mDuctFlow != tmpValue )
            {
                mDuctFlow = tmpValue;

                mTableItems.at( VAL_ID_PDA_flow )->setText( QString( "%1" ).arg( mDuctFlow, 5, 'f', 2 ) );
            }
        }
    }
}


void PValueView::copyToClipBoard(void)
{
    QPixmap         pixmap( width(), height() );
    QClipboard *    cb = QApplication::clipboard();

    cb->setPixmap( pixmap.grabWidget( this ), QClipboard::Clipboard );
}
