/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: pvalueview.h 935 2013-11-28 22:25:23Z peter $
 *
 */

#ifndef __PVALUEVIEW_H
#define __PVALUEVIEW_H

#include <QWidget>
#include <QLabel>
#include <QSettings>


#include "ucirculation.h"
#include "ui_PValueView.h"

namespace Peacs
{
    namespace CircAdapt
    {

        class PValueView : public QWidget
        {
            Q_OBJECT

        public:

            explicit PValueView( UCirculation &data ,QWidget * parent = 0 );

            ~PValueView( void );


            /** @brief Reads the settings of the view items
              *
              * @param settings The object to retrieve the settings from
              * @return true if the settings were retrieved
              */
            bool readSettings( QSettings & settings );


            /** @brief Writes the settings of the view items to the settings object
              *
              * @param settings The object to write the settings to
              * @return true if the settings were written
              */
            bool writeSettings( QSettings & settings );


            /** @brief
              */
            void copyToClipBoard(void);


        public slots:

            /** @brief
              */
            void updateInfo( );


        private:

            /** @brief
              */
            UCirculation & mData;


            /** @brief
              */
            Ui_PValueView mUi;


            /** @brief
              */
            double  mCo;
            double  mQs;
            double  mQp;
            double  mPVP;
            double  mMAP;
            double  mCVP;
            double  mLAP;   // Left atrial Pressure
            double  mRAP;   // right atrial Pressure
            double  mPVR;   // systemic  Pressure
            double  mSVR;   // pulmonary  Pressure
            double  mPAP;   // pulmonary arterial Pressure

//            double mPulmFlow;
            double mASDFlow;
            double mVSDFlow;
            double mDuctFlow;


            /** @brief
              */
            QVector<QTableWidgetItem*> mTableItems;
        };
    }
}
#endif // __PVALUEVIEW_H
