/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: tatria.cpp 776 2013-07-11 20:33:36Z eelco $
 *
 */

#include <QDebug>

#include "defs.h"
#include "tatria.h"
#include "atria.h"
#include "uatria.h"

using namespace Peacs::CircAdapt;
TAtria::TAtria() :
    TCircElement( ),
    mLeftA(  ),
    mRigtA( )
{

}
TAtria::TAtria(const TAtria & atria):
    TCircElement( atria ),
    mLeftA( atria.mLeftA ),
    mRigtA( atria.mRigtA )
{

}

TAtria::TAtria(const SAtria & atria) :
    TCircElement( atria.getName() ),
    mLeftA( atria.getLeftAtrium() ),
    mRigtA( atria.getRightAtrium() )
{
}

const TAtria & TAtria::operator =(const TAtria &atria)
{
    if ( this != &atria)
    {
        TCircElement::operator =(atria);
        mLeftA      = atria.mLeftA;
        mRigtA      = atria.mRigtA;
    }
    return *this;
}

void TAtria::copyUserSpecifications(const UAtria &atria)
{

    mLeftA.copyUserSpecifications( atria.getLeftAtrium() );
    mRigtA.copyUserSpecifications( atria.getRightAtrium() );
}


void TAtria::copySpecifications(const SAtria &atria)
{
    setName(atria.getName());
    mLeftA.copySpecifications( atria.getLeftAtrium() );
    mRigtA.copySpecifications( atria.getRightAtrium() );
}

void TAtria::resetData( int size )
{
    mLeftA.resetData(size);
    mRigtA.resetData(size);
}


void TAtria::addSample( const SAtria & sim, int pos)
{
    mLeftA.addSample(sim.getLeftAtrium(),  pos);
    mRigtA.addSample(sim.getRightAtrium(), pos);
}




