/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: tatria.h 776 2013-07-11 20:33:36Z eelco $
 *
 */

#ifndef _TATRIA_H_
#define _TATRIA_H_

#include "defs.h"
#include "tchamber.h"
#include "tcircelement.h"


using namespace Peacs::Support;
namespace Peacs
{

    namespace CircAdapt
    {

        class SAtria;
        class UAtria;
        /**
        Class for muscular chamber. Inherits from MCompliance.
        */
        class TAtria: public TCircElement
        {

        public:
            TAtria();

            TAtria(const TAtria & atria);

            TAtria(const SAtria & atria);

            virtual ~TAtria() {;}


            virtual const TAtria& operator = (const TAtria &atria);

            /** @brief copy the sepcification parameters from the user class to the transfer class
              */
            void copyUserSpecifications(const UAtria &atria);

            /** @brief copy the sepcification parameters from the simulation class to the transfer class
              * no specification parameters in this class
              */
            void copySpecifications(const SAtria &);

            /** @brief At the start of a simulation run the state variables are resized
              * @param size the number of samples in this heart beat
              */
            void resetData( int size );

            /** @brief add a sample to the state variables
              * @param sim the simulate class
              * @param pos the sample number in the state variables
              */
            void addSample( const SAtria & sim, int pos);

            void init();

            inline const TChamber & getRightAtrium() const { return mRigtA; }

            inline TChamber & rightAtrium() { return mRigtA; }

            inline const TChamber & getLeftAtrium() const { return mLeftA; }

            inline TChamber & leftAtrium() { return mLeftA; }


        private:
            TChamber mLeftA;

            TChamber mRigtA;
        };
    }
}

// This makes it possible for TTubeState values to be stored in QVariant
// objects and retrieved later
Q_DECLARE_METATYPE( Peacs::CircAdapt::TAtria )


#endif
