/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: tcircelement.h 776 2013-07-11 20:33:36Z eelco $
 *
 */
#ifndef __TCIRCELEMENT_H_
#define __TCIRCELEMENT_H_

#include <QString>
#include <QMetaType>
#include <QObject>

namespace Peacs
{
    namespace CircAdapt
    {
        /**
        Abstract base class for circulation elements.
        */
        class TCircElement : public QObject
        {
            Q_OBJECT
        public:
            /* @brief Default Constructor */
            TCircElement();

            /* @brief Constructor */
            TCircElement (const QString &name);

            TCircElement (const TCircElement &element);

            virtual ~TCircElement() { ; }

            /** @returns the name of the CircAdapt element */
            inline const QString & getName() const { return mElementName; }

            /** @returns the name of the CircAdapt element */
            inline void setName(const QString & name) { mElementName = name; }

            virtual bool operator == (const TCircElement &element) { return mElementName == element.mElementName; }

            virtual bool operator != (const TCircElement &element) { return mElementName != element.mElementName; }

            virtual const TCircElement & operator = (const TCircElement &element);


        private:
            // Element name
            QString mElementName;

        };
    }
}
// This makes it possible for TTubeState values to be stored in QVariant
// objects and retrieved later
Q_DECLARE_METATYPE( Peacs::CircAdapt::TCircElement )

#endif
