/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: tcompliance.cpp 776 2013-07-11 20:33:36Z eelco $
 *
 */

#include <QString>
#include "compliance.h"
#include "tcompliance.h"
#include "ucompliance.h"



using namespace Peacs::CircAdapt;
// Constructor
TCompliance::TCompliance():
    TCircElement(),
    mVolume(),
    mVDot(),
    mA(),
    mP(),
    mZ(),
    mPin(),
    mQRemod(),
    mInitialVolume(-1.0)
{

}

TCompliance::TCompliance(const TCompliance &comp):
    TCircElement(comp.getName()),
    mVolume(comp.mVolume),
    mVDot(comp.mVDot),
    mA(comp.mA),
    mP(comp.mP),
    mZ(comp.mZ),
    mPin(comp.mPin),
    mQRemod(comp.mQRemod),
    mInitialVolume(comp.getInitialVolume())
{

}

TCompliance::TCompliance(const SCompliance &comp):
    TCircElement(comp.getName()),
    mVolume(),
    mVDot(),
    mA(),
    mP(),
    mZ(),
    mPin(),
    mQRemod(),
    mInitialVolume(-1)
{

}


const TCompliance& TCompliance::operator = (const TCompliance &comp)
{
    if ( this != &comp )
    {
        TCircElement::operator =(comp);

        mVolume         = comp.mVolume;
        mVDot           = comp.mVDot;
        mA              = comp.mA;
        mP              = comp.mP;
        mZ              = comp.mZ;
        mPin            = comp.mPin;
        mQRemod         = comp.mQRemod;
        mInitialVolume  = comp.mInitialVolume;
    }

    return *this;
}


void TCompliance::copyUserSpecifications(const UCompliance &comp)
{
//    mInitialVolume = comp.getInitialVolume();
}


void TCompliance::copySpecifications(const SCompliance &comp)
{
    setName(comp.getName());
    mInitialVolume = comp.getVolume();   // Needed at initialization time to fill UCompliance::mRefVolume
}


void TCompliance::resetData( int size )
{
    mVolume.resize(size);  // Volume of compliance

    mVDot.resize(size);    // d/dt Volume

    mA.resize(size);       // Cross-section of compliance

    mP.resize(size);       // Pressure of compliance

    mZ.resize(size);       // Impedance

    mPin.resize(size);     // Pressure at entrance

    mQRemod.resize(size);  // Through flow used for diameter adaptation

}



void TCompliance::addSample( const SCompliance & comp, int pos)
{
    mVolume[pos]    = comp.getVolume();

    mVDot[pos]      = comp.getVolumeDot();

    mA[pos]         = comp.getArea();

    mP[pos]         = comp.getPressure();

    mZ[pos]         = comp.getImpedance();

    mPin[pos]       = comp.getPressureAtEntrance();

    mQRemod[pos]    = comp.getQRemod();
}



