/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: tflowduct.h 776 2013-07-11 20:33:36Z eelco $
 *
 */

#ifndef __TFLOWDUCT_H_
#define __TFLOWDUCT_H_


#include "tcircelement.h"
#include "pvector.h"

namespace Peacs
{
    using namespace Support;

    namespace CircAdapt
    {
    class SFlowDuct;
    class UFlowDuct;
        /**
        Class for flow channel between compliances. Inherits from CircElement.
        */

        class TFlowDuct: public TCircElement
        {
        public:

            TFlowDuct ();

            TFlowDuct (const SFlowDuct & flowduct);

            /// Destructor
            virtual ~TFlowDuct() {  }

            const TFlowDuct & operator = (const TFlowDuct &flowduct);

            /** @brief copy the sepcification parameters from the user class to the transfer class
              */
            void copyUserSpecifications(const UFlowDuct &) {;}


            /** @brief copy the sepcification parameters from the simulation class to the transfer class
              * no specification parameters in this class
              */
            void copySpecifications(const SFlowDuct &flowduct);

            /** @brief At the start of a simulation run the state variables are resized
              * @param size the number of samples in this heart beat
              */
            virtual void resetData( int size );

            /** @brief add a sample to the state variables
              * @param sim the simulate class
              * @param pos the sample number in the state variables
              */
            virtual void addSample( const SFlowDuct & sim, int pos);


            /// Method to get flow
            inline const PVector & getFlow() const { return mQ; }

        protected:
            // Simulation state variables
            Peacs::Support::PVector mQ;          // Flow
        };
    }
}
// This makes it possible for TTubeState values to be stored in QVariant
// objects and retrieved later
Q_DECLARE_METATYPE( Peacs::CircAdapt::TFlowDuct )


#endif
