/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: theart.cpp 776 2013-07-11 20:33:36Z eelco $
 *
 */

#include <QDebug>

#include "defs.h"
#include "theart.h"
#include "heart.h"
#include "uheart.h"



using namespace  Peacs::CircAdapt;

// Constructor
THeart::THeart():
    TCircElement(),
    mAtria(),
    mVentricle(),
    mPericardialSack(),
    mValveLVen( ),
    mValveLAv(  ),
    mValveLArt( ),
    mValveRVen( ),
    mValveRAv(  ),
    mValveRArt( ),
    mValveASD(  ),
    mValveVSD(  ),
    mValveDUCT( )
{

}

THeart::THeart(const SHeart & heart) :
    TCircElement(       heart.getName()),
    mAtria(             heart.getAtria()),
    mVentricle(         heart.getVentricles()),
    mPericardialSack(   heart.getSack()),
    mValveLVen(         heart.getValveLVen()),            // Inlet left atrium
    mValveLAv(          heart.getValveLAv()),             // Mitral valve
    mValveLArt(         heart.getValveLArt()),            // Aortic valve
    mValveRVen(         heart.getValveRVen()),            // Inlet right atrium
    mValveRAv(          heart.getValveRAv()),             // Tricuspid valve
    mValveRArt(         heart.getValveRArt()),            // Pulmonary valve
    mValveASD(          heart.getValveASD()),            // Valve for ASD
    mValveVSD(          heart.getValveVSD()),           // Valve for VSD
    mValveDUCT(         heart.getValveDUCT())            // Valve for DUCT))
{

}


const THeart & THeart::operator =(const THeart &heart)
{
    if ( this != &heart)
    {
        TCircElement::operator =(heart);
        mAtria     = heart.mAtria;
        mVentricle = heart.mVentricle;
        mPericardialSack = heart.mPericardialSack;
        mValveLVen = heart.mValveLVen;            // Inlet left atrium
        mValveLAv  = heart.mValveLAv;             // Mitral valve
        mValveLArt = heart.mValveLArt;            // Aortic valve
        mValveRVen = heart.mValveRVen;            // Inlet right atrium
        mValveRAv  = heart.mValveRAv;             // Tricuspid valve
        mValveRArt = heart.mValveRArt;            // Pulmonary valve
        mValveASD  = heart.mValveASD;            // Valve for ASD
        mValveVSD  = heart.mValveVSD;           // Valve for VSD
        mValveDUCT = heart.mValveDUCT;            // Valve for DUCT))
    }
    return *this;
}

void THeart::copyUserSpecifications(const UHeart &heart)
{
    mAtria.copyUserSpecifications(             heart.getAtria() );
    mVentricle.copyUserSpecifications(         heart.getVentricles() );
    mPericardialSack.copyUserSpecifications(   heart.getSack() );
    mValveLVen.copyUserSpecifications(         heart.getValveLVen() );
    mValveLAv.copyUserSpecifications(          heart.getValveLAv() );
    mValveLArt.copyUserSpecifications(         heart.getValveLArt() );
    mValveRVen.copyUserSpecifications(         heart.getValveRVen() );
    mValveRAv.copyUserSpecifications(          heart.getValveRAv() );
    mValveRArt.copyUserSpecifications(         heart.getValveRArt() );
    mValveASD.copyUserSpecifications(          heart.getValveASD() );
    mValveVSD.copyUserSpecifications(          heart.getValveVSD() );
    mValveDUCT.copyUserSpecifications(         heart.getValveDUCT() );
}


void THeart::copySpecifications(const SHeart &heart)
{
    setName( heart.getName() );
    mAtria.copySpecifications(             heart.getAtria() );
    mVentricle.copySpecifications(         heart.getVentricles() );
    mPericardialSack.copySpecifications(   heart.getSack() );
    mValveLVen.copySpecifications(         heart.getValveLVen() );
    mValveLAv.copySpecifications(          heart.getValveLAv() );
    mValveLArt.copySpecifications(         heart.getValveLArt() );
    mValveRVen.copySpecifications(         heart.getValveRVen() );
    mValveRAv.copySpecifications(          heart.getValveRAv() );
    mValveRArt.copySpecifications(         heart.getValveRArt() );
    mValveASD.copySpecifications(          heart.getValveASD() );
    mValveVSD.copySpecifications(          heart.getValveVSD() );
    mValveDUCT.copySpecifications(         heart.getValveDUCT() );
}

void THeart::resetData( int size )
{
    mAtria.resetData(size);
    mVentricle.resetData(size);
    mPericardialSack.resetData(size);
    mValveLVen.resetData(size);
    mValveLAv.resetData(size);
    mValveLArt.resetData(size);
    mValveRVen.resetData(size);
    mValveRAv.resetData(size);
    mValveRArt.resetData(size);
    mValveASD.resetData(size);
    mValveVSD.resetData(size);
    mValveDUCT.resetData(size);

}

void THeart::addSample( const SHeart & heart, int pos)
{
    mAtria.addSample(             heart.getAtria(),     pos );
    mVentricle.addSample(         heart.getVentricles(),pos );
    mPericardialSack.addSample(   heart.getSack(),      pos );
    mValveLVen.addSample(         heart.getValveLVen(), pos );
    mValveLAv.addSample(          heart.getValveLAv(),  pos );
    mValveLArt.addSample(         heart.getValveLArt(), pos );
    mValveRVen.addSample(         heart.getValveRVen(), pos );
    mValveRAv.addSample(          heart.getValveRAv(),  pos );
    mValveRArt.addSample(         heart.getValveRArt(), pos );
    mValveASD.addSample(          heart.getValveASD(),  pos );
    mValveVSD.addSample(          heart.getValveVSD(),  pos );
    mValveDUCT.addSample(         heart.getValveDUCT(), pos );
}

