/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: theart.h 776 2013-07-11 20:33:36Z eelco $
 *
 */

#ifndef __THEART_H_
#define __THEART_H_

#include "defs.h"
#include "tatria.h"
#include "tventricle.h"
#include "tvalve.h"

#include "tsack.h"
#include "tcircelement.h"

using namespace Peacs::Support;

namespace Peacs
{

    namespace CircAdapt
    {
        /**


        */
        class SHeart;
        class UHeart;

        class THeart: public TCircElement
        {
            Q_OBJECT


        public:
            /** @brief Default Constructor */
            THeart();

            THeart(const SHeart & heart);

            virtual ~THeart() {;}


            virtual const THeart& operator = (const THeart &heart);


            /** @brief copy the sepcification parameters from the user class to the transfer class
              */
            void copyUserSpecifications(const UHeart &heart);

            /** @brief copy the sepcification parameters from the simulation class to the transfer class
              * no specification parameters in this class
              */
            void copySpecifications(const SHeart &);

            /** @brief At the start of a simulation run the state variables are resized
              * @param size the number of samples in this heart beat
              */
            void resetData( int size );

            /** @brief add a sample to the state variables
              * @param sim the simulate class
              * @param pos the sample number in the state variables
              */
            void addSample( const SHeart & sim, int pos);


            inline const TAtria & getAtria() const { return mAtria; }

            inline TAtria & atria() { return mAtria; }

            inline const TVentricle & getVentricles() const { return mVentricle; }

            inline TVentricle & ventricles() { return mVentricle; }

            inline const TSack &getSack() const { return mPericardialSack; }

            inline const TValve &getValveLVen() const { return mValveLVen; }           // Inlet left atrium

            inline const TValve &getValveLAv()  const { return mValveLAv;   }        // Mitral valve

            inline const TValve &getValveLArt() const { return mValveLArt;  }          // Aortic valve

            inline const TValve &getValveRVen() const { return mValveRVen; }            // Inlet right atrium

            inline const TValve &getValveRAv()  const { return mValveRAv; }            // Tricuspid valve

            inline const TValve &getValveRArt() const { return mValveRArt; }           // Pulmonary valve

            inline const TValve &getValveASD()  const { return mValveASD;  }           // Valve for ASD

            inline const TValve &getValveVSD()  const { return mValveVSD;  }           // Valve for VSD

            inline const TValve &getValveDUCT() const { return mValveDUCT; }            // Valve for DUCT


        private:

            TAtria       mAtria;

            TVentricle   mVentricle;

            TSack        mPericardialSack;

            TValve       mValveLVen;            // Inlet left atrium
            TValve       mValveLAv;             // Mitral valve
            TValve       mValveLArt;            // Aortic valve
            TValve       mValveRVen;            // Inlet right atrium
            TValve       mValveRAv;             // Tricuspid valve
            TValve       mValveRArt;            // Pulmonary valve
            TValve       mValveASD;             // Valve for ASD
            TValve       mValveVSD;             // Valve for VSD
            TValve       mValveDUCT;            // Valve for DUCT

        };
    }
}

// This makes it possible for THeartState values to be stored in QVariant
// objects and retrieved later
Q_DECLARE_METATYPE( Peacs::CircAdapt::THeart )

#endif
