/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: tpatch.cpp 776 2013-07-11 20:33:36Z eelco $
 *
 */

#include <QDebug>

#include "patch.h"
#include "upatch.h"
#include "tpatch.h"
#include "tchamber.h"




using namespace  Peacs::CircAdapt;
// Constructor
TPatch::TPatch():
    TCircElement(),
    mAmRef(0.0),
    mVWall(0.0),
    mPatchAreaMid( ),
    mCm( ),
    mT( ),
    mPTrans( ),
    mDTDAm( ),
    mEf( ),
    mSarcomere()
{
}
TPatch::TPatch(const TPatch &patch):
    TCircElement( patch ),
    mAmRef( patch.mAmRef),
    mVWall( patch.mVWall),
    mPatchAreaMid(patch.mPatchAreaMid      ),
    mCm(patch.mCm                ),
    mT( patch.mT                ),
    mPTrans(  patch.mPTrans          ),
    mDTDAm(   patch.mDTDAm          ),
    mEf( patch.mEf               ),
    mSarcomere( patch.getSarcomere() )
{

}

TPatch::TPatch(const SPatch &patch) :
    TCircElement( patch.getName() ),
    mAmRef( patch.getRefArea()),
    mVWall( patch.getWallVolume()),
    mPatchAreaMid(      ),
    mCm(                ),
    mT(                 ),
    mPTrans(            ),
    mDTDAm(             ),
    mEf(                ),
    mSarcomere( patch.getSarcomere() )
{
}


const TPatch& TPatch::operator = (const TPatch &patch)
{
    if ( this != &patch)
    {
        TCircElement::operator = (patch);

        mAmRef          = patch.mAmRef;
        mVWall          = patch.mVWall;
        mPatchAreaMid   = patch.mPatchAreaMid;
        mCm             = patch.mCm;
        mT              = patch.mT;
        mPTrans         = patch.mPTrans;
        mDTDAm          = patch.mDTDAm;
        mEf             = patch.mEf;
        mSarcomere      = patch.mSarcomere;
    }
    return *this;
}


void TPatch::copyUserSpecifications(const UPatch &patch)
{
    mAmRef = patch.getRefArea();
    mVWall = patch.getWallVolume();
    mSarcomere.copyUserSpecifications( patch.getSarcomere() );
}

void TPatch::copySpecifications(const SPatch &patch)
{
    setName( patch.getName() );
    mAmRef = patch.getRefArea();
    mVWall = patch.getWallVolume();
    mSarcomere.copySpecifications( patch.getSarcomere());
}



void TPatch::resetData( int size )
{
    mPatchAreaMid.resize(size);
    mCm.resize(size);
    mT.resize(size);
    mPTrans.resize(size);
    mDTDAm.resize(size);
    mEf.resize(size);
    mSarcomere.resetData( size );

}

void TPatch::addSample( const SPatch & sim, int pos)
{
    mPatchAreaMid[pos]  = sim.getArea();
    mCm[pos]            = sim.getCurvature();
    mT[pos]             = sim.getWallTension();
    mPTrans[pos]        = sim.getTransmuralPressure();
    mDTDAm[pos]         = sim.getWallStiffness();
    mEf[pos]            = sim.getFiberStrain();
    mSarcomere.addSample( sim.getSarcomere() ,pos);
}

