/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: tresistance.cpp 776 2013-07-11 20:33:36Z eelco $
 *
 */

#include "resistance.h"
#include "uresistance.h"
#include "tresistance.h"



using namespace  Peacs::CircAdapt;
// Constructor

TResistance::TResistance():
    TFlowDuct(),
    mR(0.0),
    mResistanceType( SResistance::UNDEFINED_RESISTANCE )
{

}

TResistance::TResistance(const TResistance &res):
    TFlowDuct(res),
    mR(res.mR),
    mResistanceType( res.getResistanceType() )
{

}


TResistance::TResistance (const SResistance &res):
    TFlowDuct( res ),
    mR( res.getResistance() ),
    mResistanceType( res.getResistanceType() )
{

}



const TResistance& TResistance::operator = (const TResistance &res)
{
    if ( this != &res )
    {
        TFlowDuct::operator =(res);
        mR = res.mR;
        mResistanceType = res.getResistanceType();
    }
    return *this;
}

void TResistance::copyUserSpecifications(const UResistance &res)
{
    TFlowDuct::copyUserSpecifications(res);
    mR = res.getResistance();
}



void TResistance::copySpecifications(const SResistance &res)
{
    TFlowDuct::copySpecifications(res);
    setName( res.getName() );
    mR = res.getResistance();
    mResistanceType = res.getResistanceType();
}


void TResistance::resetData( int size )
{
    TFlowDuct::resetData(size);
}
