/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: tresistance.h 776 2013-07-11 20:33:36Z eelco $
 *
 */

#ifndef __TRESISTANCE_H_
#define __TRESISTANCE_H_


#include "resistance.h"
#include "tflowduct.h"
#include "pvector.h"

namespace Peacs
{

    namespace CircAdapt
    {
    class SResistance;
    class UResistance;

        /**
        Class for microcirculation. Inherits from FlowDuct.
        */
        class TResistance: public TFlowDuct {
            Q_OBJECT


        public:
            TResistance ();

            TResistance (const TResistance &res);

            TResistance (const SResistance &res);

            /// Destructor
            virtual ~TResistance() { ;}

            const TResistance& operator = (const TResistance &comp);

            /** @brief copy the sepcification parameters from the user class to the transfer class
              */
            void copyUserSpecifications(const UResistance &res);

            /** @brief copy the sepcification parameters from the simulation class to the transfer class
              * no specification parameters in this class
              */
            void copySpecifications(const SResistance &);

            /** @brief At the start of a simulation run the state variables are resized
              * @param size the number of samples in this heart beat
              */
            void resetData( int size );

            inline double getResistance() const { return mR; }

            inline void setResistance(double res) { mR = res; }

            SResistance::resistance_type getResistanceType() const { return mResistanceType; }

        private:
            // Specification parameters
            double mR; // Peripheral resistance

            SResistance::resistance_type mResistanceType;


       };
    }
}
// This makes it possible for TResistanceState values to be stored in QVariant
// objects and retrieved later
Q_DECLARE_METATYPE( Peacs::CircAdapt::TResistance )

#endif
