/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: tsack.cpp 776 2013-07-11 20:33:36Z eelco $
 *
 */

#include "sack.h"
#include "tsack.h"
#include "usack.h"

using namespace Peacs::CircAdapt;

TSack::TSack():
    TCompliance(),
    mVrefSack(0.0),
    mPSack(0.0),
    mVSack(0.0)
{
}


// Constructor
TSack::TSack(const SSack &sack) :
    TCompliance(sack),
    mVrefSack( sack.getRefVolume()),
    mPSack( 0.0 ),
    mVSack( 0.0 )
{

}

const TSack& TSack::operator =(const TSack & sack)
{
    if ( this  != &sack )
    {
        TCompliance::operator =(sack);
        mVrefSack           = sack.mVrefSack;
        mPSack              = sack.mPSack;
        mVSack              = sack.mVSack;
    }
    return *this;
}

void TSack::copyUserSpecifications(const USack &sack)
{
    TCompliance::copyUserSpecifications(sack);

    mVrefSack = sack.getRefVolume();
}


void TSack::copySpecifications(const SSack &sack)
{
    TCompliance::copySpecifications(sack);

    mVrefSack = sack.getRefVolume();
}

void TSack::resetData( int size )
{
    TCompliance::resetData(size);
    mPSack.resize(size);
    mVSack.resize(size);
}

void TSack::addSample( const SSack & sim, int pos)
{
    TCompliance::addSample(sim,pos);

    mPSack[pos] = sim.getPSack();
    mVSack[pos] = sim.getVSack();
}
