/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: tsack.h 776 2013-07-11 20:33:36Z eelco $
 *
 */

#ifndef __TSACK_H_
#define __TSACK_H_

#include "tcompliance.h"

namespace Peacs
{

    namespace CircAdapt
    {

        class SSack;
        class USack;
        /**
        Class for pericardial sack. Inherits from Compliance.
        */
        class TSack: public TCompliance {
            Q_OBJECT


        public:
            TSack ( );

            TSack (const SSack & sack);

            /// Destructor
            ~TSack() { }

            virtual const TSack& operator = (const TSack &sack);

            /** @return the reference volume of the sack
              */
            inline double getRefVolume() const { return mVrefSack; }

            /** @@brief set the reference volume of the sack
              * @param vol the new volume
              */
            inline void setRefVolume(double vol) { mVrefSack = vol; }


            inline const PVector &getPSack() const { return mPSack; }

            inline const PVector &getVSack() const { return mVSack; }


            /** @brief copy the sepcification parameters from the user class to the transfer class
              */
            void copyUserSpecifications(const USack &sack);

            /** @brief copy the sepcification parameters from the simulation class to the transfer class
              * no specification parameters in this class
              */
            void copySpecifications(const SSack &);

            /** @brief At the start of a simulation run the state variables are resized
              * @param size the number of samples in this heart beat
              */
            void resetData( int size );

            /** @brief add a sample to the state variables
              * @param sim the simulate class
              * @param pos the sample number in the state variables
              */
            void addSample( const SSack & sim, int pos);


        private:
            // Specification parameters
            double mVrefSack;     // Reference volume of sack

            // Simulation state variables
            PVector mPSack;        // Pressure of sack

            PVector mVSack;        // Volume of sack

        };
    }
}
// This makes it possible for TTubeState values to be stored in QVariant
// objects and retrieved later
Q_DECLARE_METATYPE( Peacs::CircAdapt::TSack )



#endif
