/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: tsarcomere.cpp 784 2013-07-16 17:35:19Z peter $
 *
 */

#include <QDebug>

#include "tsarcomere.h"
#include "sarcomere.h"
#include "usarcomere.h"


using namespace Peacs::Support;
using namespace  Peacs::CircAdapt;

// Constructor
TSarcomere::TSarcomere():
    TCircElement(),
    mActivationDelay(0.0),
    mLsRef(0.0),
    mLs0Pas(0.0),
    mdLsPas(0.0),
    mSfPas(0.0),
    mLsi0Act(0.0),
    mLenSeriesElement(0.0),
    mSfAct(0.0),
    mvMax(0.0),
    mTimeAct(0.0),
    mTR(0.0),
    mTD(0.0),
    mCRest(0.0),
    mAdapt_LsBe(0.0),
    mAdapt_LsEe(0.0),
    mAdapt_SfPasMax(0.0),
    mFiberStrain(0.0),
    mC(0.0),
    mCDot(0.0),
    mLsi(0.0),
    mLsiDot(0.0),
    mLs(0.0),
    mSfPasT(0.0),
    mSf(0.0),
    mDSfDEf(0.0)
{
}

TSarcomere::TSarcomere (const SSarcomere &sarco):
    TCircElement(sarco.getName()),
    mActivationDelay(sarco.getActivationdelay()),
    mLsRef(sarco.getLsRef()),
    mLs0Pas(sarco.getLs0Pas()),
    mdLsPas(sarco.getdLsPas()),
    mSfPas(sarco.getSfPas()),
    mLsi0Act(sarco.getLsi0Act()),
    mLenSeriesElement(sarco.getLenSeriesElement()),
    mSfAct(sarco.getSfAct()),
    mvMax(sarco.getvMax()),
    mTimeAct(sarco.getTimeAct()),
    mTR(sarco.getTR()),
    mTD(sarco.getTD()),
    mCRest(sarco.getCRest()),
    mAdapt_LsBe(0.0),
    mAdapt_LsEe(0.0),
    mAdapt_SfPasMax(0.0),
    mFiberStrain(sarco.getFiberStrain()),
    mC(0.0),
    mCDot(0.0),
    mLsi(0.0),
    mLsiDot(0.0),
    mLs(0.0),
    mSfPasT(0.0),
    mSf                 ( sarco.getFiberStress() ),
    mDSfDEf             ( sarco.getFiberStiffness() )
{

}

const TSarcomere& TSarcomere::operator = (const TSarcomere &sarco)
{
    if ( this != &sarco)
    {
        TCircElement::operator =(sarco);

        mActivationDelay    = sarco.mActivationDelay;
        mLsRef              = sarco.mLsRef;
        mLs0Pas             = sarco.mLs0Pas;
        mdLsPas             = sarco.mdLsPas;
        mSfPas              = sarco.mSfPas;
        mLsi0Act            = sarco.mLsi0Act;
        mLenSeriesElement   = sarco.mLenSeriesElement;
        mSfAct              = sarco.mSfAct;
        mvMax               = sarco.mvMax;
        mTimeAct            = sarco.mTimeAct;
        mTR                 = sarco.mTR;
        mTD                 = sarco.mTD;
        mCRest              = sarco.mCRest;
        mAdapt_LsBe         = sarco.mAdapt_LsBe;
        mAdapt_LsEe         = sarco.mAdapt_LsEe;
        mAdapt_SfPasMax     = sarco.mAdapt_SfPasMax;
        mFiberStrain        = sarco.mFiberStrain;
        mC                  = sarco.mC;
        mCDot               = sarco.mCDot;
        mLsi                = sarco.mLsi;
        mLsiDot             = sarco.mLsiDot;
        mLs                 = sarco.mLs;
        mSfPasT             = sarco.mSfPasT;
        mSf                 = sarco.mSf;
        mDSfDEf             = sarco.mDSfDEf;
    }
    return *this;
}

void TSarcomere::copyUserSpecifications(const USarcomere &sarco)
{
    mActivationDelay    = sarco.getActivationdelay();
    mLsRef              = sarco.getLsRef();
    mLs0Pas             = sarco.getLs0Pas();
    mdLsPas             = sarco.getdLsPas();
    mSfPas              = sarco.getSfPas();
    mLsi0Act            = sarco.getLsi0Act();
    mLenSeriesElement   = sarco.getLenSeriesElement();
    mSfAct              = sarco.getSfAct();
    mvMax               = sarco.getvMax();
    mTimeAct            = sarco.getTimeAct();
    mTR                 = sarco.getTR();
    mTD                 = sarco.getTD();
    mCRest              = sarco.getCRest();
    mAdapt_LsBe         = sarco.getAdapt_LsBe();
    mAdapt_LsEe         = sarco.getAdapt_LsEe();
    mAdapt_SfPasMax     = sarco.getAdapt_SfPasMax();

}

void TSarcomere::copySpecifications(const SSarcomere &sarco)
{
    setName( sarco.getName() );
    mActivationDelay    = sarco.getActivationdelay();
    mLsRef              = sarco.getLsRef();
    mLs0Pas             = sarco.getLs0Pas();
    mdLsPas             = sarco.getdLsPas();
    mSfPas              = sarco.getSfPas();
    mLsi0Act            = sarco.getLsi0Act();
    mLenSeriesElement   = sarco.getLenSeriesElement();
    mSfAct              = sarco.getSfAct();
    mvMax               = sarco.getvMax();
    mTimeAct            = sarco.getTimeAct();
    mTR                 = sarco.getTR();
    mTD                 = sarco.getTD();
    mCRest              = sarco.getCRest();
    mAdapt_LsBe         = sarco.getAdapt_LsBe();
    mAdapt_LsEe         = sarco.getAdapt_LsEe();
    mAdapt_SfPasMax     = sarco.getAdapt_SfPasMax();

}


void TSarcomere::resetData( int size )
{
    mSf.resize(size);
    mDSfDEf.resize(size);
    mFiberStrain.resize(size);
    mC.resize(size);
    mCDot.resize(size);
    mLsi.resize(size);
    mLsiDot.resize(size);
    mLs.resize(size);
    mSfPasT.resize(size);
}

void TSarcomere::addSample( const SSarcomere & sim, int pos)
{
    mSf[pos]    = sim.getSf();
    mDSfDEf[pos]= sim.getDSfDEf();
    mFiberStrain[pos]    = sim.getFiberStrain();
    mC[pos]    = sim.getC();           // Contraction
    mCDot[pos]    = sim.getCDot();        // d/dt Contraction
    mLsi[pos]    = sim.getLsi();         // Internal sarcomere length
    mLsiDot[pos]    = sim.getLsiDot();      // d/dt Internal sarcomere length
    mLs[pos]    = sim.getLs();          // Sarcomere length
    mSfPasT[pos]    = sim.getSfPasT();      // Fiber passive stress
}
