/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: ttube.cpp 776 2013-07-11 20:33:36Z eelco $
 *
 */

#include <QDebug>

#include "tube.h"
#include "ttube.h"
#include "utube.h"

using namespace  Peacs::CircAdapt;

TTube::TTube():
    TCompliance(),
    mK(0.0),
    mQ0(0.0),
    mP0(0.0),
    mVesselLength(0.0),
    mPmax(0.0),
    mAWall(0.0),
    mA0(0.0),
    mWallStressAdapt(0.0),
    mVFlowMeanAdapt(0.0),
    mVImpactAdapt(0.0)
{

}

TTube::TTube(const TTube &tube):
    TCompliance(tube),
    mK(tube.mK),
    mQ0(tube.mQ0),
    mP0(tube.mP0),
    mVesselLength(tube.meanVesselLength()),
    mPmax(tube.mPmax),
    mAWall(tube.mAWall),
    mA0(tube.mA0),
    mWallStressAdapt(tube.mWallStressAdapt),
    mVFlowMeanAdapt(tube.mVFlowMeanAdapt),
    mVImpactAdapt(tube.mVImpactAdapt)
{

}


TTube::TTube(const STube &tube):
    TCompliance(tube),
    mK(tube.stiffnessWall()),
    mQ0(tube.meanRestFlow()),
    mP0(tube.meanRestPressure()),
    mVesselLength(tube.meanVesselLength()),
    mPmax(tube.Pmax()),
    mAWall(tube.Awall()),
    mA0(tube.A0()),
    mWallStressAdapt(tube.wallStressAdapt()),
    mVFlowMeanAdapt(tube.vFlowMeanAdapt()),
    mVImpactAdapt(tube.vImpactAdapt())
{

}


const TTube& TTube::operator = (const TTube &tube)
{
    if ( this != &tube )
    {
        TCompliance::operator =(tube);
        mK                  = tube.mK;
        mQ0                 = tube.mQ0;
        mP0                 = tube.mP0;
        mVesselLength       = tube.mVesselLength;
        mPmax               = tube.mPmax;
        mAWall              = tube.mAWall;
        mA0                 = tube.mA0;
        mWallStressAdapt    = tube.mWallStressAdapt;
        mVFlowMeanAdapt     = tube.mVFlowMeanAdapt;
        mVImpactAdapt       = tube.mVImpactAdapt;
    }
    return *this;
}

void TTube::copyUserSpecifications(const UTube &tube)
{
    TCompliance::copyUserSpecifications(tube);
    mK                  = tube.stiffnessWall();
    mQ0                 = tube.meanRestFlow();
    mP0                 = tube.meanRestPressure();
    mVesselLength       = tube.meanVesselLength();
    mPmax               = tube.Pmax();
    mAWall              = tube.Awall();
    mA0                 = tube.A0();
    mWallStressAdapt    = tube.wallStressAdapt();
    mVFlowMeanAdapt     = tube.vFlowMeanAdapt();
    mVImpactAdapt       = tube.vImpactAdapt();
}




void TTube::copySpecifications(const STube &tube)
{
    TCompliance::copySpecifications(tube);
    mK                  = tube.stiffnessWall();
    mQ0                 = tube.meanRestFlow();
    mP0                 = tube.meanRestPressure();
    mVesselLength       = tube.meanVesselLength();
    mPmax               = tube.Pmax();
    mAWall              = tube.Awall();
    mA0                 = tube.A0();
    mWallStressAdapt    = tube.wallStressAdapt();
    mVFlowMeanAdapt     = tube.vFlowMeanAdapt();
    mVImpactAdapt       = tube.vImpactAdapt();
}




