/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: tvalve.cpp 776 2013-07-11 20:33:36Z eelco $
 *
 */
#include "valve.h"
#include "tvalve.h"
#include "uvalve.h"

using namespace Peacs::CircAdapt;


TValve::TValve():
    TFlowDuct(),
    mLen(0.0),    
    mAOpenMax(0.0),
    mALeakMin(0.0),
//    mAOpen(0.0),
//    mALeak(0.0),
    mQDot(0.0),
    mL(0.0)
{

}




TValve::TValve(const SValve &valve):
    TFlowDuct(valve),
    mLen( valve.getLength() ),
    mAOpenMax(valve.getAreaOpenMax()),
    mALeakMin(valve.getAreaLeakMin()),
//    mAOpen(valve.getAreaOpen()  ),
//    mALeak(valve.getALeak() ),
    mQDot(   ),
    mL( )
{

}

const TValve& TValve::operator = (const TValve &valve)
{
    if ( this != &valve)
    {
        TFlowDuct::operator =(valve);
        mLen        = valve.mLen;
        mAOpenMax   = valve.mAOpenMax;
        mALeakMin   = valve.mALeakMin;
//        mAOpen      = valve.mAOpen;
//        mALeak      = valve.mALeak;
        mQDot       = valve.mQDot;
        mL          = valve.mL;
    }
    return *this;
}

void TValve::copyUserSpecifications(const UValve &valve)
{
    TFlowDuct::copyUserSpecifications( valve );

    mLen        = valve.getLength();
    mAOpenMax   = valve.getAreaOpenMax();
    mALeakMin   = valve.getAreaLeakMin();
//    mAOpen      = valve.getAreaOpen();
//    mALeak      = valve.getALeak();

}


void TValve::copySpecifications(const SValve &valve)
{
    TFlowDuct::copySpecifications(valve);

    mLen        = valve.getLength();
    mAOpenMax   = valve.getAreaOpenMax();
    mALeakMin   = valve.getAreaLeakMin();
//    mAOpen      = valve.getAreaOpen();
//    mALeak      = valve.getALeak();

}

void TValve::resetData( int size )
{
    TFlowDuct::resetData(size);
    mQDot.resize(size);
    mL.resize(size);
}

void TValve::addSample( const SValve & sim, int pos)
{
    TFlowDuct::addSample(sim,pos);
    mQDot[pos] = sim.getQdot();
    mL[pos] = sim.getLwithInertia();
}
