/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: tvalve.h 776 2013-07-11 20:33:36Z eelco $
 *
 */

#ifndef __TVALVE_H_
#define __TVALVE_H_

#include "tflowduct.h"
#include "pvector.h"


namespace Peacs
{
    namespace CircAdapt
    {
        class SValve;
        class UValve;

        /**
        Class for valve. Inherits from FlowDuct.
        */
        class TValve: public TFlowDuct {
            Q_OBJECT


        public:

            /* @brief Default Constructor */
            TValve( );

            TValve (const SValve & valve);

            virtual ~TValve() {;}

            const TValve& operator = (const TValve &valve);

            /** @brief copy the sepcification parameters from the user class to the transfer class
              */
            void copyUserSpecifications(const UValve &valve);


            /** @brief copy the sepcification parameters from the simulation class to the transfer class
              * no specification parameters in this class
              */
            void copySpecifications(const SValve &);

            /** @brief At the start of a simulation run the state variables are resized
              * @param size the number of samples in this heart beat
              */
            void resetData( int size );

            /** @brief add a sample to the state variables
              * @param sim the simulate class
              * @param pos the sample number in the state variables
              */
            void addSample( const SValve & sim, int pos);



            double getLength() const { return mLen; }

            double getAreaOpenMax() const {return mAOpenMax; }

            double getAreaLeakMin() const { return mALeakMin; }

//            double getAreaOpen() const  { return mAOpen; }     // Cross-section of open valve

//            double getALeak() const     { return mALeak; }      // Cross-section of closed valve


            // Simulation state variables

            const PVector & getQdot() const      { return mQDot;  }       // d/dt Flow

            const PVector &getLwithInertia() const         { return mL;     }          // Channel length with inertia




        private:
            // Specification parameters
            double mLen;        // Length DEBUG: What is Len??
            double mAOpenMax;
            double mALeakMin;
//            double mAOpen;      // Cross-section of open valve
//            double mALeak;      // Cross-section of closed valve


            // Simulation state variables
            PVector mQDot;       // d/dt Flow
            PVector mL;          // Channel length with inertia

        };
    }
}
// This makes it possible for TTubeState values to be stored in QVariant
// objects and retrieved later
Q_DECLARE_METATYPE( Peacs::CircAdapt::TValve )


#endif
