/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: tventricle.cpp 776 2013-07-11 20:33:36Z eelco $
 *
 */

#include <QDebug>
#include "tventricle.h"
#include "ventricle.h"
#include "uventricle.h"



using namespace Peacs::CircAdapt;

// Constructor
TVentricle::TVentricle() :
    TCircElement(),
    mLeftV(),
    mSeptV(),
    mRigtV(),
    mVentrType(BISEG)
{

}

TVentricle::TVentricle( const TVentricle & ventricle ) :
    TCircElement( ventricle ),
    mLeftV( ventricle.mLeftV ),
    mSeptV( ventricle.mSeptV ),
    mRigtV( ventricle.mRigtV ),
    mVentrType( ventricle.mVentrType )
{

}


TVentricle::TVentricle( const SVentricle & ventricle ) :
    TCircElement( ventricle.getName() ),
    mLeftV( ventricle.getLeftVentricle() ),
    mSeptV( ventricle.getSeptum() ),
    mRigtV( ventricle.getRightVentricle() ),
    mVentrType( ventricle.getType() )
{

}

void TVentricle::copyUserSpecifications(const UVentricle &ventr)
{
    mLeftV.copyUserSpecifications( ventr.getLeftVentricle() );
    mRigtV.copyUserSpecifications( ventr.getRightVentricle() );
    mSeptV.copyUserSpecifications( ventr.getSeptum() );
}

void TVentricle::copySpecifications(const SVentricle &ventr)
{
    setName(ventr.getName());
    mLeftV.copySpecifications( ventr.getLeftVentricle() );
    mRigtV.copySpecifications( ventr.getRightVentricle() );
    mSeptV.copySpecifications( ventr.getSeptum() );
}

const TVentricle& TVentricle::operator = (const TVentricle &ventricle)
{
    if ( this != &ventricle)
    {
        TCircElement::operator =(ventricle);
        mLeftV = ventricle.mLeftV;
        mSeptV =  ventricle.mSeptV;
        mRigtV = ventricle.mRigtV;
        mVentrType = ventricle.mVentrType;
    }
    return *this;
}


void TVentricle::resetData( int size )
{
    mLeftV.resetData(size);
    mRigtV.resetData(size);
    mSeptV.resetData(size);
}


void TVentricle::addSample( const SVentricle & sim, int pos)
{
    mLeftV.addSample(sim.getLeftVentricle(),pos);
    mRigtV.addSample(sim.getRightVentricle(),pos);
    mSeptV.addSample(sim.getSeptum(),pos);
}

