/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: tventricle.h 776 2013-07-11 20:33:36Z eelco $
 *
 */

#ifndef __TVENTRICLE_H_
#define __TVENTRICLE_H_

#include "tchamber.h"



using namespace Peacs::Support;
namespace Peacs
{

    namespace CircAdapt
    {
        class SVentricle;
        class UVentricle;

        /**
        Class for muscular chamber. Inherits from UCompliance.
        */
        class TVentricle : public TCircElement
        {
            Q_OBJECT

        public:
            TVentricle();

            TVentricle(const SVentricle & ventricle);

            TVentricle(const TVentricle & ventricle);

            virtual ~TVentricle() {;}


            virtual const TVentricle& operator = (const TVentricle &ventr);



            /** @brief copy the sepcification parameters from the user class to the transfer class
              */
            void copyUserSpecifications(const UVentricle &);


            /** @brief copy the sepcification parameters from the simulation class to the transfer class
              * no specification parameters in this class
              */
            void copySpecifications(const SVentricle &);

            /** @brief At the start of a simulation run the state variables are resized
              * @param size the number of samples in this heart beat
              */
            void resetData( int size );

            /** @brief add a sample to the state variables
              * @param sim the simulate class
              * @param pos the sample number in the state variables
              */
            void addSample( const SVentricle & sim, int pos);


            inline const TChamber & getRightVentricle() const { return mRigtV; }

            inline TChamber & rightVentricle() { return mRigtV; }

            inline const TChamber & getLeftVentricle() const { return mLeftV; }

            inline TChamber & leftVentricle() { return mLeftV; }

            inline const TChamber & getSeptum() const { return mSeptV; }

            inline TChamber & septum() { return mSeptV; }


            inline ventricle_type getType() const { return mVentrType; }

            double getVolume() const;

        private:
            TChamber mLeftV;

            TChamber mSeptV;

            TChamber mRigtV;

            ventricle_type mVentrType;

        };
    }
}
// This makes it possible for TTubeState values to be stored in QVariant
// objects and retrieved later
Q_DECLARE_METATYPE( Peacs::CircAdapt::TVentricle )


#endif
