/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: uatria.h 776 2013-07-11 20:33:36Z eelco $
 *
 */

#ifndef __UATRIA_H_
#define __UATRIA_H_

#include "defs.h"
#include "uchamber.h"
#include "ucircelement.h"
#include "PCircAdaptDefines.h"


using namespace Peacs::Support;
namespace Peacs
{

    namespace CircAdapt
    {

        class TAtria;
        /**
        Class for muscular chamber. Inherits from UCompliance.
        */
        class UAtria: public UCircElement
        {

        public:
            UAtria(int size= DEFAULT_SIGNAL_LENGTH, const QColor & color=QColor(255,255,255));

            UAtria(const UAtria & atria);

            virtual ~UAtria() {;}


            virtual const UAtria& operator = (const UAtria &atria);


            /** @brief copy the sepcification parameters from the simulation class to the transfer class
              * no specification parameters in this class
              */
            void copySpecifications(const TAtria &);

            /** @brief At the start of a simulation run the state variables are resized
              * @param size the number of samples in this heart beat
              */
            void resetData( int size );

            /** @brief add a sample to the state variables
              * @param sim the simulate class
              * @param pos the sample number in the state variables
              */
            void addData( const TAtria & sim);


            /** @brief save the signals in the variables
              * @see UCircAdaptSignal*/
            inline void saveSignals() {
                mRigtA.saveSignals();
                mLeftA.saveSignals();
            }

            void setRefValues() { mLeftA.setRefValues();
                                  mRigtA.setRefValues(); }

            inline const UChamber & getRightAtrium() const { return mRigtA; }

            inline const UChamber & getLeftAtrium() const { return mLeftA; }


            inline UChamber & rightAtrium() { return mRigtA; }

            inline UChamber & leftAtrium() { return mLeftA; }

            inline double getRelSfAct() const { return mLeftA.getRelSfAct(); }

            inline double getRelSfPas() const { return mLeftA.getRelSfPas(); }


        private:
            UChamber mLeftA;

            UChamber mRigtA;
        };
    }
}
#endif
