/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id:
 *
 */


#include "ucircadaptsignal.h"
#include "circsettings.h"

//#include <QDebug>


using namespace Peacs::Support;
using namespace Peacs::CircAdapt;

UCircAdaptSignal::UCircAdaptSignal(const UCircElement &element, const QString &units, int size) :
    mSize( size ),
    mElement( element ),
    mUnits( units ),
//    mSignal(),
//    mSignal( size ),
    mLastBeat(),
    mReferenceSignal(),
    mSavedSignal(),
    mReferenceUpdated( false ),
    mSaveReference( -1 ),   // Take the first heartbeat that comes in
    mSavedUpdated( false ),
    mSaveSaved( 0 ),
    mSampleTime( (double)CIRCMONTIMESTEP / 1000.0 ),
//    mStatistics( mLastBeat, PStatistics::doMinMax ),
    mSavedStatistics( mSavedSignal, PStatistics::doMinMax ),
    mReferenceStatistics( mReferenceSignal, PStatistics::doMinMax )
{
    reset( size );
}


UCircAdaptSignal::UCircAdaptSignal( const UCircAdaptSignal & signal ):
    mSize( signal.mSize ),
    mElement( signal.mElement ),
    mUnits( signal.mUnits ),
//    mSignal( signal.mSignal ),
    mLastBeat( signal.mLastBeat ),
    mReferenceSignal( signal.mReferenceSignal ),
    mSavedSignal( signal.mSavedSignal ),
    mReferenceUpdated( signal.mReferenceUpdated ),
    mSaveReference( signal.mSaveReference ),
    mSavedUpdated( signal.mSavedUpdated ),
    mSaveSaved( signal.mSaveSaved ),
    mSampleTime( signal.mSampleTime ),
//    mStatistics( mLastBeat, PStatistics::doMinMax ),
    mSavedStatistics( mSavedSignal, PStatistics::doMinMax),
    mReferenceStatistics( mReferenceSignal, PStatistics::doMinMax)
{
}

const UCircAdaptSignal &UCircAdaptSignal::operator= (const UCircAdaptSignal & signal)
{
    if ( this != & signal )
    {
        mSize               = signal.mSize;
        mUnits              = signal.mUnits;
//        mSignal             = signal.mSignal;
        mLastBeat           = signal.mLastBeat;
        mReferenceSignal    = signal.mReferenceSignal;
        mSavedSignal        = signal.mSavedSignal;
        mReferenceUpdated   = signal.mReferenceUpdated;
        mSaveReference      = signal.mSaveReference;
        mSavedUpdated       = signal.mSavedUpdated;
        mSaveSaved          = signal.mSaveSaved;
        mSampleTime         = signal.mSampleTime;

        // TODO Add asignment operator to the statistics class
    }
    return *this;
}

void UCircAdaptSignal::reset(int size)
{
//    mSignal.clear();
    mSavedSignal.clear();
    mReferenceSignal.clear();
    mLastBeat.clear();

    mReferenceUpdated   = false;
    mSaveReference      = -1;    // Take the first heartbeat that comes in
    mSavedUpdated       = false;
    mSaveSaved          = 0;
}


double UCircAdaptSignal::getMin( void ) const
{
    double min = 0.0;

    if ( mLastBeat.size() > 0 )
    {
        PStatistics statistics( mLastBeat, PStatistics::doMin );

        min = qMin( statistics.getMin(),
                    qMin( mSavedStatistics.getMin(), mReferenceStatistics.getMin() ) );
    }
    else
    {
        min = qMin( mSavedStatistics.getMin(), mReferenceStatistics.getMin() );
    }

    return min;
}


double UCircAdaptSignal::getMax( void ) const
{
    double max = 0.0;

    if ( mLastBeat.size() > 0 )
    {
        PStatistics statistics( mLastBeat.last(), PStatistics::doMax );

        max = qMax( statistics.getMax(),
                    qMax( mSavedStatistics.getMax(), mReferenceStatistics.getMax() ) );
    }
    else
    {
        max = qMax( mSavedStatistics.getMax(), mReferenceStatistics.getMax() );
    }

    return max;
}


//int UCircAdaptSignal::getSignalCount( void ) const
//{
//    int itemCount = 0;

//    // First find the container that holds the value at 'index'
//    for ( int liIndex = 0; liIndex < mSignal.size(); liIndex++ )
//    {
//        itemCount += mSignal.at( liIndex ).N();
//    }

//    return itemCount;
//}


//double UCircAdaptSignal::getSignalSample(int index) const
//{
//    double value    = 0.0;
//    int listItem    = -1;
//    int itemIndex   = 0;

//    // First find the container that holds the value at 'index'
//    for ( int liIndex = 0; (liIndex < mSignal.size()) && (listItem == -1); liIndex++ )
//    {
//        int tmp = mSignal.at( liIndex ).N();

//        if ( itemIndex + tmp > index )
//        {
//            // We found the corresponding value
//            listItem    = liIndex;
//            value       = mSignal.at( liIndex ).at( index - itemIndex );
//        }
//        else
//        {
//            itemIndex += tmp;
//        }
//    }

//    return value;
//}


void UCircAdaptSignal::addBeat( const PVector & beat )
{
    const double* source = beat.data();

    if ( mSaveReference < mReferenceSignal.N() )
    {
        mReferenceSignal.resize( beat.N() );

        double* dest = mReferenceSignal.data();

        memcpy( (void *)dest,
                (const void *)source,
                size_t( beat.N() * sizeof(double) ) );

        mSaveReference = beat.N();

        double prevMax = mReferenceStatistics.getMax();
        double prevMin = mReferenceStatistics.getMin();

        mReferenceStatistics.updateStatistics( PStatistics::doMinMax );

        // Test if the initial signal is valid; this means that the signal is approximately equal to
        // the previous signal, i.e. is stable
        if ( (fabs(prevMax - mReferenceStatistics.getMax()) / mReferenceStatistics.getMax() > 0.0051) ||
             (fabs(prevMin - mReferenceStatistics.getMin()) / mReferenceStatistics.getMin() > 0.0051) )
        {
            // If not stable, the next signal to recieve may the needed one and this signal may be
            // garbaged
            mSaveReference = -1;
            mReferenceSignal.clear();
        }
        else
        {
            mReferenceUpdated = true;
        }
    }
    else
    {
        mReferenceUpdated = false;
    }

    if ( mSaveSaved < mSavedSignal.N() )
    {
        mSavedSignal.resize( beat.N() );

        double* dest  = mSavedSignal.data();

        memcpy( (void *)dest,
                (const void *)source,
                size_t( beat.N() * sizeof(double) ) );

        mSavedUpdated   = true;
        mSaveSaved      = beat.N();

        mSavedStatistics.updateStatistics( PStatistics::doMinMax );
    }
    else
    {
        mSavedUpdated = false;
    }

//    // TODO: Make the number of beats stored (12) configurable by using a member
//    if ( mSignal.count() > 12 )
//    {
//        mSignal.removeFirst();
//    }

//    mSignal.append( beat );

    mLastBeat = beat;
}


//PVector UCircAdaptSignal::getSignal( int nSamples ) const
//{
//    PVector sig;

//    bool    done            = false;
//    int     extraBeatCount  = 1;    // Always get the last whole beat. This results in a signal that is drawn outside the item-view
//    int     sampleCount     = 0;

//    // Get the number of heartbeats to retrieve
//    for ( int liIndex = mSignal.size() - 1; (liIndex >= 0) && !done; liIndex-- )
//    {
//        if ( sampleCount + mSignal.at( liIndex ).N() < nSamples )
//        {
//            sampleCount += mSignal.at( liIndex ).N();

//            extraBeatCount++;
//        }
//        else
//        {
//            done = true;
//        }
//    }

//    extraBeatCount = qMin( extraBeatCount, mSignal.size() );

//    for ( int liIndex = mSignal.size() - extraBeatCount; liIndex < mSignal.size(); liIndex++ )
//    {
//        sig.append( mSignal.at( liIndex ) );
//    }

//    return sig;
//}


PVector UCircAdaptSignal::getReferenceSignal( int minTime, int maxTime ) const
{
    PVector signal;

    getSignal( signal, minTime, maxTime, mReferenceSignal );

    //const_cast<UCircAdaptSignal*>(this)->mReferenceUpdated = false;

    return signal;
}


PVector UCircAdaptSignal::getSavedSignal( int minTime, int maxTime ) const
{
    PVector signal;

    getSignal( signal, minTime, maxTime, mSavedSignal );

    //const_cast<UCircAdaptSignal*>(this)->mSavedUpdated = false;

    return signal;
}


void UCircAdaptSignal::getSignal( PVector& signal, int minTime, int maxTime, const PVector& vector ) const
{
    int sigSize = vector.N();

    Q_ASSERT( minTime < maxTime );

    if ( sigSize > 0 )     // If there is data present ...
    {
        int             totalSampleCount    = maxTime - minTime;
        int             multiplyCount       = 0;
        const double *   sourcePtr           = vector.data();
        double *         destPtr             = 0;

        // This function will return a sequence of the stored sample. Due to this, the minTime
        // can be bigger then 0. If this is the case we have to find the exact location in the
        // virtual sequence of this sample to get the starting point
        while ( minTime >= sigSize )
        {
            minTime -= sigSize;
            maxTime -= sigSize;
        }

        while ( maxTime >= sigSize )
        {
            multiplyCount++;    // One extra sample must be added

            maxTime -= sigSize;
        }

        // Now we know how much samples we need and from which upto where data to copy
        signal.resize( totalSampleCount );

        destPtr = signal.data();

        if ( multiplyCount == 0 )   // Needed samples are within one sample
        {
            memcpy( (void *)destPtr,
                    (const void *)&sourcePtr[ minTime ],
                    size_t( totalSampleCount * sizeof(double) ) );
        }
        else
        {
            int insertPtr = 0;

            // We need more then one sample, or the needed sample doesn't start at the beginning
            // Start copying the starting bits
            memcpy( (void *)destPtr,
                    (const void *)&sourcePtr[ minTime ],
                    size_t( (sigSize - minTime) * sizeof(double) ) );

            totalSampleCount    -= sigSize - minTime;
            insertPtr           += sigSize - minTime;

            // Add whole samples as long as needed
            while ( totalSampleCount >= sigSize )
            {
                memcpy( (void *)&destPtr[ insertPtr ],
                        (const void *)sourcePtr,
                        size_t( sigSize * sizeof(double) ) );

                totalSampleCount    -= sigSize;
                insertPtr           += sigSize;
            }

            // Finally add the tailing samples upto the end if there is some left
            if ( 0 < totalSampleCount )
            {
                memcpy( (void *)&destPtr[ insertPtr ],
                        (const void *)sourcePtr,
                        size_t( totalSampleCount * sizeof(double) ) );
            }
        }
    }
}


const PVector * UCircAdaptSignal::getLastBeat( void ) const
{
//    if ( mSignal.size() > 0 )
//    {
//        return & mSignal.last();
//    }
//    else
//    {
//        return NULL;
//    }
    return & mLastBeat;
}
