/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id:
 *
 */

#ifndef __TCIRCADAPTSIGNAL_H
#define __TCIRCADAPTSIGNAL_H

#include <QString>
#include <QColor>
#include "pvector.h"
#include "PStatistics.h"

#include "ucircelement.h"


namespace Peacs
{
using namespace Support;
    namespace CircAdapt
    {

        class UCircAdaptSignal
        {


        public:
            UCircAdaptSignal(const UCircElement &element,const QString &units, int size );

            UCircAdaptSignal(const UCircAdaptSignal &signal);

            const UCircAdaptSignal &operator= (const UCircAdaptSignal & signal);


            inline int getSize( void ) const { return mSize; }

            const QString & getName() const { return mElement.getName(); }

            const QString & getUnits() const { return mUnits; }


            void addBeat( const PVector & beat );


            //int getSignalCount( void ) const;


            /** @brief Retrieves the value corresponding to the specified index
              *
              * @param index zero based index of the value to retrieve
              */
            //double getSignalSample( int index ) const;


            /** @brief Returns mReferenceUpdated
              */
            inline bool isReferenceUpdated( void ) const { return mReferenceUpdated; }


            /** @brief Returns mReferenceSignal
              */
            inline const PVector & getReferenceSignal( void ) const { return mReferenceSignal; }


            /** @brief Returns the stored initial signal data between minTime and maxTime
              * If the signal data contains less data then requested extra data will be added
              * by adding the same initial signal data until the requested time interval is
              * filled
              *
              * @param minTime The start time index to retrieve
              * @param maxTime The end time index to retrieve
              */
            PVector getReferenceSignal( int minTime, int maxTime ) const;


            /** @brief Starts a new retrieval of the initial heartbeat
              * Sets mSaveReference to -1, meaning that the next heartbeat must be stored.
              * A previously stored initial heartbeat is cleared
              */
            inline void startSaveReference() { mSaveReference = -1; mReferenceSignal.clear(); }


            /** @brief Returns mSavedUpdated
              */
            inline bool isSavedUpdated( void ) const { return mSavedUpdated; }


            /** @brief
              */
            inline const PVector & getSavedSignal( void ) const { return mSavedSignal; }


            /** @brief Returns the saved (snapshot) signal data between minTime and maxTime
              * If the signal data contains less data then requested extra data will be added
              * by adding the same saved signal data until the requested time interval is
              * filled
              *
              * @param minTime The start time index to retrieve
              * @param maxTime The end time index to retrieve
              */
            PVector getSavedSignal( int minTime, int maxTime ) const;


            /** @brief Starts a new retrieval of a heartbeat for the snapshot
              * Sets mSaveSaved to -1, meaning that the next heartbeat must be stored.
              * A previously stored heartbeat is cleared
              */
            inline void startSave( void ) { mSaveSaved = -1; mSavedSignal.clear(); }


            inline bool containsReference() const { return mReferenceSignal.N() > 0; }

            inline bool containsSaved() const { return mSavedSignal.N() > 0; }

            //PVector getSignal( int nSamples ) const;


            /** @brief Returns the last recieved heartbeat or NULL if not found
              */
            const PVector * getLastBeat( void ) const;

            inline double getSampleTime() const { return mSampleTime; }

            inline const QColor &getColor() const { return mElement.getColor(); }

            /** @brief set the size of the signal to size and all values to zero
              */
            void reset(int size);

            double getMin( void ) const;

            double getMax( void ) const;

        private:

            void getSignal( PVector& signal, int minTime, int maxTime, const PVector& vector ) const;

            int mSize;

            const UCircElement &mElement;

            QString mUnits;

            //QList<PVector> mSignal;

            PVector mLastBeat;

            PVector mReferenceSignal;

            PVector mSavedSignal;

            /** @brief If set to true, the reference signal data has changed and can be used
              */
            bool mReferenceUpdated;


            int mSaveReference;


            /** @brief If set to true, the saved (snapshot) signal data has changed and can be used
              */
            bool mSavedUpdated;


            int mSaveSaved;

            double mSampleTime; // in ms

            //PStatistics mStatistics;

        public:
            PStatistics mSavedStatistics;

            PStatistics mReferenceStatistics;
        };
    }
}
#endif // __TCIRCADAPTSIGNAL_H
