/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: ucompliance.cpp 776 2013-07-11 20:33:36Z eelco $
 *
 */

#include <QString>

#include "ucompliance.h"
#include "PCircAdaptDefines.h"


using namespace Peacs::CircAdapt;

// Constructor
UCompliance::UCompliance(int size, const QColor &color):
    UCircElement(color),
    mVolume( *this, VOLUME_UNITS,size),
    mVDot( *this, DVOLUME_DT_UNITS,size),
    mA( *this, AREA_UNITS,size),
    mP( *this, PRESSURE_UNITS,size),
    mZ( *this, IMPEDANCE_UNITS,size),
    mPin( *this, PRESSURE_UNITS,size),
    mQRemod( *this, FLOW_UNITS,size)
{

}


UCompliance::UCompliance(const UCompliance &comp):
    UCircElement(comp.getName()),
    mVolume(comp.mVolume),
    mVDot(comp.mVDot),
    mA(comp.mA),
    mP(comp.mP),
    mZ(comp.mZ),
    mPin(comp.mPin),
    mQRemod(comp.mQRemod)
{

}


const UCompliance& UCompliance::operator = (const UCompliance &comp)
{
    if (this != &comp )
    {
        UCircElement::operator =(comp);

        mVolume     = comp.mVolume;
        mVDot       = comp.mVDot;
        mA          = comp.mA;
        mP          = comp.mP;
        mZ          = comp.mZ;
        mPin        = comp.mPin;
        mQRemod     = comp.mQRemod;
    }

    return *this;
}

void UCompliance::copySpecifications(const TCompliance &comp)
{
    setName( comp.getName() );
}


void UCompliance::resetData( int size )
{
    mVolume.reset(size);  // Volume of compliance

    mVDot.reset(size);    // d/dt Volume

    mA.reset(size);       // Cross-section of compliance

    mP.reset(size);       // Pressure of compliance

    mZ.reset(size);       // Impedance

    mPin.reset(size);     // Pressure at entrance

    mQRemod.reset(size);  // Through flow used for diameter adaptation

}



void UCompliance::addData(const TCompliance & comp)
{
    mVolume.addBeat(comp.getVolume() );

    mVDot.addBeat( comp.getVolumeDot() );

    mA.addBeat( comp.getArea() );

    mP.addBeat( comp.getPressure() );

    mZ.addBeat( comp.getImpedance() );

    mPin.addBeat(comp.getPressureAtEntrance() );

//    mQRemod.addBeat(comp.getQRemod() );
}


void UCompliance::saveSignals()
{
    mVolume.startSave();

    mVDot.startSave();

    mA.startSave();

    mP.startSave();

    mZ.startSave();

    mPin.startSave();

//    mQRemod.startSave();

}
