/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: uflowduct.h 776 2013-07-11 20:33:36Z eelco $
 *
 */

#ifndef __UFLOWDUCT_H_
#define __UFLOWDUCT_H_


#include "ucircelement.h"
#include "ucircadaptsignal.h"
#include "PCircAdaptDefines.h"

namespace Peacs
{
    namespace CircAdapt
    {
        /**
        Class for flow channel between compliances. Inherits from CircElement.
        */
        class TFlowDuct;

        class UFlowDuct: public UCircElement {

        public:

            UFlowDuct ( int size = DEFAULT_SIGNAL_LENGTH,const QColor & color = QColor(255,255,255));

            UFlowDuct (const UFlowDuct & flowduct);

            /// Destructor
            virtual ~UFlowDuct() {  }

            const UFlowDuct & operator = (const UFlowDuct &flowduct);


            /** @brief copy the sepcification parameters from the simulation class to the transfer class
              * no specification parameters in this class
              */
            virtual void copySpecifications(const TFlowDuct &flow);

            /** @brief At the start of a simulation run the state variables are resized
              * @param size the number of samples in this heart beat
              */
            virtual void resetData( int size );

            /** @brief add a sample to the state variables
              * @param sim the transfer class
              */
            virtual void addData( const TFlowDuct & sim);

            /** @brief save the signals in the variables
              * @see UCircAdaptSignal*/
            virtual void saveSignals();

            virtual void setRefValues() {}

            /// Method to get flow
            inline const UCircAdaptSignal & getFlow() const { return mQ; }

            inline UCircAdaptSignal & flow() { return mQ; }


        protected:
            // Simulation state variables
            UCircAdaptSignal mQ;          // Flow
        };
    }
}

#endif
