/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: uheart.cpp 776 2013-07-11 20:33:36Z eelco $
 *
 */

#include <QDebug>

#include <QMap>

#include "uheart.h"
using namespace Peacs::CircAdapt;


QDebug & operator<< ( QDebug & dbg, const UHeart & state )
{
    dbg.nospace() << "THeartState:";
//    dbg.nospace() << "mLeftAtrialPressure : " << state.getLeftAtrialPressure();
//    dbg.nospace() << "mLeftAtrialVolume : " << state.getLeftAtrialVolume();
//    dbg.nospace() << "mRightAtrialPressure : " << state.getRightAtrialPressure();
//    dbg.nospace() << "mRightAtrialVolume : " << state.getRightAtrialVolume();
//    dbg.nospace() << "mLeftVentricularPressure : " << state.getLeftVentricularPressure();
//    dbg.nospace() << "mLeftVentricularVolume : " << state.getLeftVentricularVolume();
//    dbg.nospace() << "mRightVentricularPressure : " << state.getRightVentricularPressure();
//    dbg.nospace() << "mRightVentricularVolume : " << state.getRightVentricularVolume();

    return dbg.maybeSpace();
}


// Constructor
UHeart::UHeart(int size, const QColor & color):
    UCircElement("",color),
    mAtria(size,ATRIA_COLOR),
    mVentricle(size,VENTRICLE_COLOR),
    mPericardialSack(size,HEART_COLOR),
    mValveLVen(size ,LVEN_COLOR),
    mValveLAv( size, LAV_COLOR),
    mValveLArt( size, LART_COLOR),
    mValveRVen(size, RVEN_COLOR),
    mValveRAv( size ,RAV_COLOR),
    mValveRArt( size, RART_COLOR),
    mValveASD( size, ASD_COLOR ),
    mValveVSD( size, VSD_COLOR),
    mValveDUCT(size, DUCT_COLOR )
{
}

UHeart::UHeart( const UHeart & heart ) :
    UCircElement(heart),
    mAtria(heart.mAtria),
    mVentricle(heart.mVentricle),
    mPericardialSack(heart.mPericardialSack),
    mValveLVen(heart.mValveLVen ),
    mValveLAv( heart.mValveLAv ),
    mValveLArt( heart.mValveLArt),
    mValveRVen(heart.mValveRVen ),
    mValveRAv( heart.mValveRAv ),
    mValveRArt( heart.mValveRArt),
    mValveASD( heart.mValveASD ),
    mValveVSD( heart.mValveVSD ),
    mValveDUCT(heart.mValveDUCT )
{

}


void UHeart::copySpecifications(const THeart &heart)
{
    mAtria.copySpecifications(           heart.getAtria() );
    mVentricle.copySpecifications(       heart.getVentricles() );
    mPericardialSack.copySpecifications( heart.getSack() );
    mValveLVen.copySpecifications(heart.getValveLVen());
    mValveLAv.copySpecifications( heart.getValveLAv() );
    mValveLArt.copySpecifications(heart.getValveLArt());
    mValveRVen.copySpecifications(heart.getValveRVen() );
    mValveRAv.copySpecifications( heart.getValveRAv() );
    mValveRArt.copySpecifications(heart.getValveRArt());
    mValveASD.copySpecifications( heart.getValveASD() );
    mValveVSD.copySpecifications( heart.getValveVSD() );
    mValveDUCT.copySpecifications(heart.getValveDUCT() );

}

void UHeart::resetData(int size)
{
    mAtria.resetData(size);
    mVentricle.resetData(size);
    mPericardialSack.resetData(size);
    mValveLVen.resetData(size );
    mValveLAv.resetData( size );
    mValveLArt.resetData( size);
    mValveRVen.resetData(size );
    mValveRAv.resetData( size );
    mValveRArt.resetData( size);
    mValveASD.resetData( size );
    mValveVSD.resetData( size );
    mValveDUCT.resetData(size );
}

void UHeart::addData( const THeart & heart )
{
    mAtria.addData(heart.getAtria());
    mVentricle.addData(heart.getVentricles());
    mPericardialSack.addData(heart.getSack());
    mValveLVen.addData(heart.getValveLVen());
    mValveLAv.addData( heart.getValveLAv() );
    mValveLArt.addData(heart.getValveLArt());
    mValveRVen.addData(heart.getValveRVen() );
    mValveRAv.addData( heart.getValveRAv() );
    mValveRArt.addData(heart.getValveRArt());
    mValveASD.addData( heart.getValveASD() );
    mValveVSD.addData( heart.getValveVSD() );
    mValveDUCT.addData(heart.getValveDUCT() );
}

void UHeart::saveSignals()
{
    mAtria.saveSignals();
    mVentricle.saveSignals();
    mPericardialSack.saveSignals();

    mValveLVen.saveSignals();
    mValveLAv.saveSignals();
    mValveLArt.saveSignals();
    mValveRVen.saveSignals();
    mValveRAv.saveSignals();
    mValveRArt.saveSignals();
    mValveASD.saveSignals();
    mValveVSD.saveSignals();
    mValveDUCT.saveSignals();
}

void UHeart::setRefValues()
{
    mAtria.setRefValues();
    mVentricle.setRefValues();
//    mPericardialSack.setRefValues();

    mValveLVen.setRefValues();
    mValveLAv.setRefValues();
    mValveLArt.setRefValues();
    mValveRVen.setRefValues();
    mValveRAv.setRefValues();
    mValveRArt.setRefValues();
    mValveASD.setRefValues();
    mValveVSD.setRefValues();
    mValveDUCT.setRefValues();
}


const UHeart & UHeart::operator= ( const UHeart & heart )
{
    if ( this != &heart )
    {
        mAtria     = heart.mAtria;
        mVentricle = heart.mVentricle;
        mPericardialSack = heart.mPericardialSack;
        mValveLVen = heart.mValveLVen;            // Inlet left atrium
        mValveLAv  = heart.mValveLAv;             // Mitral valve
        mValveLArt = heart.mValveLArt;            // Aortic valve
        mValveRVen = heart.mValveRVen;            // Inlet right atrium
        mValveRAv  = heart.mValveRAv;             // Tricuspid valve
        mValveRArt = heart.mValveRArt;            // Pulmonary valve
        mValveASD  = heart.mValveASD;            // Valve for ASD
        mValveVSD  = heart.mValveVSD;           // Valve for VSD
        mValveDUCT = heart.mValveDUCT;            // Valve for DUCT))
    }
    return *this;
}


double UHeart::getVolume() const
{
    if ( ( mAtria.getLeftAtrium().getVolume().getLastBeat() != NULL ) &&
         !mAtria.getLeftAtrium().getVolume().getLastBeat()->isEmpty() &&
         ( mVentricle.getLeftVentricle().getVolume().getLastBeat() != NULL ) &&
         !mVentricle.getLeftVentricle().getVolume().getLastBeat()->isEmpty() )
    {
        return mAtria.getLeftAtrium().getVolume().getLastBeat()->last() +
               mAtria.getRightAtrium().getVolume().getLastBeat()->last() +
               mVentricle.getLeftVentricle().getVolume().getLastBeat()->last() +
               mVentricle.getRightVentricle().getVolume().getLastBeat()->last();
    }

    return 5.0;
}
