/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: upatch.cpp 776 2013-07-11 20:33:36Z eelco $
 *
 */

#include <QDebug>

#include "upatch.h"
#include "tpatch.h"
#include "uchamber.h"




using namespace  Peacs::CircAdapt;
// Constructor
UPatch::UPatch(int size, const QColor &color):
    UCircElement(color),
    mAmRef(0.0),
    mVWall(0.0),
    mPatchAreaMid( *this,   AREA_UNITS, size),
    mCm( *this,  CURVATIRE_UNITS, size),
    mT( *this,  TENSION_UNITS, size),
    mPTrans( *this,  PRESSURE_UNITS, size),
    mDTDAm( *this,  STIFFNESS_UNITS, size),
    mEf( *this,  STRAIN_UNITS, size),
    mSarcomere(size,color)
{
}


UPatch::UPatch(const UPatch &patch) :
    UCircElement( patch.getName()        ),
    mAmRef(         patch.mAmRef         ),
    mVWall(         patch.mVWall         ),
    mPatchAreaMid(  patch.mPatchAreaMid  ),
    mCm(            patch.mCm            ),
    mT(             patch.mT             ),
    mPTrans(        patch.mPTrans        ),
    mDTDAm(         patch.mDTDAm         ),
    mEf(            patch.mEf            ),
    mSarcomere(     patch.getSarcomere() )
{
}


const UPatch& UPatch::operator = (const UPatch &patch)
{
    if ( this != &patch)
    {
        UCircElement::operator = (patch);

        mAmRef          = patch.mAmRef;
        mVWall          = patch.mVWall;
        mPatchAreaMid   = patch.mPatchAreaMid;
        mCm             = patch.mCm;
        mT              = patch.mT;
        mPTrans         = patch.mPTrans;
        mDTDAm          = patch.mDTDAm;
        mEf             = patch.mEf;
        mSarcomere      = patch.mSarcomere;
    }
    return *this;
}

void UPatch::copySpecifications(const TPatch &patch)
{
    setName(patch.getName());
    mAmRef = patch.getRefArea();
    mVWall = patch.getWallVolume();
    mSarcomere.copySpecifications( patch.getSarcomere() );
}

void UPatch::resetData( int size )
{
    mPatchAreaMid.reset(size);
    mCm.reset(size);
    mT.reset(size);
    mPTrans.reset(size);
    mDTDAm.reset(size);
    mEf.reset(size);
    mSarcomere.resetData( size );

}

void UPatch::addData( const TPatch & sim)
{
    mPatchAreaMid.addBeat(sim.getArea());
    mCm.addBeat(sim.getCurvature());
    mT.addBeat(sim.getWallTension());
    mPTrans.addBeat(sim.getTransmuralPressure());
    mDTDAm.addBeat(sim.getWallStiffness());
    mEf.addBeat(sim.getFiberStrain());
    mSarcomere.addData( sim.getSarcomere() );
}

void UPatch::saveSignals()
{
    mPatchAreaMid.startSave();
    mCm.startSave();
    mT.startSave();
    mPTrans.startSave();
    mDTDAm.startSave();
    mEf.startSave();
    mSarcomere.saveSignals();
}

void UPatch::setRefValues()
{
    mSarcomere.setRefValues();
}
