/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: uresistance.cpp 776 2013-07-11 20:33:36Z eelco $
 *
 */

#include <QDebug>
#include <QStringList>

#include "uresistance.h"
#include "PCircAdaptDefines.h"

using namespace Peacs::CircAdapt;


QDebug & operator<< ( QDebug & dbg, const UResistance & state )
{
    dbg.nospace() << "UResistance:";
//    dbg.nospace() << "mName : " << state.getName();

//    dbg.nospace() << "mFlow : " << state.getFlow();

    return dbg.maybeSpace();
}


UResistance::UResistance(int size):
    UFlowDuct(size),
    mR(0),
    mRefR(0),
    mRelNonVitalRes(1.0),
    mResistanceType( SResistance::UNDEFINED_RESISTANCE )
{
}


UResistance::UResistance( const UResistance & res ) :
    UFlowDuct( res),
    mR(res.mR),
    mRefR(res.mRefR),
    mRelNonVitalRes( res.mRelNonVitalRes ),
    mResistanceType( res.getResistanceType() )
{
}

const UResistance & UResistance::operator= ( const UResistance & state )
{
    if ( this != & state )
    {
        UFlowDuct::operator =(state);

        mR              = state.mR;
        mRefR           = state.mRefR;
        mRelNonVitalRes = state.mRelNonVitalRes;
        mResistanceType = state.getResistanceType();
    }
    return *this;
}


void UResistance::resetData(int size)
{
    UFlowDuct::resetData(size);

    mR = mRefR;
}

void UResistance::copySpecifications(const TResistance &res)
{
    UFlowDuct::copySpecifications( res );
    mR = res.getResistance();
    mResistanceType = res.getResistanceType();
}

void UResistance::addData(const TResistance &sim)
{
    UFlowDuct::addData(sim);
}


void UResistance::setRelUserResistanceWithFixedVital(double relNonVitalRes)
{
    if ( mRelNonVitalRes != relNonVitalRes )
    {
        mRelNonVitalRes = relNonVitalRes;
        double Rvital = mRefR / getVitalPercentage();
        double Rnonvital_default = mRefR * Rvital / (Rvital  - mRefR);
        double Rnonvital = relNonVitalRes * Rnonvital_default;
        mR = (Rnonvital * Rvital)/(Rnonvital + Rvital);
    }
}

double UResistance::getRelUserResistanceWithFixedVital() const
{
    return mRelNonVitalRes;
}

void UResistance::setRefValues()
{
    mRefR = mR;
    mRelNonVitalRes = 1.0;
}
