/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: uresistance.h 776 2013-07-11 20:33:36Z eelco $
 *
 */

#ifndef _URESISTANCE_H_
#define _URESISTANCE_H_


#include <QMetaType>
#include "uflowduct.h"
#include "tresistance.h"
#include "ucircadaptsignal.h"
#include "PCircAdaptDefines.h"

using namespace Peacs::Support;

namespace Peacs
{
    namespace CircAdapt
    {
        class SResistance;
        /** @brief Class for microcirculation. Inherits from FlowDuct.
          */
        class UResistance : public UFlowDuct
        {

        public:

            /** @brief Default Constructor
              */
            UResistance(  int size = DEFAULT_SIGNAL_LENGTH);

            /** @brief Copy Constructor
              * Creates a new instance of TResistanceState and initialyzes its members
              * with the specified source
              * @param state the source to copy the data from
              */
            UResistance( const UResistance & state );

            /** @brief Destructor
              */
            ~UResistance( void ) {}


            /** @brief Asignment operator
              * @param state the source to copy the data from
              */
            const UResistance & operator= ( const UResistance & state );

            /** @brief copy the sepcification parameters from the simulation class to the transfer class
              * no specification parameters in this class
              */
            void copySpecifications(const TResistance &res);

            /** @brief At the start of a simulation run the state variables are resized
              * @param size the number of samples in this heart beat
              */
            void resetData( int size );

            /** @brief add a sample to the state variables
              * @param sim the simulate class
              * @param pos the sample number in the state variables
              */
            void addData( const TResistance & sim);


            /** @brief save the signals in the variables
              * @see UCircAdaptSignal*/
            inline void saveSignals() { UFlowDuct::saveSignals(); }

            virtual void setRefValues();


            inline double getResistance() const { return mR; }

            inline void setResistance(double res) { mR = res; }

            inline double getRefResistance() const { return mRefR; }

            inline double getRelUserResistance() const { return mR / mRefR; }

            void setRelUserResistance(double res) { mR = mRefR * res; }

            double getRelUserResistanceWithFixedVital() const;

            void setRelUserResistanceWithFixedVital(double res);

            inline double getVitalPercentage() const { return 0.25; }

            SResistance::resistance_type getResistanceType() const { return mResistanceType; }

        private:
            double mR;

            double mRefR;

            double mRelNonVitalRes;

            SResistance::resistance_type mResistanceType;

        };
    }
}

QDebug & operator<< ( QDebug & dbg, const Peacs::CircAdapt::UResistance & state );




#endif
