/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: usack.cpp 776 2013-07-11 20:33:36Z eelco $
 *
 */

#include "usack.h"
#include "tsack.h"
#include "PCircAdaptDefines.h"

using namespace Peacs::CircAdapt;

USack::USack(int size, const QColor &color):
    UCompliance(size,color),
    mVrefSack(0.0),
    mPSack( *this,  PRESSURE_UNITS,size),
    mVSack( *this,  VOLUME_UNITS,size)
{
}


// Constructor
USack::USack(const USack &sack) :
    UCompliance(sack),
    mVrefSack( sack.mVrefSack ),
    mPSack( sack.mPSack ),
    mVSack( sack.mVSack )
{

}

const USack& USack::operator =(const USack & sack)
{
    if ( this  != &sack )
    {
        UCompliance::operator =(sack);
        mVrefSack           = sack.mVrefSack;
        mPSack              = sack.mPSack;
        mVSack              = sack.mVSack;
    }
    return *this;
}



void USack::copySpecifications(const TSack &sack)
{
    UCompliance::copySpecifications(sack);
    mVrefSack = sack.getRefVolume();
}

void USack::resetData( int size )
{
    UCompliance::resetData(size);
    mPSack.reset(size);
    mVSack.reset(size);
}

void USack::addData(const TSack &sim)
{
    UCompliance::addData(sim);

    mPSack.addBeat(sim.getPSack());
    mVSack.addBeat(sim.getVSack());
}

void USack::saveSignals()
{
    UCompliance::saveSignals();

    mPSack.startSave();
    mVSack.startSave();
}
