/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: usack.h 776 2013-07-11 20:33:36Z eelco $
 *
 */

#ifndef __USACK_H_
#define __USACK_H_

#include "ucompliance.h"
#include "ucircadaptsignal.h"
#include "PCircAdaptDefines.h"

namespace Peacs
{

    namespace CircAdapt
    {

        class TSack;
        /**
        Class for pericardial sack. Inherits from Compliance.
        */
        class USack: public UCompliance {

        public:
            USack ( int size = DEFAULT_SIGNAL_LENGTH, const QColor &color =QColor(255,255,255));

            USack (const USack & sack);

            /// Destructor
            ~USack() { }

            virtual const USack& operator = (const USack &sack);

            /** @return the reference volume of the sack
              */
            inline double getRefVolume() const { return mVrefSack; }


            /** @@brief set the reference volume of the sack
              * @param vol the new volume
              */
            inline void setRefVolume(double vol) { mVrefSack = vol; }


            inline const UCircAdaptSignal &getPSack() const { return mPSack; }


            inline const UCircAdaptSignal &getVSack() const { return mVSack; }


            /** @brief copy the sepcification parameters from the simulation class to the transfer class
              * no specification parameters in this class
              */
            void copySpecifications(const TSack &);

            /** @brief At the start of a simulation run the state variables are resized
              * @param size the number of samples in this heart beat
              */
            void resetData( int size );

            /** @brief add a sample to the state variables
              * @param sim the simulate class
              * @param pos the sample number in the state variables
              */
            void addData( const TSack & sim);

            /** @brief save the signals in the variables
              * @see UCircAdaptSignal*/
            void saveSignals();


        private:
            // Specification parameters
            double mVrefSack;     // Reference volume of sack

            // Simulation state variables
            UCircAdaptSignal mPSack;        // Pressure of sack

            UCircAdaptSignal mVSack;        // Volume of sack

        };
    }
}

#endif
