/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: usarcomere.cpp 776 2013-07-11 20:33:36Z eelco $
 *
 */

#include <QDebug>

#include "tsarcomere.h"
#include "usarcomere.h"


using namespace Peacs::Support;
using namespace  Peacs::CircAdapt;

// Constructor
USarcomere::USarcomere(int size, const QColor &color):
    UCircElement(color),
    mActivationDelay(0.0),
    mLsRef(0.0),
    mLs0Pas(0.0),
    mdLsPas(0.0),
    mSfPas(0.0),
    mSfPasRef(mSfPas),
    mLsi0Act(0.0),
    mLenSeriesElement(0.0),
    mSfAct(0.0),
    mSfActRef(mSfAct),
    mvMax(0.0),
    mTimeAct(0.0),
    mTR(0.0),
    mTD(0.0),
    mCRest(0.0),
    mAdapt_LsBe(0.0),
    mAdapt_LsEe(0.0),
    mAdapt_SfPasMax(0.0),
    mFiberStrain( *this,  STRAIN_UNITS,size),
    mC( *this,  CONTRACTION_UNITS,size),
    mCDot( *this,  DCONTRACTION_UNITS,size),
    mLsi( *this,  LENGTH_UNITS,size),
    mLsiDot( *this,  DLENGTH_UNITS,size),
    mLs( *this,  LENGTH_UNITS,size),
    mSfPasT( *this,  STRESS_UNITS,size),
    mSf( *this,  STRESS_UNITS,size),
    mDSfDEf( *this,  STRAIN_UNITS,size)
{
}

USarcomere::USarcomere (const USarcomere &sarco):
    UCircElement(sarco.getName()),
    mActivationDelay    ( sarco.mActivationDelay ),
    mLsRef              ( sarco.mLsRef ),
    mLs0Pas             ( sarco.mLs0Pas ),
    mdLsPas             ( sarco.mdLsPas ),
    mSfPas              ( sarco.mSfPas ),
    mSfPasRef           ( mSfPas ),
    mLsi0Act            ( sarco.mLsi0Act ),
    mLenSeriesElement   ( sarco.mLenSeriesElement ),
    mSfAct              ( sarco.mSfAct ),
    mSfActRef           ( sarco.mSfActRef ),
    mvMax               ( sarco.mvMax ),
    mTimeAct            ( sarco.mTimeAct ),
    mTR                 ( sarco.mTR ),
    mTD                 ( sarco.mTD ),
    mCRest              ( sarco.mCRest ),
    mAdapt_LsBe         ( sarco.mAdapt_LsBe ),
    mAdapt_LsEe         ( sarco.mAdapt_LsEe ),
    mAdapt_SfPasMax     ( sarco.mAdapt_SfPasMax ),
    mFiberStrain        ( sarco.mFiberStrain ),
    mC                  ( sarco.mC ),
    mCDot               ( sarco.mCDot ),
    mLsi                ( sarco.mLsi ),
    mLsiDot             ( sarco.mLsiDot ),
    mLs                 ( sarco.mLs ),
    mSfPasT             ( sarco.mSfPasT ),
    mSf                 ( sarco.mSf ),
    mDSfDEf             ( sarco.mDSfDEf )
{

}

const USarcomere& USarcomere::operator = (const USarcomere &sarco)
{
    if ( this != &sarco)
    {
        UCircElement::operator =(sarco);

        mActivationDelay    = sarco.mActivationDelay;
        mLsRef              = sarco.mLsRef;
        mLs0Pas             = sarco.mLs0Pas;
        mdLsPas             = sarco.mdLsPas;
        mSfPas              = sarco.mSfPas;
        mSfPasRef           = mSfPas;
        mLsi0Act            = sarco.mLsi0Act;
        mLenSeriesElement   = sarco.mLenSeriesElement;
        mSfAct              = sarco.mSfAct;
        mSfActRef           = sarco.mSfActRef;
        mvMax               = sarco.mvMax;
        mTimeAct            = sarco.mTimeAct;
        mTR                 = sarco.mTR;
        mTD                 = sarco.mTD;
        mCRest              = sarco.mCRest;
        mAdapt_LsBe         = sarco.mAdapt_LsBe;
        mAdapt_LsEe         = sarco.mAdapt_LsEe;
        mAdapt_SfPasMax     = sarco.mAdapt_SfPasMax;
        mFiberStrain        = sarco.mFiberStrain;
        mC                  = sarco.mC;
        mCDot               = sarco.mCDot;
        mLsi                = sarco.mLsi;
        mLsiDot             = sarco.mLsiDot;
        mLs                 = sarco.mLs;
        mSfPasT             = sarco.mSfPasT;
        mSf                 = sarco.mSf;
        mDSfDEf             = sarco.mDSfDEf;
    }
    return *this;
}


void USarcomere::copySpecifications(const TSarcomere &sarco)
{
    mActivationDelay    = sarco.getActivationdelay();
    mLsRef              = sarco.getLsRef();
    mLs0Pas             = sarco.getLs0Pas();
    mdLsPas             = sarco.getdLsPas();
    mSfPas              = sarco.getSfPas();
    mLsi0Act            = sarco.getLsi0Act();
    mLenSeriesElement   = sarco.getLenSeriesElement();
    mSfAct              = sarco.getSfAct();
    mvMax               = sarco.getvMax();
    mTimeAct            = sarco.getTimeAct();
    mTR                 = sarco.getTR();
    mTD                 = sarco.getTD();
    mCRest              = sarco.getCRest();
    mAdapt_LsBe         = sarco.getAdapt_LsBe();
    mAdapt_LsEe         = sarco.getAdapt_LsEe();
    mAdapt_SfPasMax     = sarco.getAdapt_SfPasMax();
}


void USarcomere::resetData( int size )
{
    mFiberStrain.reset(size);
    mC.reset(size);
    mCDot.reset(size);
    mLsi.reset(size);
    mLsiDot.reset(size);
    mLs.reset(size);
    mSfPasT.reset(size);
    mSf.reset(size);
    mDSfDEf.reset(size);

}

void USarcomere::addData(const TSarcomere & sim)
{
    mFiberStrain.addBeat(sim.getFiberStrain());
    mC.addBeat(sim.getC());
    mCDot.addBeat(sim.getCDot());
    mLsi.addBeat(sim.getLsi());
    mLsiDot.addBeat(sim.getLsiDot());
    mLs.addBeat(sim.getLs());
    mSfPasT.addBeat(sim.getSfPasT());
    mSf.addBeat(sim.getSf());
    mDSfDEf.addBeat(sim.getDSfDEf());

}

void USarcomere::saveSignals()
{
    mFiberStrain.startSave();
    mC.startSave();
    mCDot.startSave();
    mLsi.startSave();
    mLsiDot.startSave();
    mLs.startSave();
    mSfPasT.startSave();
    mSf.startSave();
    mDSfDEf.startSave();

}

void USarcomere::setRefValues()
{
    mSfActRef           = mSfAct;
    mSfPasRef           = mSfPas;

}


