#include <QDebug>
#include "ttube.h"
#include "utube.h"
#include "PCircAdaptDefines.h"

using namespace Peacs::CircAdapt;

QDebug & operator<< ( QDebug & dbg, const UTube & state )
{
    dbg.nospace() << "UTube:";
//    dbg.nospace() << "mVolume : " << state.getVolume();
//    dbg.nospace() << "mPressure : " << state.getPressure();

    return dbg.maybeSpace();
}


// Constructor
UTube::UTube ( int size,const QColor &color) :
    UCompliance(size,color),
    mK(0),
    mQ0(0),
    mP0(0),
    mVesselLength(0),
    mPmax(0),
    mAWall(0),
    mA0(0),
    mWallStressAdapt(0),
    mVFlowMeanAdapt(0),
    mVImpactAdapt(0)
{

}


UTube::UTube( const UTube & state ) :
    UCompliance(state),
    mK(state.mK),
    mQ0(state.mQ0),
    mP0(state.mP0),
    mVesselLength(state.mVesselLength),
    mPmax(state.mPmax),
    mAWall(state.mAWall),
    mA0(state.mA0),
    mWallStressAdapt(state.mWallStressAdapt),
    mVFlowMeanAdapt(state.mVFlowMeanAdapt),
    mVImpactAdapt(state.mVImpactAdapt)
{

}


void UTube::copySpecifications(const TTube & tube)
{
    UCompliance::copySpecifications(tube);

    mK  = tube.stiffnessWall();
    mQ0 = tube.meanRestFlow();
    mP0 = tube.meanRestPressure();
    mVesselLength   = tube.meanVesselLength();
    mPmax   = tube.Pmax();
    mAWall  = tube.Awall();
    mA0 = tube.A0();
    mWallStressAdapt    =   tube.wallStressAdapt();
    mVFlowMeanAdapt = tube.vFlowMeanAdapt();
    mVImpactAdapt   = tube.vImpactAdapt();
}


const UTube & UTube::operator= ( const UTube & state )
{
    if ( this != & state )
    {
        UCompliance::operator =(state);
        mK                  = mK;
        mQ0                 = mQ0;
        mP0                 = mP0;
        mVesselLength       = mVesselLength;
        mPmax               = mPmax;
        mAWall              = mAWall;
        mA0                 = mA0;
        mWallStressAdapt    = state.mWallStressAdapt;
        mVFlowMeanAdapt     = state.mVFlowMeanAdapt;
        mVImpactAdapt       = state.mVImpactAdapt;
    }
    return *this;
}


void UTube::resetData( int size )
{
    UCompliance::resetData(size);

}

void UTube::addData( const TTube & sim)
{
    UCompliance::addData(sim);

}

void UTube::saveSignals()
{
    UCompliance::saveSignals();

}
