/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: uvalve.cpp 776 2013-07-11 20:33:36Z eelco $
 *
 */
#include "uvalve.h"
#include "tvalve.h"
#include "valve.h"
#include "PCircAdaptDefines.h"

using namespace Peacs::CircAdapt;


UValve::UValve(int size, const QColor &color):
    UFlowDuct(size,color),
    mLen(0.0),    
    mAOpenMax(DEFAULT_VALVE_MAX_AOPEN),
    mAOpenRef(DEFAULT_VALVE_MAX_AOPEN),
    mALeakMin(0.0),
//    mAOpen( DEFAULT_VALVE_MAX_AOPEN),
//    mALeak( DEFAULT_VALVE_MIN_ALEAK),
    mFlowVelocity(*this, FLOWVELOCITY_UNITS,size),
    mQDot( *this,   DFLOW_UNITS,size),
    mL( *this,      LENGTH_UNITS,size)
{

}


UValve::UValve(const UValve &valve):
    UFlowDuct(valve),
    mLen( valve.mLen ),
    mAOpenMax(valve.mAOpenMax),
    mAOpenRef(valve.mAOpenRef),
    mALeakMin(valve.mALeakMin),
//    mAOpen( valve.mAOpen ),
//    mALeak( valve.mALeak ),
    mFlowVelocity( valve.mFlowVelocity ),
    mQDot( valve.mQDot  ),
    mL( valve.mL )
{

}

const UValve& UValve::operator = (const UValve &valve)
{
    if ( this != &valve)
    {
        UFlowDuct::operator =(valve);
        mLen        = valve.mLen;
        mAOpenMax   = valve.mAOpenMax;
        mAOpenRef   = valve.mAOpenRef;
        mALeakMin   = valve.mALeakMin;
//        mAOpen      = valve.mAOpen;
//        mALeak      = valve.mALeak;
        mFlowVelocity = valve.mFlowVelocity;
        mQDot       = valve.mQDot;
        mL          = valve.mL;
    }
    return *this;
}


void UValve::copySpecifications(const TValve &valve)
{
    UFlowDuct::copySpecifications(valve);

    mLen        = valve.getLength();
//    mAOpen      = valve.getAreaOpen();
//    mALeak      = valve.getALeak();
    mAOpenMax   = valve.getAreaOpenMax();
    mALeakMin   = valve.getAreaLeakMin();
}

void UValve::resetData( int size )
{
    UFlowDuct::resetData(size);
    mFlowVelocity.reset(size);
    mQDot.reset(size);
    mL.reset(size);
}

void UValve::addData(const TValve & sim)
{
    UFlowDuct::addData(sim);
    PVector flowVelo( sim.getFlow() );
    // TODO does not work for the ducts
    if ( mAOpenMax <= DEFAULT_VALVE_MIN_ALEAK  )
    {
        flowVelo.toZero();
    }
    else if ( mALeakMin < DEFAULT_VALVE_MIN_ALEAK * 2.0 )
    {
        flowVelo /= mAOpenMax;
    }
    else
    {
        for (int i=0; i < flowVelo.N(); i++)
        {
            if ( flowVelo[i] > 0 )
            {
                flowVelo[i] /= mAOpenMax;
            }
            else
            {
                flowVelo[i] /= mALeakMin;
            }
        }
    }
    mFlowVelocity.addBeat(flowVelo);
    mQDot.addBeat(sim.getQdot());
    mL.addBeat(sim.getLwithInertia());
}

void UValve::saveSignals()
{
    UFlowDuct::saveSignals();
    mFlowVelocity.startSave();
    mQDot.startSave();
    mL.startSave();
}


void UValve::setRefValues()
{
    UFlowDuct::setRefValues();
    mAOpenRef   = mAOpenMax;
}
