/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: uvalve.h 923 2013-11-11 14:03:15Z eelco $
 *
 */

#ifndef __UVALVE_H_
#define __UVALVE_H_

#include "uflowduct.h"
#include "ucircadaptsignal.h"

#include "PCircAdaptDefines.h"


namespace Peacs
{
    namespace CircAdapt
    {

        class TValve;
        /**
        Class for valve. Inherits from FlowDuct.
        */
        class UValve: public UFlowDuct
        {

        public:

            /* @brief Default Constructor */
            UValve(int size = DEFAULT_SIGNAL_LENGTH,const QColor & color = QColor(255,255,255));

            UValve (const UValve & valve);

            virtual ~UValve() {;}

            const UValve& operator = (const UValve &valve);


            /** @brief copy the sepcification parameters from the simulation class to the transfer class
              * no specification parameters in this class
              */
            virtual void copySpecifications(const TValve &valve);

            /** @brief At the start of a simulation run the state variables are resized
              * @param size the number of samples in this heart beat
              */
            virtual void resetData( int size );

            /** @brief add a sample to the state variables
              * @param sim the simulate class
              * @param pos the sample number in the state variables
              */
            void addData(const TValve & sim);

            /** @brief save the signals in the variables
              * @see UCircAdaptSignal*/
            void saveSignals();

            virtual void setRefValues();

            inline double getLength() const { return mLen; }

            inline double getAreaOpenMax() const {return mAOpenMax; }

            inline double getAreaRef() const {return mAOpenRef; }

            inline void setAreaOpenMax(double val) { mAOpenMax = val; }

            inline void setStenosis(double val) { mAOpenMax = qBound(0.0,(1.0 - val),1.0) * mAOpenRef; }//mAOpen = mAOpenMax; }

            inline double getStenosis( void ) const { return qBound( 0.0, 1.0 - mAOpenMax / mAOpenRef, 1.0 ); }

            inline double getAreaLeakMin() const { return mALeakMin; }

//            inline double getAreaOpen() const  { return mAOpen; }     // Cross-section of open valve

//            inline double getALeak() const     { return mALeak; }      // Cross-section of closed valve

            inline void setAreaLeakMin(double leak) { mALeakMin = leak; }


            // Simulation state variables

            UCircAdaptSignal &flowVelocity() { return mFlowVelocity;  }

            const UCircAdaptSignal & getQdot() const              { return mQDot;  }       // d/dt Flow

            const UCircAdaptSignal &getLwithInertia() const       { return mL;     }          // Channel length with inertia

        private:
            // Specification parameters
            double mLen;        // Length DEBUG: What is Len??

            double mAOpenMax;

            double mAOpenRef;

            double mALeakMin;

//            double mAOpen;      // Cross-section of open valve

//            double mALeak;      // Cross-section of closed valve

            // Simulation state variables
            UCircAdaptSignal mFlowVelocity;

            UCircAdaptSignal mQDot;       // d/dt Flow

            UCircAdaptSignal mL;          // Channel length with inertia

        };
    }
}

#endif
