/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: uventricle.cpp 784 2013-07-16 17:35:19Z peter $
 *
 */

#include <QDebug>
#include "tventricle.h"
#include "uventricle.h"



using namespace Peacs::CircAdapt;

// Constructor
UVentricle::UVentricle(int size, const QColor &color) :
    UCircElement(color),
    mLeftV(size,LEFTVENTRICLE_COLOR),
    mSeptV(size,VENTRICLE_COLOR),
    mRigtV(size,RIGHT_VENTRICLE),
    mVentrType(BISEG)
{

}

UVentricle::UVentricle( const UVentricle & ventricle ) :
    UCircElement( ventricle),
    mLeftV( ventricle.mLeftV ),
    mSeptV( ventricle.mSeptV ),
    mRigtV( ventricle.mRigtV),
    mVentrType( ventricle.mVentrType )
{

}


const UVentricle& UVentricle::operator = (const UVentricle &ventricle)
{
    if ( this != &ventricle)
    {
        UCircElement::operator =(ventricle);
        mLeftV = ventricle.mLeftV;
        mSeptV = ventricle.mSeptV;
        mRigtV = ventricle.mRigtV;
        mVentrType = ventricle.mVentrType;
    }
    return *this;
}


void UVentricle::copySpecifications(const TVentricle &ventr)
{
    setName( ventr.getName() );
    mLeftV.copySpecifications( ventr.getLeftVentricle() );
    mRigtV.copySpecifications( ventr.getRightVentricle() );
    mSeptV.copySpecifications( ventr.getSeptum() );
    mVentrType = ventr.getType();

}



void UVentricle::resetData( int size )
{
    mLeftV.resetData(size);
    mRigtV.resetData(size);
    mSeptV.resetData(size);
}


void UVentricle::addData( const TVentricle & sim)
{
    mLeftV.addData(sim.getLeftVentricle());
    mRigtV.addData(sim.getRightVentricle());
    mSeptV.addData(sim.getSeptum());
}


void UVentricle::setRelVentriclesSfAct(double percentage)
{
    mLeftV.setRelSfAct(percentage);
    mRigtV.setRelSfAct(percentage);
    mSeptV.setRelSfAct(percentage);
}

void UVentricle::setRelVentriclesSfPas(double percentage)
{
    mLeftV.setRelSfPas(percentage);
    mRigtV.setRelSfPas(percentage);
    mSeptV.setRelSfPas(percentage);
}




void UVentricle::setActivationDelay(double delay)
{
    mLeftV.setActivationDelay(delay);
    mRigtV.setActivationDelay(delay);
    mSeptV.setActivationDelay(delay);
}

double UVentricle::getActivationDelay() const
{
    return mLeftV.getActivationDelay();
}

