/*
 *  This file is part of the CircAdapt package (see http:/www.circadapt.org)
 *
 *  Copyright (C) 2012 Peacs (www.peacs.nl)
 *
 *  CircAdapt is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  CircAdapt is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CircAdapt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    Project:              CircAdapt
 *    Compiler:             MinGW, gcc
 *    Tatrget platforms:    WIndows, Mac OS-X
 *
 *
 *    Subversion info:
 *
 *    $Id: uventricle.h 784 2013-07-16 17:35:19Z peter $
 *
 */

#ifndef __UVENTRICLE_H__
#define __UVENTRICLE_H__

#include "uchamber.h"
#include "PCircAdaptDefines.h"

using namespace Peacs::Support;
namespace Peacs
{

    namespace CircAdapt
    {
        class TVentricle;

        /**
        Class for muscular chamber. Inherits from UCompliance.
        */
        class UVentricle: public UCircElement
        {

        public:
            UVentricle(int size = DEFAULT_SIGNAL_LENGTH, const QColor & color = QColor(255,255,255));

            UVentricle(const UVentricle & ventricle);

            virtual ~UVentricle() {;}


            const UVentricle& operator = (const UVentricle &ventr);

            /** @brief copy the sepcification parameters from the simulation class to the transfer class
              * no specification parameters in this class
              */
            void copySpecifications(const TVentricle &);

            /** @brief At the start of a simulation run the state variables are resized
              * @param size the number of samples in this heart beat
              */
            void resetData( int size );

            /** @brief add a sample to the state variables
              * @param sim the simulate class
              * @param pos the sample number in the state variables
              */
            void addData( const TVentricle & sim);


            /** @brief TODO
              */
            inline void saveSignals() {
                mRigtV.saveSignals();
                mLeftV.saveSignals();
            }

            /** @brief TODO
              */
            inline void setRefValues( void ) {
                mLeftV.setRefValues();
                mRigtV.setRefValues();
                mSeptV.setRefValues();
            }


            /** @brief TODO
              */
            inline UChamber & rightVentricle() { return mRigtV; }

            /** @brief TODO
              */
            inline UChamber & leftVentricle() { return mLeftV; }

            /** @brief TODO
              */
            inline UChamber & septum() { return mSeptV; }

            /** @brief TODO
              */
            inline const UChamber & getRightVentricle() const { return mRigtV; }

            /** @brief TODO
              */
            inline const UChamber & getLeftVentricle()const  { return mLeftV; }

            /** @brief TODO
              */
            inline const UChamber & getSeptum() const { return mSeptV; }

            /** @brief TODO
              */
            inline ventricle_type getType() const { return mVentrType; }

            /** @brief TODO
              */
            void setRelVentriclesSfAct(double percentage);

            /** @brief TODO
              */
            void setRelVentriclesSfPas(double percentage);



            void setActivationDelay(double delay);

            double getActivationDelay() const;


            /** @brief TODO
              */
            inline double getRelSfAct() const { return mLeftV.getRelSfAct(); }

            inline double getRelSfPas() const { return mLeftV.getRelSfPas(); }

//            double getVolume() const;

        private:
            UChamber mLeftV;

            UChamber mSeptV;

            UChamber mRigtV;

            ventricle_type mVentrType;

        };
    }
}

#endif
